 /*=============================================================================

  Name: bpm_reqreply_struc.h

  Abs:  Structure definitions for request and reply messages between VMS    
        and slc BPM threads (and micros).  

  Auth: 23-JUN-2005, Debbie Rogind (drogind):
  Rev:  25-OCT-2006, Debbie Rogind
        Add rinqabo_precinj in bpm_alldata_replybeg_ts per VAX 
		bpm_reqreply_struct.hc file

-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  (newest to oldest)  
  23-apr-2007 dmf added new elements nmicountv and pulseidv to 
                  bpm_alldata_replybeg_ts to keep up=to-date with SLC version
                  of this file
  25-OCT-2006, Debbie Rogind
        Add rinqabo_precinj in bpm_alldata_replybeg_ts per VAX 
		bpm_reqreply_struct.hc file

=============================================================================*/
 /*===========================================================================*/
 /*                                                            */
 /*                                                                           */
 /* Abs:  Structure definitions for request and reply messages between Vax    */
 /*       and micro BPM subjob.                                               */
 /*                                                                           */
 /* Name: slctxt:bpm_reqreply_struc.hc                                        */
 /*                                                                           */
 /* Prev: slctxt:slc_macros.hc             vmsstat_t, int2u,...               */
 /*       slctxt:msghead.hc                msgheader_ts.                      */
 /*       slctxt:bpmparam.hc               BPM_RING_MAXCAL, BPM_MAXPP, MBPMP, */
 /*                                        MBPMS, MAXTOR, MGADCH, MDUGUNITS,  */
 /*                                        MDUGPRIMS, MAXNMUX, MAX_SCAN_STEPS */
 /*                                        MSCANCORMIC, BPM_ULIST_PRIMS,      */
 /*                                        BPM_ULIST_MAX, MAXPTTP,            */
 /*                                        MAX_FIFO_CHAN, MAX_WIRE_CHAN,      */
 /*                                        TOR_CAL_MAX_READS,                 */
 /*                                        BPM_CONNSUSP_MAXIDS.               */
 /*       slctxt:rgbm_defines.hc           (Optional) MAX_MODFR_MASK.         */
 /* Auth: 1987, Tony Gromme (TEG).                                            */
 /* Revw:                                                                     */
 /*                                                                           */
 /*---------------------------------------------------------------------------*/ 
 
#ifndef BPM_REQREPLY_STRUC_H
 
 
 typedef struct
 {   unsigned short nunits[BPM_ULIST_PRIMS],
                    ulist[BPM_ULIST_MAX];
 } bpm_measulist_ts;
 
#define BPMULIST_CALBX_SHFC 14        /* Calibration index is in hi-order 2 bits */
                                      /*  of each word in ulist (except in       */
                                      /*  bpm_measringcre_req_ts).               */
#define BPMULIST_UNIT_MASK ((1 << BPMULIST_CALBX_SHFC) - 1)
 
#if defined(MAX_MODFR_MASK)
 
 /**** BPM buffer ring creation/deletion:                        */
 /* Structure of message (func code BPMO_BUFRING_CRE) requesting */
 /* shareable BPM hardware readout buffer ring to be built.      */
 
 typedef struct
 {   unsigned short   i_ring,
                      nbufs,
                      nicals,
                      i_cal[BPM_RING_MAXCAL],
                      npp;
     unsigned char    pp[BPM_MAXPP];
     unsigned short   flip;
     unsigned long    exclmask[MAX_MODFR_MASK],
                      inclmask_strt[MAX_MODFR_MASK],
                      inclmask_nxt[MAX_MODFR_MASK],
                      nopp_mindel;
     float            bunchdel[BPM_RING_MAXCAL][2];   /* nanoseconds */
     unsigned long    spare3[4];
     unsigned short   rinq_dsp_mode,
                      rinq_dlymax,
                      rinq_nthturn,
                      rinq_nturns,
                      rinq_bucket[BPM_RING_MAXCAL];
     unsigned long    start_turns_skip,
                      morflgs,
                      spare5[6];
     bpm_measulist_ts ulist;
     unsigned short   ucalx[BPM_ULIST_MAX];
 } bpm_measringcre_req_ts;
 
 typedef struct
 {   msgheader_ts           hdr;
     bpm_measringcre_req_ts dat;
 } bpm_measringcre_reqh_ts;
#endif
 
 /* Structure of message (func code BPMO_BUFRING_DEL) requesting       */
 /* existing shareable BPM hardware readout buffer ring to be deleted. */
 
 typedef struct
 {   msgheader_ts   hdr;
     unsigned short i_ring;
 } bpm_measringdel_reqh_ts;
 
 
 
 /* Structure of message to download parameters for setting up PEP2          */
 /* beam abort monitoring by RINQ modules (function code BPMO_RINQABO_INIT). */
 /* Note:  subscript values BPM_RINQ_HERX and BPM_RINQ_LERX for ring_as are  */
 /* defined in BPMPARAM.HC.                                                  */
 
 typedef struct
 {   unsigned short  spare2[2],
                     i_cal,
                     spare3[3],
                     rinq_nthturn,
                     rinq_bucket;
     float           bunchdel;
     unsigned long   spare[15];
 } bpm_rinqabo_init_one_ts;
 
 typedef struct
 {   unsigned long           spare1;
     bpm_rinqabo_init_one_ts ring_as[2];
     unsigned long           spare9[9];
 } bpm_rinqabo_init_req_ts;
 
 typedef struct
 {   msgheader_ts            hdr;
     bpm_rinqabo_init_req_ts dat;
 } bpm_rinqabo_init_reqh_ts;
 
 
 
 /* Structure of beginning of request message that may be larger than */
 /* message service can handle.                                       */
 
 typedef struct
 {   unsigned long  reqmsgtotwc,
                    reqmsgpartwc;
 } bpm_largreq_hdrr_ts;
 
 typedef struct
 {   msgheader_ts        hdr;
     bpm_largreq_hdrr_ts hdrr;
 } bpm_largreq_hdrrh_ts;
 
 
 /**** BPM beam position measurement:                                          */
 /* Structures of messages (func codes BPMO_MEAS_PREP, BPMO_NOGATE_PREP, or    */
 /* BPMO_SCAN_PREP) requesting BPM measurement (simple or scan) to be set up.  */
 /* These two messages (bpm_measprep_reqh_ts and bpm_scanprep_reqh_ts) are     */
 /* also used with function codes BPMO_RINQABO_MEAS and BPMO_RINQABO_SCAN      */
 /* respectively, in which case the micro sends the data reply immediately,    */
 /* without any bpm_allprep_replyh_ts and without expecting any communication  */
 /* with the MPG.                                                              */
 /* Note:  This request message can be larger than message service can handle: */
 /* reqmsgtotwc should give the request message total size in words not        */
 /* counting the message service header, and reqmsgpartwc should give the      */
 /* cumulative wordcount of what's been sent so far.  Pieces after the first   */
 /* should have those two longwords again (but this time not counted by        */
 /* themselves though still counting against the max per piece) immediately    */
 /* followed by the next fragment of the request message.  Micro will          */
 /* assemble the pieces and do the prep when assembly is complete.  Each       */
 /* piece before the last will get a longword iss reply (if no error)          */
 /* containing BPMO_PART_OK (informational).                                   */
 
#if defined(MAX_MODFR_MASK)
 
 typedef struct
 {   bpm_largreq_hdrr_ts  hdrr;
     unsigned short       i_seg,                 /* measdef id */
                          nicals,                /* num of cals needed by data reduction */
                          i_cal[BPM_RING_MAXCAL],/* ids of calibrations */
                          npp;                   /* num of base beamcodes for this acq */
     unsigned char        pp[BPM_MAXPP],         /* base beamcodes */
                          long_pulse_bpm_phas_type, 
                          bunch[2],
                          eminus,
                          tryy_flag,
                          navg,                  /* num of meas (beam pulses) to be avged */
                          nmuxyy,                /*     to obtain eac resulting value */
                          yy,                    /* value MPG broadcasts over PNET to enable */
                          yyseq_type,            /*     beg of net complete instance of this acq */
                          spareb[3];         
     unsigned long        measbits;              /* which prims to consider - BPMPARAM.HC */
     unsigned short       umin,
                          umax,
                          umin2,
                          umax2,
                          flip,
                          spare1,
                          long_pulse_bpm_phas[2];
     float                bunchdel[BPM_RING_MAXCAL][2];   /* nanoseconds */
     unsigned long        nopp_mindel,
                          vetomask;
     float                beamstrength[2];            /* E+, E- */
     unsigned long        exclmask[MAX_MODFR_MASK],
                          inclmask_strt[MAX_MODFR_MASK],
                          inclmask_nxt[MAX_MODFR_MASK];
     unsigned short       spare2,
                          drdel,
                          nfwd_sec_max,
                          tryy_unit,
                          nturns,              /* Number of PEP2 ring turns to average.     */
                          nthturn,             /* Measure every Nth PEP2 ring turn.         */
                          rinq_dsp_mode,       /* Mode of protocol between micro and DSP's. */
                          rinq_dlymax;         /* Max delay for DSP processing & readout.   */
     unsigned long        start_turns_skip[2]; /* # of PEP2 ring turns to skip at start.    */
     unsigned short       rinq_bucket[BPM_RING_MAXCAL]; /* PEP2 bucket number.              */
     unsigned long        rinq_sine_freq,      /* PEP2 freq for sine fit.                   */
                          morflgs;
     unsigned short       spare4[8];
     bpm_measulist_ts     ulist;
 } bpm_measprep_req_ts;        /* This is what non-scan measprep looks like. */

                    /* 0x00020000 through 0x04000000 available */
            /* If the measdef refers to multiple calibrations, then           */
            /* references to calibrations other than the first can be         */
            /* meaningful only for units specified in explicit ulist.  In     */
            /* flip, only the least significant bit (FLIP_BPMS_XCOOR) and the */
            /* XYZDRDEL bit and the TIMING_FIDUCIAL bit are significant in    */
            /* the measprep message;  other bits are taken from the flip word */
            /* in the calbprep message, except in the case of timing diag,    */
            /* which supplies all bits in the measprep message.               */
 
            /* For long pulse (A-line) BPM's, long_pulse_bpm_phas_type can    */
            /* have the values  X_phase_modulator, Y_phase_modulator,         */
            /* TMIT_phase_modulator, or Calibrator_phase_modulator as         */
            /* defined in slctxt:bpmparam.hc.                                 */
            /* For long pulse (A-line) BPM's, long_pulse_bpm_phas[0] is       */
            /* the In-phase and long_pulse_bpm_phas[1] is the Quadrature      */
            /* as defined:                                                    */
            /*                                                                */
            /*   In_phase = In_phase_scale * cos(calibration_phase_angle)     */
            /*            + In_phase_offset                                   */
            /*                                                                */
            /*   Quadrature = Quadrature_scale * sin(calibration_phase_angle) */
            /*              + Quadrature_offset                               */
            /*                                                                */
            /*   In_phase_offset, Quadrature_offset, In_phase_scale, and      */
            /*   Quadrature_scale are defined in slctxt:bpmparam.hc.          */
 
 typedef struct
 {   unsigned short wun,
                    nrpos,
                    wmovf,
                    wire_bitid;
     unsigned long  wire_type;
     unsigned short sped,
                    posdel[4],
                    fbvecflag,
                    wposrb_scaler_unit,
                    wposrb_scaler_sa,
                    spare[8];
     float          bunchdel_incr;   /* nanoseconds */
     unsigned long  sbdctlw,
                    sbdbitns,
                    wmovc[4],
                    magctlw[MSCANCORMIC],
                    rbkctlw[MSCANCORMIC];
     unsigned short magdac[MAX_SCAN_STEPS_P1][MSCANCORMIC];
 } bpm_scanprep_exten_ts;
 
 typedef struct
 {   bpm_measprep_req_ts   datm;
     bpm_scanprep_exten_ts dats;
 } bpm_scanprep_ts;            /* This is what scan measprep looks like. */
 
 typedef struct
 {   msgheader_ts        hdr;
     bpm_measprep_req_ts datm;
 } bpm_measprep_reqh_ts;
 
 typedef struct
 {   msgheader_ts          hdr;
     bpm_measprep_req_ts   datm;
     bpm_scanprep_exten_ts dats;
 } bpm_scanprep_reqh_ts;
#endif
 
                               /* Values for tryy_flag:                          */
#define TRYY_FLGM_NLCTA_TORO 1   /* Fire TRYY on and only on first mux step of   */
                                 /*  each mux cycle.                             */
#define TRYY_FLGM_ORSAY_BSM  3   /* Fire TRYY on and only on an extra discarded  */
                                 /*  beam pulse at beginning of first scan step. */
#define TRYY_FLGM_EACHPULSE  5   /* Fire TRYY on every beam pulse.               */
 
 
 /* Structure of immediate success/fail reply message to any of BPM */
 /* measprep, scanprep, calbprep, or calbtorprep request.           */
 
 typedef struct
 {   vmsstat_t      iss;
     unsigned char  nmuxmin,
                    spareb[3];
     unsigned long  datrply_totwc,
                    sparel;
     unsigned short npulstep,
                    sparew;
 } bpm_allprep_reply_ts;
 
 typedef struct
 {   msgheader_ts         hdr;
     bpm_allprep_reply_ts dat;
 } bpm_allprep_replyh_ts;      /* If you change bpm_allprep_reply_ts you must */
                               /* also change STRUCTURE /NONDATAREPLY_STRUC/  */
                               /* in REF_BPMUTIL:BPMREAD.FOR and              */
                               /* STRUCTURE /CALBPREPREPLY_STRUC/ in          */
                               /* REF_BPMSHR:TOR_CAL.FOR.                     */
 
 
 /* Structure of beginning of BPM measurement (non-scan or scan) data replies */
 /* and calibration data replies.  Beginning of non-self-identified data or   */
 /* first prefix of self-identified data immediately follows last item in     */
 /* bpm_alldata_replybeg_ts.
                                                  */
  typedef struct
 {   vmsstat_t      iss;
     union
     {   unsigned long  measbits,
                        calbbits;
     } bt;
     unsigned long  datrply_totwc,
                    datrply_partwc;
     unsigned short ndatblks,
                    nrpos;
     unsigned long  nmicountv,
	                pulseidv,
                    rinqabo_precinj[RNQABO_MXINJTSTMP],
                    spare[16-RNQABO_MXINJTSTMP],
                    checksum;
 } bpm_alldata_replybeg_ts;    /* In non-scan meas and non-self-identified     */
                               /* calb, ndatblks and nrpos are ignored.  In    */
                               /* self-identified calb, nrpos is ignored.      */
                               /* Checksum is such that if entire message      */
                               /* excluding message service header is summed   */
                               /* longwordwise (temporarily appending zeroes   */
                               /* to the end of the message to make its total  */
                               /* length be 0 mod 4 bytes, and using the final */
                               /* value of datrply_partwc), then the 32-bit    */
                               /* sum is -1, assuming checksum itself begins   */
                               /* on a longword boundary relative to the       */
                               /* beginning of bpm_alldata_replybeg_ts. Note     */
                               /* that datrply_totwc does not include msgheader_ts.      */
                               /* Only for retrieved beamabort data, rinqabo_precinj     */
                               /* contains at most RNQABO_MXINJTSTMP timestamps of most  */
                               /* recent injection cycles before beamabort (most recent  */
                               /* first);  and nmicountv contains timestamp of beamabort */
                               /* itself (these timestamps are 360ths of a second since  */
                               /* micro bootup);  and pulseidv contains pulse id of      */
                               /* beamabort itself.                                      */

 typedef struct
 {   msgheader_ts            hdr;
     bpm_alldata_replybeg_ts dat;
 } bpm_alldata_replybegh_ts;   /* If you change bpm_alldata_replybeg_ts you must */
                               /* also change /CALBTORREPLY_START_STRUC/ in      */
                               /* REF_SLCTXT:TORCALB_STRUC.TXT, and              */
                               /* STRUCTURE /CALBREPLY_STRUC/ in                 */
                               /* REF_BPMUTIL:BPMREAD.FOR.                       */
 
 
 /* Prefix used only for self-identified data (meas scan data reply or calb */
 /* self-identified data reply).  Note that wc = wordcount of specified     */
 /* unit's data, not including scanblk_prefix_ts itself.                    */
 
 typedef struct
 {   unsigned long  prim;
     unsigned short unit,
                    wc;
 } scanblk_prefix_ts;
 
                               /* For structures of individual data items */
                               /* in BPM measurement/scan data responses, */
                               /* see BPMSTRUC.HC.                        */
 
 
#if defined(MAX_MODFR_MASK)
 
 /**** BPM/GADC/TORO calibration:                              */
 /* Structure of message (func code BPMO_CALB_PREP) requesting */
 /* BPM+GADC calibration to be set up.                         */
 
 typedef struct
 {   unsigned short i_seg,
                    i_cal,
                    npp;
     unsigned char  pp[BPM_MAXPP],
                    spare1,
                    bunch,
                    eminus,
                    spare2,
                    navg,
                    nmuxyy,
                    yy,
                    yyseq_type;
     unsigned long  calbbits;
     unsigned short umin,
                    umax,
                    umin2,
                    umax2,
                    flip,
                    rinq_dsp_mode,
                    rinq_dlymax,
                    spare3;
     unsigned long  start_turns_skip;
     unsigned short spare4[2],
                    rinq_bucket,
                    ical_ref,
                    spare5[12];
     float          bunchdel[2];             /* nanoseconds */
     unsigned long  exclmask[MAX_MODFR_MASK],
                    inclmask[MAX_MODFR_MASK];
     float          beamstrength[2];         /* E+, E- */
 } bpm_calbprep_req_ts;        /* This is what BPMP/GADC calbprep looks like. */
 
#define FLIP_BPMD_2BNCH 0x0004        /* Same as BPMD_HSTA_2BNCH.             */
#define FLIP_BPMD_RVSGN 0x0008        /* If 2BNCH, then the two bunches have  */
                                      /*  opposite sign.                      */
 
                   /* From what I (Mike Zelazny on 15-Aug-1995) can tell,     */
                   /* the following FLIP bits are used:                       */
                   /*                                                         */
                   /* 0001 same as DGRP XYZ DGRP_FLIP_X_ELEC bit for E-, or   */
                   /*              DGRP XYZ DGRP_FLIP_X_POSI bit for E+.      */
                   /* 0002                                                    */
                   /* 0004 same as BPMD_HSTA_2BNCH bit.                       */
                   /* 0008 If 2BNCH, the two bunches have opposite sign.      */
                   /* 0010                                                    */
                   /* 0020                                                    */
                   /* 0040 same as DGRP XYZ DGRP_OUT_BEAM bit.                */
                   /* 0080 same as DGRP XYZ DGRP_EBOTH bit.                   */
                   /* 0100                                                    */
                   /* 0200                                                    */
                   /* 0400 same as DGRP XYZ DGRP_TOROP bit.                   */
                   /* 0800 same as DGRP XYZ DGRP_DRDEL bit.                   */
                   /* 1000 BUNCH_TRAIN bit defined in SLCTXT:BPMPARAM.HC.     */
                   /* 2000 TIMING_FIDUCIAL bit defined in SLCTXT:BPMPARAM.HC. */
                   /* 4000                                                    */
                   /* 8000                                                    */
 
 typedef struct
 {   msgheader_ts        hdr;
     bpm_calbprep_req_ts dat;
 } bpm_calbprep_reqh_ts;
 
                               /* Structure of extension of calbprep message */
                               /* (func code BPMO_CALTOR_PREP) requesting    */
 typedef struct                /* TORO calibration to be set up.             */
 {   unsigned short ndiscard,
                    spare[9];
     float          tort_ch11_adj;   /* nanoseconds */
     unsigned short npulvals,
                    pulvals[TOR_CAL_MAX_READS],
                    nunits,
                    ulist[MAXTOR];
 } bpm_calbtorprep_exten_ts;
 
 typedef struct
 {   bpm_calbprep_req_ts      datc;
     bpm_calbtorprep_exten_ts datt;
 } bpm_calbtorprep_req_ts;     /* This is what TORO calbprep looks like. */
 
 typedef struct
 {   msgheader_ts             hdr;
     bpm_calbprep_req_ts      datc;
     bpm_calbtorprep_exten_ts datt;
 } bpm_calbtorprep_reqh_ts;
#endif                         /* Structure of success/fail reply message */
                               /* to BPM calbprep or calbtorprep request  */
                               /* is bpm_allprep_replyh_ts above.         */
 
                               /* For structures of data items in BPMP and */
                               /* GADC calibration data responses, see     */
 typedef struct                /* BPMCALSTRUC.HC (but TORO is right here). */
 {   unsigned short stat,
                    qraw;
     float          tmit,
                    tmit_var;
 } toro_calbreply_ts;
 
 
 /**** BPM download calibration data:                       */
 /* Structure of message (func code BPMO_CAL_RSTR) carrying */
 /* BPMP/GADC calibration data to be downloaded.            */
 
 typedef struct
 {   bpm_largreq_hdrr_ts hdrr;
     unsigned short      spare1,
                         i_cal;
     unsigned char       pp,
                         bunch,
                         eminus,
                         spare2,
                         restflg,
                         spare3;
     unsigned short      spare4;
     unsigned long       calbbits;
     unsigned short      spare5[3],
                         flip;
     unsigned long       spare6[4];
     float               beamstrength[2];     /* E+, E- */
     unsigned long       spare7[6];
 } bpm_calbrstr_reqbeg_ts;
 
 typedef struct
 {   msgheader_ts           hdr;
     bpm_calbrstr_reqbeg_ts dat;
 } bpm_calbrstr_reqbegh_ts;
 
                               /* Data for 1st unit being downloaded begins */
                               /* immediately after bpm_calbrstr_reqbeg_ts. */
                               /* For structures of individual BPMP/GADC    */
                               /* unit calibration download items, see      */
                               /* BPMCALSTRUC.HC.                           */
 
 
 /**** BPM miscellaneous:                               */
 /* Structure of message (func code BPMO_ACCEPT_PUBLIC) */
 /* requesting an existing BPM calibration to be        */
 /* renamed from private to public.                     */
 
 typedef struct
 {   unsigned short private_id,
                    public_id;
 } bpm_calb_acc_pub_req_ts;
 
 typedef struct
 {   msgheader_ts            hdr;
     bpm_calb_acc_pub_req_ts dat;
 } bpm_calb_acc_pub_reqh_ts;
 
 
 /* Structure of message (request & reply for func code BPMO_TORO_CIRC_GET)    */
 /* retrieving circular buffer of TORO readout data.                           */
 /* See also BPMTORO_CIRCMAX in BPMPARAM.HC.  See also bpm_torocircbuf_desc_ts */
 /* in BPMUTIL_PROTO.H (for callers of Bpm_get_torocirc (non-micro)).          */
 
 typedef struct
 {   unsigned short spare,
                    abbrev;
 } bpm_toro_cirget_req_ts;
 
 typedef struct
 {   msgheader_ts           hdr;
     bpm_toro_cirget_req_ts dat;
 } bpm_toro_cirget_reqh_ts;
 
 typedef struct
 {   unsigned short unit,
                    camst[2],
                    tmit_u2;
     float          tmit_r4;
     unsigned short drtc,
                    bukt;
     unsigned long  modf32,
                    nmicount,
                    gate_timval;
 } bpm_toro_circ_item_ts;
 
 typedef struct
 {   vmsstat_t             iss;
     unsigned short        nitem,
                           wrapped;
     unsigned long         nmicount,
                           pulseid,
                           spare[6];
     bpm_toro_circ_item_ts itms[1];
 } bpm_toro_cirget_rpy_ts;
 
 typedef struct
 {   msgheader_ts           hdr;
     bpm_toro_cirget_rpy_ts dat;
 } bpm_toro_cirget_rpyh_ts;
 
 
 /* Structure of message (func code BPMO_MEAS_STOP)   */
 /* requesting an existing BPM measurement definition */
 /* (previous measprep/scanprep) to be deleted.       */
 
 typedef struct
 {   unsigned short i_seg;
 } bpm_meas_stop_req_ts;
 
 typedef struct
 {   msgheader_ts         hdr;
     bpm_meas_stop_req_ts dat;
 } bpm_meas_stop_reqh_ts;
 
 
 /* Structure of message (func code BPMO_CAN_DEF) */
 /* requesting an existing BPM calibration        */
 /* (public or private) to be deleted.            */
 
 typedef struct
 {   unsigned short i_cal;
 } bpm_calb_delete_req_ts;
 
 typedef struct
 {   msgheader_ts           hdr;
     bpm_calb_delete_req_ts dat;
 } bpm_calb_delete_reqh_ts;
 
 
 /* Structure of message (func code BPMO_CLR_SCP) */
 /* requesting all existing BPM measurement       */
 /* definitions and private calibrations          */
 /* belonging to requesting SCP to be deleted.    */
 
 typedef struct
 {   msgheader_ts   hdr;
 } bpm_clrscp_reqh_ts;
 
 
                               /* For structure of message (func codes BPMO_BPMPINIT */
                               /* and BPMO_PDU_REINIT) from timing subjob in same    */
                               /* micro requesting reinitialization of BPM hardware  */
                               /* in specified crate, see tcrate_reinit_reqh_ts in   */
                               /* TIME_REQREPLY_STRUC.HC.                            */
 
 
#define BPM_SHOBRNG_MAXBC 32256      /* Max bytecount of reply message     */
                                     /* (including message service header) */
                                     /* to BPMO_SHO_RBUFS request.         */
 
 /* Structures of components of reply message to request having func code  */
 /* BPMO_SHO_RBUFS, to display micro's current BPM data acquisition        */
 /* buffer structure.                                                      */
 
 typedef struct
 {   unsigned long  rpy_dest;
     unsigned short id,
                    flagsr,
                    proctyp,
                    nscanpos,
                    sparew[2],
                    nscampkts,
                    tryy_unit;
     unsigned char  enabyyval,
                    spareb,
                    nppvals,
                    tryynvals,
                    ppvals[BPM_MAXPP],
                    tryyvals[26];
     unsigned long  bpmsmap[(MBPMS+31)/32],
                    toromap[(MAXTOR+31)/32],
                    dugadcmap[(MDUGUNITS+31)/32];
 } sho_1sampreq_ts;
 
#if defined(MAX_MODFR_MASK)
 typedef struct
 {   unsigned short id,
                    nfelbs,
                    misdone[3],
                    mindelay;
     unsigned long  spare[2],
                    exclmask[MAX_MODFR_MASK],
                    inclmask[MAX_MODFR_MASK];
     unsigned char  spareb,
                    nppvals,
                    ppvals[BPM_MAXPP];
 } sho_1fbconn_ts;
 
 typedef struct
 {   unsigned short id;
     unsigned char  nyymux,
                    nmuxmin;
     unsigned long  flagsh,
                    muxcysdone;
     unsigned short nbuftot,
                    nbufavl,
                    nsamps;
     unsigned char  nfbconns,
                    nicals;
     unsigned short nbpmpx,
                    ngadcx,
                    ntorox,
                    nbpmpep2x;
     unsigned       m2buf_mbc;
     unsigned long  creaid;
     unsigned short ncampkts,
                    npducc,
                    rinq_bucket[BPM_RING_MAXCAL];
     unsigned char  nppvals,
                    nyyvals,
                    ppvals[BPM_MAXPP],
                    yyvals[(MAXNMUX+2)*7+4];
     unsigned long  exclmask[MAX_MODFR_MASK],
                    inclmask_strt[MAX_MODFR_MASK],
                    inclmask_nxt[MAX_MODFR_MASK],
                    bpmpmap[(MBPMP+31)/32],
                    gadcmap[(MGADC+31)/32],
                    toromap[(MAXTOR+31)/32],
                    rinqabomap[(MBPMP+31)/32];
 } sho_1bufring_ts;
#endif
 
 typedef struct
 {   unsigned short i_cal,
                    flagscc,
                    rinq_calbx,
                    flip;
     unsigned long  rpy_dest;
 } sho_1calconst_ts;
 
 typedef struct
 {   unsigned long  iss;
     unsigned short sparew,
                    nrings,
                    npubcalconst,
                    nprivcalconst;
     unsigned long  calconst_boff;
     unsigned char  nstopyy,
                    abo_dflt_stopyy[3];
     unsigned long  spare[19];
 } bpm_shobufs_rplybeg_ts;
                         /* The number of sho_1bufring_ts's is given by nrings.   */
                         /* The first sho_1bufring_ts immediately follows         */
                         /* bpm_shobufs_rplybeg_ts.  Any sho_1bufring_ts is       */
                         /* immediately followed by that bufring's nsamps         */
                         /* sho_1sampreq_ts's, which are immediately followed     */
                         /* by that bufring's nbfconns sho_1fbconn_ts's, which    */
                         /* are immediately followed by the next sho_1bufring_ts. */
                         /* The sho_1calconst_ts's follow last.                   */
 
 typedef struct
 {   msgheader_ts           hdr;
     bpm_shobufs_rplybeg_ts dat;
 } bpm_shobufs_rplybegh_ts;
 
 
 /* Structures of request & reply messages (func code BPMO_DUMP_RINQDSP)  */
 /* requesting dump of certain contents of memory from one RINQ BPMP DSP. */
 
#define BPMRINQDSPDUMPTYPCAL 1           /* Value for dumptyp. */
 
 typedef struct
 {   unsigned long  dumptyp;
     unsigned short punit,
                    spare;
 } bpm_dumprinq_req_ts;
 
 typedef struct
 {   msgheader_ts        hdr;
     bpm_dumprinq_req_ts dat;
 } bpm_dumprinq_reqh_ts;
 
#if defined(PB_STRUC_HC)
 typedef struct
 {   unsigned long  iss;
     unsigned short nitem,
                    spare[61];
 } bpm_dumprinq_rplybeg_ts;
 
 typedef struct
 {   msgheader_ts            hdr;
     bpm_dumprinq_rplybeg_ts dat;
 } bpm_dumprinq_rplybegh_ts;
 
 typedef struct
 {   unsigned short itmbc,
                    cal_scpid,
                    dspx,
                    spare;
     pb_dsp_cal_ts  dsp,
                    mic;
 } bpm_dumprinq1cal_ts;        /* If dumptyp = BPMRINQDSPDUMPTYPCAL then immediately */
                               /* after bpm_dumprinq_rplybeg_ts will follow nitem    */
                               /* instances of bpm_dumprinq1cal_ts, each of length   */
#endif                         /* itmbc (mic may be absent).                         */
 
 
 /* Structure of message (func code BPMO_CHK_PDUYYCHANS) */
 /* requesting diagnostic readout and verification of    */
 /* YY-mode PDU channels controlled by BPM code.         */
 
 typedef struct
 {   unsigned short crate,
                    chan,
                    spare[2];
 } bpm_chk_yychans_req_ts;
 
 typedef struct
 {   msgheader_ts           hdr;
     bpm_chk_yychans_req_ts dat;
 } bpm_chk_yychans_reqh_ts;
 
 
                               /* Structure of single data item in BPM-controlled */
 typedef struct                /* PDU channel diagnostic readout reply message.   */
 {   unsigned short pttp,
                    bufring_id;
     unsigned long  timval_exp,
                    timval_rb;
 } chk_yychans_item_ts;
 
                               /* Structure of BPM-controlled PDU channel diagnostic */
 typedef struct                /* readout reply message.                             */
 {   vmsstat_t           iss;
     unsigned short      pttps_n,
                         pttps_lost,
                         pttps_bad,
                         spare[9];
     chk_yychans_item_ts pttps[MAXPTTP];
 } bpm_chk_yychans_reply_ts;
 
 typedef struct
 {   msgheader_ts             hdr;
     bpm_chk_yychans_reply_ts dat;
 } bpm_chk_yychans_rplyh_ts;
 
 
#if defined(MAX_MODFR_MASK)
 
 /* Structure of message (func code BPMO_TIMING_ADJ) */
 /* requesting updating of timing and/or excl+incl   */
 /* masks in running BPM data acquisitions in micro. */
 
                                         /* Definitions of bits in maskt below:    */
#define BPM_TIMINGADJ_EXCLINCL   0x0001  /*  Use given exclusion/inclusion masks.  */
#define BPM_TIMINGADJ_DELY       0x0002  /*  Use given bunchdel.                   */
#define BPM_TIMINGADJ_RINQBUCKET 0x0004  /*  Use given rinq_bucket.                */
 
 typedef struct
 {   unsigned short maskt,
                    sparew,
                    seg_id,
                    cals_n,
                    cal_id[BPM_RING_MAXCAL];
     unsigned long  exclmask[MAX_MODFR_MASK],
                    inclmask_strt[MAX_MODFR_MASK],
                    inclmask_nxt[MAX_MODFR_MASK];
     float          bunchdel[BPM_RING_MAXCAL][2];
     unsigned short rinq_bucket[BPM_RING_MAXCAL];
     unsigned long  spare[12];
 } bpm_timingadj_item_ts;
 
 typedef struct
 {   unsigned short        nadjs,       /* # of elements present in timadj_a. */
                           sparew;
     unsigned long         spare[7];
     bpm_timingadj_item_ts timadj_a[1];
 } bpm_timingadj_req_ts;
 
 typedef struct
 {   msgheader_ts         hdr;
     bpm_timingadj_req_ts dat;
 } bpm_timingadj_reqh_ts;
#endif
 
 
 /* Structure of message (func code BPMO_BUFRING_SUSP) */
 /* requesting suspension or unsuspension of bufrings  */
 /* reading any of given devices.                      */
 
 typedef struct
 {   unsigned short mode,          /* 1 = suspend;  0 = unsuspend.               */
                    ndev,          /* # of device addrs (ignored if unsuspend).  */
                    timout,        /* Timeout in seconds (ignored if unsuspend). */
                    spare,
                    devlocs[20];   /* Low-order word of cctlw for each device.   */
 } bpm_brhangit_req_ts;
 
 typedef struct
 {   msgheader_ts        hdr;
     bpm_brhangit_req_ts dat;
 } bpm_brhangit_reqh_ts;
 
 
 /* Structure of message (func code BPMO_FDBK_SUSPEND) requesting */
 /* suspension/unsuspension of subset of micro's ffeedback loops. */
 
 typedef struct
 {   msgheader_ts   netHeader_s;
     unsigned long  spare[2];
     unsigned short sparew1,
                    suspid_n,
                    susp_loopids[BPM_CONNSUSP_MAXIDS],
                    sparew2,
                    unsuspid_n,
                    unsusp_loopids[BPM_CONNSUSP_MAXIDS];
 } mesfbconnsuspend_ts;
 
#define BPM_REQREPLY_STRUC_H
#endif
 
