/*=============================================================================
 
  Name: bsacPrepTypes.h

  Abs:  Structures and types used by the slc-aware IOC Beam Synchronous Acquisition 
        & Control (BSAC) Facility.  

  Auth:22 JUN-2005, drogind created 
  Rev: 
 
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*----------------------------------------------------------------------------- 
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/

#ifndef INCbsacPrepTypesH
#define INCbsacPrepTypesH 

#include "slcGlob.h"
#include "bpmparam.h"
#include "bpm_reqreply_struc.h"

/* bsacPrep structures */
/* structure containing epics db addrs, per mdef */
/* struct is not in bsacTypes.h to avoid .h conflicts between DBADDR and ca client */
/* These records are written to whenever a SCP-based MDEF is established */
typedef struct {
  DBADDR   CTRL_s;       /* ptr to epics db address for MDEF:IOC:MD:CTRL */
                         /*  bsacAsync also ca monitors this record */
  DBADDR   USERNAME_s;   /* ptr to epics db address for MDEF:IOC:MD:USERNAME */
  DBADDR   ID_s;         /* ptr to epics db address for MDEF:IOC:MD:ID */
  DBADDR   MODE_s;       /* ptr to epics db address for MDEF:IOC:MD:MODE */
  DBADDR   AVGCNTMAX_s;  /* ptr to epics db address for MDEF:IOC:MD: */
  DBADDR   MEASCNTMAX_s; /* ptr to epics db address for MDEF:IOC:MD: */
  DBADDR   YY_s;         /* ptr to epics db address for MDEF:IOC:MD:YY */
  DBADDR   PP_as[4];     /* ptr to epics db address for MDEF:IOC:MD:PP1-4 */
  DBADDR   INCLUSIONA_as[5];/* ptr to epics db address for MDEF:IOC:MD:INCLUSION1-5A */
  DBADDR   INCLUSIONB_as[5];/* ptr to epics db address for MDEF:IOC:MD:INCLUSION1-5B */
  DBADDR   EXCLUSIONA_as[5];/* ptr to epics db address for MDEF:IOC:MD:EXCLUSION1-5A */
  DBADDR   EXCLUSIONB_as[5];/* ptr to epics db address for MDEF:IOC:MD:EXCLUSION1-5B */
}mdefAddr_ts;

/* structure of temp measdef_s that stores parsed measdef info */
/*  if msg is sucessful, this data will be output to the proper */
/*   MDEF:LOCA:MD:** epics PVs */
typedef struct {
  /* start of data stored in EPICS db:  */
  unsigned short measid;             /* i_seg; unique acrss all SCPs & measdefs & cals */
                                     /*   mdef ID */
  unsigned char pp[BPM_MAXPP];       /* NOTE: This is 10, not 4 */
  unsigned char yy;                 

  unsigned long inclmask[MAX_SLCMODFR_MASK];
  unsigned long exclmask[MAX_SLCMODFR_MASK];

} bsacMDEF_ts;

/* Epics address for records PRIM:LOCA:UNIT:UNITID */
/* The Epics records are used to identify the device/unit reporting data */
/* to the bsacHdlr in the BSAC_DATA/DONE messages from the epics db */
/* Note: Other PRIM:LOCA:UNIT:* records are monitored by bsacAsync, and */
/*  therefore are not a part of this structure */
typedef struct {
  DBADDR   UNITID_s;  /* DBADDR for unit index for this unit */
  DBADDR   CTRL_s;    /* DBADDR for CTRL for this unit */
  DBADDR   mdefCTRL_as[MDEF_MAX]; /* dBADDR for CTRL record for this unit, all mdefs  */
} unitIndex_ts; 

typedef struct {
  unitIndex_ts   unit_as[UNITS_MAX]; /* array of primary units */
} primAddrStruct_ts;


#endif /*bsacPrepTypesH*/
