/*=============================================================================
 
  Name: bsacTypes.h

  Abs:  Structures and types used by the slc-aware IOC Beam Synchronous Acquisition 
        & Control (BSAC) Facility.  

  Auth:22 JUN-2005, drogind created 
  Rev: 23 OCT-2006, drogind changed MDEF_MAX from 14 to 10
 
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*----------------------------------------------------------------------------- 
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
		13-jul-2007, dfairley reversed order of CTRL_ONLINE...CTRL_OFFLINE consts.
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/

#ifndef INCbsacTypesH
#define INCbsacTypesH 

#include "slcGlob.h"
#include "msgTypes.h"
#include "bpmparam.h"
#include "bpm_reqreply_struc.h"
#include "bpmstruc.h"
#include "dblist.h"

  /* defines */
#define MDEF_MAX 10    /* was 14; no CA defs now - use EDEFS */
#define MAX_SLCMODFR_MASK 3  /* max modifiers coming from slc */
#define MD_DATABUFS_MAX 10 /* maximum number of large msg buffers for meas data reply */

  /* PRIMINDEX - indexes into array of primStruct_ts */
#define BSACBPMS_INDEX 0
#define BSACTORO_INDEX 1
#define BSACARRY_INDEX 2
#define BSACGADC_INDEX 2
#define BSACWIRE_INDEX 3
#define BSACPUID_INDEX 4
  /* max number of bsac primaries */
#define BSACPRIM_MAX 5

/* MDEF MODE values */
#define MODE_USER      0
#define MODE_FEEDBACK  1
#define MODE_SLCSINGLE 2
#define MODE_SLCBUFFER 3
#define MODE_SLCSINGLE_PUID 4
#define MODE_SLCBUFFER_PUID 5

/* max number of units */
#define BPMS_MAX 10      /* 8 - 10, ie Till */
#define TORO_MAX 16      /* check w Doug */
#define ARRY_MAX 64      /* check w Doug; what about multiple GADC modules? */
#define WIRE_MAX 10      /* TBD */
#define PUID_MAX 1       /* pulse id - one at a time */
#define UNITS_MAX 64     /* max unit # for any primary */

/* indicate mdef is not timing down */
#define NO_TIMEOUT -1

/* define VMS byte size of  bpmp_calbreply_linac_ts for BPMO_CALB_PREP/PROC */
#define CALB_BPMP_REPLY_BC 66
#define CALB_BPMP_REPLY_WC 33
/* define VMS byte size of  bpmp_calbreply_ts for BPMO_CALB_PREP/PROC */
/*  structure has a union -largest member structure is bpmp_creply_ep_ts */
#define CALB_BPMPUNION_REPLY_BC 150
#define CALB_BPMPUNION_REPLY_WC 75
/* define VMS byte size of  gadc_calbreply_ts for BPMO_CALB_PREP/PROC */
#define CALB_GADC_REPLY_BC 160
#define CALB_GADC_REPLY_WC 80 

/* the device CTRL record possible values, BPMS use the CTRL vals */
/* NOTE: dmf now this pv is called ACCESS, and the order is reversed */
#define CTRL_ONLINE   0
#define CTRL_CHECKOUT 2
#define CTRL_MAINT    1
#define CTRL_OFFLINE  3


/* enums */

typedef enum {  /* enumeration of bsac primaries */
   bsacBPMS, bsacTORO, bsacARRY, bsacWIRE, bsacPUID, bsacEnd
} bsacPrim_te;

typedef enum { /* enum of PRIM:LOCA:MD:CTRL states */
  bsacCtrlOnBySCP, bsacCtrlOn, bsacCtrlOff
} bsacCtrl_te;

/* structure definitions */
typedef struct
{  
  unsigned long  job_id;    /* holds the slcIOC thread enum this msg is for NOT THE JOB ENUM */
  fwd_hdr_ts     tcpfwdhdr; /* the tcp forward header used by the proxy */
  msgheader_ts   msgheader; /* the original slc message header */
} bsacHdrs_ts;

typedef struct {
  bsacHdrs_ts reqHdrs;               /* store the request headers for setting up reply bufs */
  unsigned short navg;               /* navg from prep msg; = mdef avgcntmax */
  unsigned short nrpos;              /* nrpos from wire prep msg(=1 if meas prep)*/
                                     /*   ; = mdef meascntmax */  
  unsigned long  nunits;             /* total number of units involved in  measdef (all prims) */
  unsigned long  noffline;           /* number of units that are requested, but offline (when using range) */
  unsigned long  msgCount;           /* inits to (nunits*nrpos); stored for entire life of md*/
  unsigned long  msgCounter;         /* inits to msgCount; counts down w every data msg */ 
  int timeout;                       /* timeout from prep msg (nfwd_sec_max) */
                                     /*   which is unsigned short - cast */
  int timeoutCntr;                   /* timeout countdown (sec), counts down to 0*/
  unsigned long  totalByteCnt;       /* total byte count of reply */
  unsigned short func;               /* function code from prep msg */
  epicsBoolean   abort;              /* flag to indicate aborting */
  unsigned long  measbits;           /* measbits from prep msg */
  /*unsigned long  reqmsgtotwc; 
	unsigned long  reqmsgpartwc;*/
  unsigned short umin, umax, umin2, umax2; /* unit ranges, if not using ulist */
  bpm_measulist_ts ulist;
  unsigned short nunitsCnt[BPM_ULIST_PRIMS]; /*actual count to compare to above */ 
  epicsBoolean measprep;             /* T=measprep; F=wireprep */
  epicsBoolean useUlist;             /* T = use ulist; F=use umin-umax */
  epicsBoolean firstPulse;           /* T = first measured pulse in navg*nrpos meas. */
  unsigned short ndatblks;           /* total # data blocks returned from buf acq*/
                                     /*  place this total in every buffer's alldata reply */
  char username[5];                  /* global storage of username */
  /* 
	 bsacAbortFindScp - the call used to use dbGetFields to read epics
	 records CTRL and USERNAME to find the md.  This caused a segmentation error
	 in LINUX only, which to this point is unexplained.  A workaround was created 
	 by changing the bsacAbortFindScp to check a bsacMD_as[md].username stored 
	 during the measurement preparation 
  */

} bsacMD_ts;


typedef struct {
  short    bufIndex;   /* index into message buffer(s)for data; -1 means none */
  unsigned long offset;    /* BYTE offset into message buffer(s) for unit data */
  unsigned long counter;   /* num of navg data pieces in buffer(s) for this unit */
  epicsBoolean   included;  /* whether this unit is included in the mdef */     
  
} unitStruct_ts; 

typedef struct {
  unitStruct_ts mdef_as[MDEF_MAX];/* array of unitStruct_ts (mdef ptr,cnters) */  
  char     slcUnit_a[5];          /*4 char ASCII unit number, null-terminated */ 
  char     slcUnit4_a[4];         /*4 char ASCII unit number */ 
  unsigned short unitnum;         /* integer unit number used for comparison to u lists */
  unsigned short chanCnt;         /* if ARRY, # of attached channels ("ADCN") */
  short chanCntr;                 /* countdown for arry channels in a mdef,
				       -1=reset to chanCnt, 
				       >0=counting down, 
				       ==0=done for this navg, or no countdown reqd*/
  unsigned short hsta_ctrl;       /* stores the hsta value for this unit in ctrl format*/
  dbheader_ts * dblistHSTA_ps;    /* dblist for this unit hsta */
  unsigned short primIndex;       /* the primary index(device type) of this unit */
} bsacUnits_ts; 

typedef struct {
  bsacUnits_ts   unit_as[UNITS_MAX]; /* array of primary units */
  char           slcPrim_a[5];      /*4 char ASCII primary name, null-terminated */
  unsigned short unitCnt;           /* number of units in this ioc for this primary */
                                    /*  if unitCnt==0, this primary not present */
  unsigned short mdefUnitCnt[MDEF_MAX]; /* number of units involved in mdef for buf acq*/
  dbheader_ts * dblistHSTAs_ps;       /* dblist for unit hstas */
} primStruct_ts;

/* bsac epics facility data message header to identify primary, unit, channel */
typedef struct {
  unsigned short md;                /* meas def # 1-14 from epics record db */
  unsigned short primIndex;         /* primary index from epics record database */
  unsigned short unitIndex;         /* unit index from epics record database */
  unsigned short chanIndex;         /* channel index from epics record database */
} bsacMsgHdr_ts;


/* one channel of arry data that is sent from epics to bsacHdlr */
 typedef struct
 {   float          data,
                    drms;
     unsigned short raw;            /* not used */
     unsigned short stat,
                    goodmeas;
 } arry_chan_data_ts;

/* bsac epics facilty data message */
typedef struct {
  unsigned long  job_id;
  fwd_hdr_ts     tcpfwdhdr_s;
  msgheader_ts   msgheader_s;
  bsacMsgHdr_ts  bsacMsgHdr_s;
  union {
	bpms_data_ts       bpm_s;
	toro_data_ts       toro_s;
    arry_chan_data_ts  arryChan_s;
	ws_bpm_data_ts     bufacq_bpm_s;
	ws_toro_data_ts    bufacq_toro_s;
	ws_puid_data_ts    puid_s;
  } data_u;
} bsacMsg_ts;

#define BSACMSGDATA_WSTART 4        /* unsigned short index for start of device data in 
									   msgmail_ps->reqdata[]; accounts for 
									   header bsacMsgHdr_ts */

/* the following structure keeps track of the totwc and partwc for a set of buffers
   used in a measdef.  word count info must be kept accross yy cycles during a start/stop
   measurement operation.  The buf_ps pointers are updated as msg buffers are 
   allocated, filled, sent, and then re-allocated for each yy cycle.
*/
typedef struct {
  unsigned long  totwc;      /* the totwc of this set of buffers */
  unsigned long  partwc;     /* the partwc of this particular buffer */
  unsigned long  cumpartwc;     /* the partwc of this particular buffer */
  lrgmail_ts    *buf_ps;     /* pointer to this buffer */
  unsigned char *data_p;     /* pointer to alldata hdr portion of buffer */
} bsacBuf_ts;

/* bsacData msg buffer structure for storing data to send */
typedef struct {
  unsigned short bufCount;               /* number of msg bufs for this active md */
  unsigned short nextBuf;                /* when sending, points to next buffer to go */
  bsacBuf_ts bufs_as[MD_DATABUFS_MAX];   /* array of buffer pointers for this md */
} bsacDataBufs_ts;


#endif /*INCbsacTypesH*/
