/*=============================================================================
 
  Name: cstrHdlr.c

        cstrHdlrThread   - main routine for msgHdlr thread
               
        cstrHdlrInitChk1
        cstrHdlrExitChk1
        cstrHdlrInitChk2
        cstrHdlrExitChk2
        cstrHdlrInitChk3
        cstrHdlrExitChk3
        cstrHdlrInitCpum
        cstrHdlrExitCpum
        cstrHdlrChk1
        cstrHdlrChk2
        cstrHdlrChk3
        cstrHdlrCpum
        cstrHdlrInitMicrStatus
        cstrHdlrExitMicrStatus
        cstrHdlrProcMicrSts

        cstrHdlrExit

        cstrHdlrDoStop
        cstrHdlrDoEcho
        cstrHdlrDoEchoMWord
        cstrHdlrDoErrMeterReset
        fillInHeaders
        printMsgheader


  Abs:  cstrHdlr contains the cstrHdlrThread() (thread) that receives
        messags from VMS including TEST function code messages. These
        messages come from the Recv thread and cstrHdlr acts on them,
	then sends a reply to the msgSend thread.

        cstrHdlrThread() also receives messages from cstrAsync via
        the same mechinism.  cstrHdlr runs the Async cycling function
        as requested in the message.

		
  Auth: 01 mar 2005, Ron MacKenzie
  Rev:  20 sep 2005, Stephanie Allison

-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  (newest to oldest)  
        21 Nov 2007, ronm        Add new check function CHK3.
                                 This will send a keepalive message to 
                                 paranoia over the MSG port every time it
                                 runs. This is to fix the problem where
                                 the proxy is rebooted and the slc-ioc doesn't
                                 realize that them msg port needs to reconnect.
                                 The dbex ports aleady have keepalive so they
                                 are ok.

        09 mau 2007, dfairley    commented out call for CPUM cycling function at init,
		  it caused a superfluous error message
        07 may 2007, dfairley    removed dbnametoaddr for MEM and CPU, the pvs 
		  no longer exist. Also fixed case where a msg buffer is not released if there
		  was a bad function code.
        01 mar 2005, ronm        Added code to handle messages from cstrAsync
                                 and do CSTR processing. 
        17 feb 2005, dfairley    copied from msgHdlr.c, the TEST messages are now
		                 handled by the cstrHdlr thread.
        30 Nov 2004, dfairley    re-write to production level coding standards, 
                                 var names etc
=============================================================================*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
#include <stdio.h>
  int cstrHdlrFlag = DP_ERROR;         /* Set to DP_INFO for production. */
#endif

/* c includes */
#include <string.h>        /* memcpy */

/* epics includes */
#include "dbAccess.h"      /* Epics Runtime Database */

/* shared slc micro includes */
#include "msgfunc.hc"      /* FUNC_TEST, TEST_Xxx */
#include "msgdef.h"        /* MSG_ error codes */
#include "slcMsg.h"        /* IOC_STOP */
#include "asyncdef.hc"     /* ASYNC_ error codes */

/* slc global vars */
#include "slcGlob.h"       /* slcThreads_as */

/* slcIOC types */
#include "msgTypes.h"      /* msgmail_ts etc */
#include "slcTypes.h"      /* XXX_JOB_NAME   */

/* slcIOC utilities */
#include "msgQUtil.h"      /* messageQSend, messageQRecv */
#include "slcUtil.h"       /* slcGetJobByEnum proto */
#include "cvtUtil.h"       /* slcSSwap, slcLSwap, etc */
#include "timeUtil.h"      /* timeGetVMS */
#include "slcCmlog.h"      /* slcCmlogLogMsg */
#include "slcAsync.h"      /* slcReqType_te and protos */
#include "cstrHdlr.h"      /* asyncmail_ts and our funcs */
#include  "dblist.h"       /* dblget proto etc */
#include  "dbl.h"          /* dblget dblput    */

#define CSTR_BIT0_MASK 1   /* bit 0 mask       */


/* local function prototypes */
LOCAL int  cstrHdlrInitChk1(void);
LOCAL int  cstrHdlrExitChk1(void);
LOCAL int  cstrHdlrInitChk2(void);
LOCAL int  cstrHdlrExitChk2(void);
LOCAL int  cstrHdlrInitChk3(void);
LOCAL int  cstrHdlrExitChk3(void);
LOCAL int  cstrHdlrInitCpum(void);
LOCAL int  cstrHdlrExitCpum(void);
LOCAL int  cstrHdlrChk1(epicsBoolean * const dbUpdateNeeded_p);
LOCAL int  cstrHdlrChk2(epicsBoolean * const dbUpdateNeeded_p);
LOCAL int  cstrHdlrChk3(epicsBoolean * const dbUpdateNeeded_p);
LOCAL int  cstrHdlrCpum(epicsBoolean * const dbUpdateNeeded_p);
LOCAL int  cstrHdlrInitMicrStatus(void);
LOCAL int  cstrHdlrExitMicrStatus(void);
LOCAL int cstrHdlrProcMicrSts(epicsBoolean * const dbUpdateNeeded_p);

LOCAL void      cstrHdlrExit(void);
LOCAL void      cstrHdlrDoStop(void);
LOCAL vmsstat_t cstrHdlrDoEcho(const msgmail_ts* rcv_ps);
LOCAL vmsstat_t cstrHdlrDoEchoMWord(const msgmail_ts* rMsg_ps);
LOCAL vmsstat_t cstrHdlrDoErrMeterReset(msgmail_ts* rMsg_ps);

  /* File scoped data.  Static makes it private to this file */

static DBADDR       addrCPU_s;         /* Pointers to Epics PV's */
static DBADDR       addrMEM_s;
static long record1_status = 0;        /* Status for Epics dbgets */
static long record2_status = 0;

static unsigned long   cstrJmsk = 0;             /* CSTR:JMSK                */

static DBLIST(dblistCTIM_ps, dbhEntry_ts);       /* DB list pointers for SLC DB data */
static DBLIST(dbdataCTIM_ps, vmsTimeStamp_ta);

static DBLIST(dblistUTIM_ps, dbhEntry_ts);      
static DBLIST(dbdataUTIM_ps, vmsTimeStamp_ta);

static DBLIST(dblistELPS_ps, dbhEntry_ts);  
static DBLIST(dbdataELPS_ps, float);

static DBLIST(dblistNRUN_ps, dbhEntry_ts);   
static DBLIST(dbdataNRUN_ps, long);

static DBLIST(dblistFAIL_ps, dbhEntry_ts);   
static DBLIST(dbdataFAIL_ps, long);

static DBLIST(dblistPUPD_ps, dbhEntry_ts);  
static DBLIST(dbdataPUPD_ps, float);

static DBLIST(dblistPVAX_ps, dbhEntry_ts);  
static DBLIST(dbdataPVAX_ps, float);

static DBLIST(dblistCPU_ps, dbhEntry_ts); 
static DBLIST(dbdataCPU_ps, long);

static DBLIST(dblistRMX_ps, dbhEntry_ts); 
static DBLIST(dbdataRMX_ps, long);

static DBLIST(dblistAMSK_ps, dbhEntry_ts);
static DBLIST(dbdataAMSK_ps, long);

static DBLIST(dblistMSTA_ps, dbhEntry_ts);
static DBLIST(dbdataMSTA_ps, short);


/*=============================================================================

  Name slcSendImAlive

  Abs: 

  Args: Type	            Name        Access	   Description
        ------------------- -----------	---------- ----------------------------
        msgheader_ts        msgheader   read-only  message header structure

  Rem: 

  Side: 

  Ret: 
   
==============================================================================*/
LOCAL vmsstat_t slcSendImAlive(void)
{
  vmsstat_t iss;
  char *microname = slcGetIOCName();
  msgmail_ts * msg_ps=NULL;

  if (SUCCESS( iss = msgQGetBuffer( sizeof(msgmail_ts),(void*) &msg_ps ) ) ) {
	msg_ps->msgheader.func = MICRREQ_IMALIVE;
	memcpy(&msg_ps->msgheader.dest,"V016",4);
	memcpy(&msg_ps->msgheader.source, microname, 4);
	/* DEBUGPRINT(DP_INFO, msgRecvFlag, ("RECV:sending ImAlive to Paranoia\n")); */
	iss = msgQMsgReply(msgSend, msg_ps);
  }
  return (iss);
}

/*=============================================================================

  Name: cstrHdlrThread

  Abs:  The cstr handler thread

  Args: none

  Rem: This thread accepts and acts on messages like FUNC_TEST from the 
       SLC control system. It also receives messages from cstrAsync 
       requesting that an Async cycling function be executed.
       An Async cycling check function is executed each time 
       any message is received.

  side: 

  Ret:  none

==============================================================================*/
/**procedure**/
void  cstrHdlrThread(void)
{
  vmsstat_t            iss;
  int                  asyncFuncIx;              /* cycling function index */
  epicsBoolean         dblputDone = epicsFalse;  /* flag to indicate if func. did dblput */
  msgmail_ts*          rMsg_ps = NULL;       /* read msg buffer pointer */
  slcThread_ts*        cstrHdlr_ps = &slcThreads_as[cstrHdlr]; /* pointer to my thread struct */ 
  slcReqType_te        requestType;          /* vms or async msgs are received */
  asyncmail_ts *       asyncmail_ps = NULL;  /* cycling func ix and name passed from cstrAsync in msg */
  epicsTimeStamp       msgRecvTime_s;        /* time the message was received */

  /*----------------------------------------- code ----------------------------*/

  /* create the message queue id */
  if ( !SUCCESS(messageQCreate(cstrHdlr, MAX_QUEUED_MSGS, MAX_MSGPTR_BYTES)) ) {
	cstrHdlrExit();
	return;
  }

  /* Call cstrHdlrInitChk1() to initialize dblist() and do dblget() for 
   * CTIM, UTIM, and ELPS (one value per function)
   */
  if (cstrHdlrInitChk1() != 0) goto egress;

  /*  Call cstrHdlrInitChk2() to initialize dblist() and do dblget() for 
   *  NRUN, FAIL, PUPD, and PVAX (one value per function)
   */
  if (cstrHdlrInitChk2() != 0) goto egress;


  /*  Call cstrHdlrInitChk3() to do any initialization it needs.
   */
  if (cstrHdlrInitChk3() != 0) goto egress;

  /* Call cstrHdlrInitCPUM() to initialize dblist() and dblget() for 
   * CPU and RMX.  Also initialized EPICS DBaddresses.
   NOTE: CPUM and MEM have been removed, this init simply sets them to 0
   */
  if (cstrHdlrInitCpum() != 0) goto egress;

  /* Call private function cstrHdlrInitMicrStatus() which initializes micro status 
   * data items and calls private function cstrHdlrProcMicrSts() to set AMSK and MSTA 
   * secondaries into the SLC database per the table below. It also echos CSTR:VTIM in MTIM.
   */
  if (cstrHdlrInitMicrStatus() != 0) goto egress;
  if (slcAsyncGetJMSK (&cstrJmsk) != epicsTrue)  goto egress;

  /*
   * Call slcAsyncMeterDbSend to do a dbupdate() to the VMS  
   * database for all the data we set into the lists above.
   */

  asyncFuncIx = slcAsyncGetFuncIxByName("CHK1");
  if (slcAsyncMeterDbSend(asyncFuncIx, slcAsync, epicsTrue) != 0) goto egress;
  
  /* Call the CPUM cycling function now to send an update */

  if (cstrHdlrCpum(&dblputDone) != 0) goto egress;
  /* remove this call, CPUM no longer exists...5/9/07 */
  /*asyncFuncIx = slcAsyncGetFuncIxByName("CPUM");  cause DbStats in global to be set 
  if (slcAsyncMeterDbSend(asyncFuncIx, slcAsync, dblputDone)!= 0) goto egress;
  */
  
  /* Set the ACTIVE flag for exectutive to see */

  cstrHdlr_ps->active = epicsTrue;
  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: becoming active (active flag set)\n"));
  
  /*  **MAIN LOOP** of the cstr hdlr thread, wait at message queue for an incoming 
   *  message and then handle it.
   *  Messages are from either VMS or from cstrAsync.
   *  In the case of msgs from VMS, we often send a reply to the msgSend thread.
   */
  
  while(!cstrHdlr_ps->stop)
  {

    dblputDone = epicsFalse;
    requestType = slcMsg;                           /* use this as default */

	/* read a message at the message queue.Exit if MsgRecv fails. 
         */

	if (!SUCCESS(msgQMsgRecv(cstrHdlr, &msgRecvTime_s, (void*)&rMsg_ps) )) goto egress;

	/* TODO * CHECK LENGTH OF MSG RECVD HERE. IF BAD, THEN, issue msg and goto egress **/

	DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, 
		   ("\nCSTR_H: **incoming msg** func code = %x\n", rMsg_ps->msgheader.func));

	/*handle the incoming message */
    asyncFuncIx = -1;
	switch(rMsg_ps->msgheader.func) {
	case IOC_STOP:
	    cstrHdlrDoStop();
	    goto egress;
	case FUNC_TEST:
	    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("\nCSTR_H:got a keepalive message\n"));
	    iss = cstrHdlrDoEcho(rMsg_ps);
	    break;
	case TEST_ECHO:
	    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("\nCSTR_H:got an ECHO message\n"));
	    iss = cstrHdlrDoEcho(rMsg_ps);
	    break;
	case TEST_ECHO_MWORD:
	    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("\nCSTR_H:got an ECHO_MWORD message\n"));
	    iss = cstrHdlrDoEchoMWord(rMsg_ps);
	    break;
	case TEST_ERR_METER_RESET:
	    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("\nCSTR_H:got an ERR_METER_RESET message\n"));
	    iss = cstrHdlrDoErrMeterReset(rMsg_ps);
	    break;
    case IOC_ASYNC:
	    requestType = slcAsync;
	    asyncmail_ps = (asyncmail_ts*)rMsg_ps->reqdata; /* point to asyncHdlr part of msg */                
            DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, 
		       ("\nCSTR_H:got an IOC_ASYNC message. INDEX = %d, FUNC = %s\n",
			 asyncmail_ps->cyclIx, asyncmail_ps->cyclName  ));
            asyncFuncIx = asyncmail_ps->cyclIx;            /* used below */

            /* Call the cycling function requested in the message */

        if (strncmp(asyncmail_ps->cyclName, "CPUM", 4) == 0) {
	      if (cstrHdlrCpum(&dblputDone) != 0) {
			goto egress;
	      }
	    }
	    else if  (strncmp(asyncmail_ps->cyclName, "CHK1", 4) == 0) {  
	      if (cstrHdlrChk1(&dblputDone) != 0) {
			goto egress;
	      }
	    }

	    else if  (strncmp(asyncmail_ps->cyclName, "CHK2", 4) == 0) {
	      if (cstrHdlrChk2(&dblputDone) != 0) {
			goto egress;
	      }
	    }

	    else if  (strncmp(asyncmail_ps->cyclName, "CHK3", 4) == 0) {
	      if (cstrHdlrChk3(&dblputDone) != 0) {
			goto egress;
	      }
	    }

	    else {      /* BAD Cycling function name */
	      requestType = none;
	      DEBUGPRINT(DP_ERROR, cstrHdlrFlag, ("CSTR_H:got an CYCL FUNC in message - dumping it\n"));
	      slcCmlogLogMsg(ASYNC_BAD_FCNNAME, "BAD_FCNNAME",  
			   "cstrHdlr received Bad Cycling Function Name = %s \n",asyncmail_ps->cyclName);
	    }

            break;
 	default:
	    /* unknown msg -  */
        requestType = none;
	    DEBUGPRINT(DP_ERROR, cstrHdlrFlag, ("CSTR_H:got an unknown message func type - dumping it\n"));
            slcCmlogLogMsg(ASYNC_BAD_FCNNAME, "BAD_FCNNAME",  
			   "cstrHdlr received Bad Function Type.  Its name = %d \n", rMsg_ps->msgheader.func);
	    break;   
	} /* switch on msg func*/

    if (requestType == slcMsg) {               /* if msg from vms, do CHK1 */ 	
	  if (cstrHdlrChk1(&dblputDone) != 0) {
	    goto egress;
	  }
	  asyncFuncIx = slcAsyncGetFuncIxByName("CHK1");  /* cause DbStats in global to be set */
	}

	/* release the receive buffer */
	msgQFreeBuffer(sizeof(msgmail_ts), rMsg_ps);

	if (requestType == none) continue;         /* just loop around for next msg if bad msg code */


    /* Call slcAsyncMeterDbSend if or if-not the Check function 
         * indicated dbUpdateNeeded.  If it returns bad status, call cstrHdrlExit() to exit the task.
	*/

    if (slcAsyncMeterDbSend(asyncFuncIx, requestType, dblputDone) != 0) goto egress;

        /*  Call slcAsyncSetFcnStats to update statistics for the function in the async 
	 *  function (global) table (this is done inside CHK functions on RMX). If it returns bad status, 
         *  call cstrHdlrExit() to exit the task. */

    if ( slcAsyncSetFcnStats(asyncFuncIx, &msgRecvTime_s, requestType) != 0) goto egress;  

  } /* while active */

  egress:

  /* We're here because stop flag set, stop function recv'd or error. It's time to shut down 
   */

  /* 
   * Release the receive buffer in case we were STOPed, or recv error happened.
   */
  msgQFreeBuffer(sizeof(msgmail_ts), rMsg_ps);


  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: shutdown\n"));
  cstrHdlrExit();

  } /* end of cstrHdlrThread */

/*==============================================================================

  Name: cstrHdlrInitChk1()

  Abs:  Initialize for CHK1 cycling function.


  Args: none

  Rem: Call dblistalloc(), dblist() are called to set up lists for later access to 
       CTIM, UTIM, and ELPS (one value per function).  Pointers to lists are in file 
       scope for use by other utilities.  Dblget() is called to get initial values 
       which may be overwritten later.

  side: 

  Ret: int  0 = success 
           -1 = Error 

==============================================================================*/
LOCAL int cstrHdlrInitChk1(void)
{

  vmsstat_t       iss       = ASYNC_OKOK;

/*-------------------------------------- code ----------------------------------*/

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrInitChk1\n"));

  /* 
   * CTIM is an array of vms timeStamps containing  last cycling function timestamp.
   * It's slc db format is 40T4. 
   */

  if (!SUCCESS(iss=DBALLOC(dblistCTIM_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataCTIM_ps, SLC_MAX_N_ASYN))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CTIM",
			   (const dbheader_ts **const) &dblistCTIM_ps))) goto egress;

  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistCTIM_ps,
                           (dbheader_ts **const)&dbdataCTIM_ps))) goto egress;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: CTIM TIMESTAMP FETCHED at INIT TIME ** First = %ld  \n",
				     (long)*DBPTR_FIRST(dbdataCTIM_ps)));

  /*
   *  --------------------------------------------------------------------
   * UTIM is an array of vms timeStamps containing last database update..
   * It's slc db format is 40T4. 
   * --------------------------------------------------------------------
   */

  if (!SUCCESS(iss=DBALLOC(dblistUTIM_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataUTIM_ps, SLC_MAX_N_ASYN))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"UTIM",
			   (const dbheader_ts **const) &dblistUTIM_ps))) goto egress;

  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistUTIM_ps,
                           (dbheader_ts **const)&dbdataUTIM_ps))) goto egress;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: UTIM TIMESTAMP FETCHED at INIT TIME ** First = %ld  \n",
				     (long)*DBPTR_FIRST(dbdataUTIM_ps)));
	
  
  /*
   *  --------------------------------------------------------------------
   * ELPS is an array of vms reals containing elapsed time for last cycling function (in seconds).
   * It's slc db format is 20R4
   * --------------------------------------------------------------------
   */

  if (!SUCCESS(iss=DBALLOC(dblistELPS_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataELPS_ps, SLC_MAX_N_ASYN))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"ELPS",
			   (const dbheader_ts **const) &dblistELPS_ps))) goto egress;

  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistELPS_ps,
                           (dbheader_ts **const)&dbdataELPS_ps))) goto egress;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: ELPS FLOAT FETCHED at INIT TIME  %f \n", *DBPTR_FIRST(dbdataELPS_ps)));


 egress:
  if (!SUCCESS(iss)) return -1;
  return 0;
}

/*==============================================================================

  Name: cstrHdlrExitChk1()

  Abs:  Release the resources allocated by cstrHdlrInitChk1().

  Args: none

  Rem: 

  side: 

  Ret:  int  0 = success 
            -1 = Error 

==============================================================================*/
LOCAL int cstrHdlrExitChk1(void)
{

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrExitChk1\n"));

  DBFREE(dblistCTIM_ps);
  DBFREE(dbdataCTIM_ps);
  DBFREE(dblistUTIM_ps);
  DBFREE(dbdataUTIM_ps);
  DBFREE(dblistELPS_ps);
  DBFREE(dbdataELPS_ps);

  return 0;
}

/*==============================================================================

  Name: cstrHdlrInitChk2()

  Abs:  Initialize for CHK2 cycling function.

  Args: none

  Rem: Call dblistalloc(), dblist() are called to set up lists for later access to 
       NRUN, FAIL, PUPD, and PVAX (one value per function).  Pointers to lists are in file 
       scope for use by other utilities.  Dblget() is called to get initial values 
       which may be overwritten later.


  side: 

  Ret:  int  0 = success 
            -1 = Error

==============================================================================*/
LOCAL int cstrHdlrInitChk2(void)
{
  vmsstat_t iss = ASYNC_OKOK;

/*-------------------------------------- code ----------------------------------*/

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrInitChk2\n"));

  /* 
   * NRUN is an array of ints containing # of executions of function (eg. checks) in
   * last calculation interval.  It's slc db format is 20I4. 
   */

  if (!SUCCESS(iss=DBALLOC(dblistNRUN_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataNRUN_ps, SLC_MAX_N_ASYN))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"NRUN",
			   (const dbheader_ts **const) &dblistNRUN_ps))) goto egress;

  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistNRUN_ps,
                           (dbheader_ts **const)&dbdataNRUN_ps))) goto egress;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: NRUN  FETCHED at INIT TIME ** First = %ld  \n",
				     *DBPTR_FIRST(dbdataNRUN_ps)));

  /* 
   * FAIL is an array of ints containing # of failed updates in last calculation interval.
   * It's slc db format is 20I4. 
   */

  if (!SUCCESS(iss=DBALLOC(dblistFAIL_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataFAIL_ps, SLC_MAX_N_ASYN))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"FAIL",
			   (const dbheader_ts **const) &dblistFAIL_ps))) goto egress;

  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistFAIL_ps,
                           (dbheader_ts **const)&dbdataFAIL_ps))) goto egress;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: FAIL  FETCHED at INIT TIME ** First = %ld  \n",
				     *DBPTR_FIRST(dbdataFAIL_ps)));


  /*
   *  --------------------------------------------------------------------
   * PUPD is an array of vms reals containing % of function executions triggering update in last
   * calculation interval (not including FAILed updates).
   * It's slc db format is 20R4
   * --------------------------------------------------------------------
   */

  if (!SUCCESS(iss=DBALLOC(dblistPUPD_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataPUPD_ps, SLC_MAX_N_ASYN))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"PUPD",
			   (const dbheader_ts **const) &dblistPUPD_ps))) goto egress;

  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistPUPD_ps,
                           (dbheader_ts **const)&dbdataPUPD_ps))) goto egress;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: PUPD FETCHED at INIT TIME %f \n", *DBPTR_FIRST(dbdataPUPD_ps)));

  /*
   *  --------------------------------------------------------------------
   * PVAX is an array of vms reals containing % of function executions triggering 
   * update in last  calculation interval (not including FAILed updates).
   * It's slc db format is 20R4
   * --------------------------------------------------------------------
   */

  if (!SUCCESS(iss=DBALLOC(dblistPVAX_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataPVAX_ps, SLC_MAX_N_ASYN))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"PVAX",
			   (const dbheader_ts **const) &dblistPVAX_ps))) goto egress;

  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistPVAX_ps,
                           (dbheader_ts **const)&dbdataPVAX_ps))) goto egress;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: PVAX FETCHED at INIT TIME %f \n", *DBPTR_FIRST(dbdataPVAX_ps)));

 egress:
  if (!SUCCESS(iss)) return -1;
  return 0;
}

/*==============================================================================

  Name: cstrHdlrExitChk2()

  Abs:  Release resources allocated by cstrHdlrInitChk1().

  Args: none

  Rem: 

  side: 

  Ret:   int  0 = success 
             -1 = Error

==============================================================================*/
LOCAL int cstrHdlrExitChk2(void)
{

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrExitChk2\n"));

  DBFREE(dblistNRUN_ps);
  DBFREE(dbdataNRUN_ps);
  DBFREE(dblistFAIL_ps);
  DBFREE(dbdataFAIL_ps);
  DBFREE(dblistPUPD_ps);
  DBFREE(dbdataPUPD_ps);
  DBFREE(dblistPVAX_ps);
  DBFREE(dbdataPVAX_ps);

  return 0;
}


/*==============================================================================

  Name: cstrHdlrInitChk3()

  Abs:  Initialize for CHK3 cycling function.

  Args: none

  Rem: Doesn't do anything because this check function does not do DB operations.


  side: 

  Ret:  int  0 = success 
            -1 = Error

==============================================================================*/
LOCAL int cstrHdlrInitChk3(void)
{
  vmsstat_t iss = ASYNC_OKOK;

/*-------------------------------------- code ----------------------------------*/

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrInitChk3\n"));
 
   return 0;
}



/*==============================================================================

  Name: cstrHdlrExitChk3()

  Abs:  Release resources allocated by cstrHdlrInitChk1().

  Args: none

  Rem:  Nothing done because this function does no db operations.

  side: 

  Ret:   int  0 = success 
             -1 = Error

==============================================================================*/
LOCAL int cstrHdlrExitChk3(void)
{

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrExitChk2\n"));

  return 0;
}


/*==============================================================================

  Name: cstrHdlrInitCpum()

  Abs:  Initialize for CPUM cycling function.

  Args: none

  Rem:  Epics PV's (IOC:micr:1:CPU and IOC:micr:1:MEM) will be accessed for use in 
        calculating SLC database CPU and RMX CSTR secondaries and the initial 
        values will be written to the SLC database using dblput().

        Perform Initialization for access to these Epics PV's and for writing
        the values to the slc database.

        A bad status from any db routine except dbNameToAddr() causes a call 
        to cstrHdlrExit by the caller (and the thread exits).

  side: 

  Ret:   int  0 = success 
             -1 = Error

==============================================================================*/
LOCAL int cstrHdlrInitCpum(void)
{
  vmsstat_t              iss              = ASYNC_OKOK;
  char         channel_c[80];           /* Epics PV channel name */


  /*----------------------------------- code --------------------------------*/

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrInitCpum\n"));

  /*
   *   Call EPICS dbNameToAddr() to get the pointer for each PV. If bad status don't 
   *     return an error: log a cmlog message and zero rmx and cpum. 
   *     Epics PV's are 
   *       IOC:micr:1:CPU 
   *       IOC:micr:1:MEM
   *     As you can see, the Epics PV name format is:
   *     IOC:(4 char ioc name):1:(secondary name of interest). 
   *     The 4 char ioc name is determined using executive routine slcGetIOCName(). 
   * 
   *   If an error occurs, just set data item to zero and keep going.
   */

  /*  THESE PVS no longer EXIST  - just set to 0
  sprintf (channel_c, "IOC:%s:1:CPU", slcGetIOCName());
  if ((record1_status = dbNameToAddr(channel_c, &addrCPU_s))) {
  */
    addrCPU_s.precord = 0;                   /* keeps us from doing dbget */
	/*
    DEBUGPRINT(DP_ERROR, cstrHdlrFlag, ("CSTR_H: dbNameToAddr of MEM failed in cstrHdlrInitCpum\n"));
  }

  sprintf (channel_c, "IOC:%s:1:MEM", slcGetIOCName());
  if ((record2_status = dbNameToAddr (channel_c, &addrMEM_s))) {
*/
    addrMEM_s.precord = 0;                   /* keeps us from doing dbget */
/*
    DEBUGPRINT(DP_ERROR, cstrHdlrFlag, ("CSTR_H: dbNameToAddr of MEM failed in cstrHdlrInitCpum\n"));
  }
*/
  
  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: EPICS MEM and CPU Pointers set to 0 ***** \n" ));

  /* Call dblistalloc(), dblist(), dblget() for CSTR: CPU (1I4 in slc db) 
   * CPU is CPU idle (0-100%)
   */ 

  if (!SUCCESS(iss=DBALLOC(dblistCPU_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataCPU_ps, 1))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CPU ",
			   (const dbheader_ts **const) &dblistCPU_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistCPU_ps,
                           (dbheader_ts **const)&dbdataCPU_ps))) goto egress;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, 
	     ("CSTR_H: SLC DB CPU FETCHED AT INIT = %d \n",(int)*DBPTR_FIRST(dbdataCPU_ps)));


  /* Call dblistalloc(), dblist(), dblget() for CSTR: RMX (1I4 in slc db) 
   * RMX is available IOC OS (was RMX for slc micros)  memory pool (bytes)
   */ 

  if (!SUCCESS(iss=DBALLOC(dblistRMX_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataRMX_ps, 1))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"RMX ",
			   (const dbheader_ts **const) &dblistRMX_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistRMX_ps,
                           (dbheader_ts **const)&dbdataRMX_ps))) goto egress;


  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, 
	     ("CSTR_H: SLC DB RMX FETCHED AT INIT ****************= %d \n",(int)*DBPTR_FIRST(dbdataRMX_ps)));
  
egress:
  if (!SUCCESS(iss)) return -1;
  return 0;

}  /* end of cstrHdlrInitCpum */

/*==============================================================================

  Name: cstrHdlrExitCpum()

  Abs:  Free resources allocated by cstrHdlrInitCpum()

  Args: none

  Rem: 

  side: 

  Ret:   int  0 = success 
             -1 = Error

==============================================================================*/
LOCAL int cstrHdlrExitCpum(void)
{

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrExitCpum\n"));

  DBFREE(dblistCPU_ps);
  DBFREE(dbdataCPU_ps);
  DBFREE(dblistRMX_ps);
  DBFREE(dbdataRMX_ps);

  return 0;
}

/*==============================================================================

  Name: cstrHdlrChk1()

  Abs:  CHK1 cycling function. Update  UTIM, CTIM, and ELPS. Also cause AMSK/MSTA processing to occur.

  Args: 
       Type:         epicsBoolean
       Name:         dbUpdateNeeded_p
       Access:       W
       Description:  Indicates if dblput was done here.
                     If set, someone needs to do dbupdate later.

  Rem: 

  side: 

  Ret:   int  0 = success 
             -1 = Error

==============================================================================*/
LOCAL int cstrHdlrChk1(epicsBoolean * const dbUpdateNeeded_p)
{

  int retStat;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrChk1\n"));

  *dbUpdateNeeded_p = epicsFalse;
  
  /* Call cstrHdlrProcMicrSts() to process and update CSTR secondaries 
   * AMSK and MSTA based on which jobs are active on this slcIoc.
   */

  if ((retStat = cstrHdlrProcMicrSts(dbUpdateNeeded_p)) != 0) goto egress;

  /* Call slcAsyncProcChk1 to update UTIM, CTIM, and ELPS.*/

  if ((retStat = slcAsyncProcChk1(DBPTR_FIRST(dbdataCTIM_ps),
                                  DBPTR_FIRST(dbdataUTIM_ps),
                                  DBPTR_FIRST(dbdataELPS_ps))) != 0) goto egress;

  if (!SUCCESS(dblput ((const dbheader_ts *const)dblistUTIM_ps,
                       (const dbheader_ts *const)dbdataUTIM_ps, cstrJob))) {
    DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
		       ("CSTR_H: dblput FAILED in cstrHdlrChk1 for UTIM \n")); 
    retStat = -1;
    goto egress;
  }  

  if (!SUCCESS(dblput ((const dbheader_ts *const)dblistCTIM_ps,
                       (const dbheader_ts *const)dbdataCTIM_ps, cstrJob))) {
    DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
		       ("CSTR_H: dblput FAILED in cstrHdlrChk1 for CTIM \n")); 
    retStat = -1;
    goto egress;
  }

  if (!SUCCESS(dblput ((const dbheader_ts *const)dblistELPS_ps,
                       (const dbheader_ts *const)dbdataELPS_ps, cstrJob))) {
    DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
		       ("CSTR_H: dblput FAILED in cstrHdlrChk1 for ELPS \n")); 
    retStat = -1;
    goto egress;
  }  


  *dbUpdateNeeded_p = epicsTrue;  /* we always do dbupdate for this check function */

 egress:
  
  return retStat;

}  /* end cstrHdlrChk1 */

/*==============================================================================

  Name: cstrHdlrChk2()

  Abs:  CHK2 Cycling function.  Update NRUN, FAIL, PVAX, and PUPD

  Args: 
       Type:         epicsBoolean
       Name:         dbUpdateNeeded_p
       Access:       W
       Description:  Indicates if dblput was done here.
                     If set, someone needs to do dbupdate later.

  Rem:

  side: 

  Ret:   int  0 = success 
             -1 = Error

==============================================================================*/
LOCAL int cstrHdlrChk2(epicsBoolean * const dbUpdateNeeded_p)
{

  int retStat;  /* return status */
  epicsBoolean NRUN_changed;
  epicsBoolean FAIL_changed;
  epicsBoolean PVAX_changed;
  epicsBoolean PUPD_changed;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrChk2\n"));

  *dbUpdateNeeded_p = epicsFalse;
  
  /* Call slcAsyncProcChk2 to update NRUN, FAIL, PVAX, and PUPD. */

  if ((retStat = slcAsyncProcChk2(DBPTR_FIRST(dbdataNRUN_ps),
                                  DBPTR_FIRST(dbdataFAIL_ps),
                                  DBPTR_FIRST(dbdataPVAX_ps),
                                  DBPTR_FIRST(dbdataPUPD_ps), 
                                  &NRUN_changed, &FAIL_changed,
                                  &PVAX_changed, &PUPD_changed)) != 0) goto egress;
    /* Do dblupts for the lists filled above if any new data in any of them */
  if (NRUN_changed) {
    *dbUpdateNeeded_p = epicsTrue;
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: cstrHdlrChk2 dblput of NRUN list \n"));
    if (!SUCCESS(dblput ((const dbheader_ts *const)dblistNRUN_ps,
                             (const dbheader_ts *const)dbdataNRUN_ps, cstrJob))) {
      DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
                 ("CSTR_H: dblput FAILED in cstrHdlrChk2 for NRUN \n"));
      retStat = -1;
      goto egress;
    }
  }
  if (FAIL_changed) {
    *dbUpdateNeeded_p = epicsTrue;
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: cstrHdlrChk2 dblput of FAIL list \n"));
    if (!SUCCESS(dblput ((const dbheader_ts *const)dblistFAIL_ps,
                             (const dbheader_ts *const)dbdataFAIL_ps, cstrJob))) {
      DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
                 ("CSTR_H: dblput FAILED in cstrHdlrChk2 for FAIL \n"));
      retStat = -1;
      goto egress;
    }
  }
  if (PUPD_changed) {
    *dbUpdateNeeded_p = epicsTrue;
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: cstrHdlrChk2 dblput of PUPD list \n"));
    if (!SUCCESS(dblput ((const dbheader_ts *const)dblistPUPD_ps,
                         (const dbheader_ts *const)dbdataPUPD_ps, cstrJob))) {
      DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
                 ("CSTR_H: dblput FAILED in cstrHdlrChk2 for PUPD \n"));
      retStat = -1;
      goto egress;
    }
  }
  if (PVAX_changed) {
    *dbUpdateNeeded_p = epicsTrue;
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: cstrHdlrChk2 dblput of PVAX list \n"));
    if (!SUCCESS(dblput ((const dbheader_ts *const)dblistPVAX_ps,
                         (const dbheader_ts *const)dbdataPVAX_ps, cstrJob))) {
      DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
                 ("CSTR_H: dblput FAILED in cstrHdlrChk2 for PVAX \n"));
      retStat = -1;
      goto egress;
    }
  }
  
egress:
  
  return retStat;

}  /* end cstrHdlrChk2 */


/*==============================================================================

  Name: cstrHdlrChk3()

  Abs:  CHK3 Cycling function.  Send keepalive msg to vms paranoia.

  Args: 
       Type:         epicsBoolean
       Name:         dbUpdateNeeded_p
       Access:       W
       Description:  Indicates if dblput was done here.
                     If set, someone needs to do dbupdate later.

  Rem: The framework is here if you ever want to do DB operations like other 
       CHK functions do.

  side: 

  Ret:   int  0 = success 
             -1 = Error

==============================================================================*/
LOCAL int cstrHdlrChk3(epicsBoolean * const dbUpdateNeeded_p)
{

  int retStat;  /* return status */

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrChk3\n"));

  /* Call slcAsyncProcChk3 to see if it's time to do DB operations */
  /* This is a NO_OP now.  But if you ever want to, you can do db operations */

  if (retStat = slcAsyncProcChk3() != 0) goto egress;

    /* 
     * If you ever want to,
     * do dblupts here for the lists filled above if any new data in any in them 
     */



  /* This is the only real work this CHK function does */

  slcSendImAlive();



egress:  

  return retStat;

}  /* end cstrHdlrChk3 */



/*==============================================================================

  Name: cstrHdlrCpum()

  Abs:  CPUM cycling (check) function.

  Args: 
       Type:         epicsBoolean
       Name:         dbUpdateNeeded_p
       Access:       W
       Description:  Indicates if dblput was done here.
                     If set, someone needs to do dbupdate later.

  Rem: dbget on epics pv's and check for change. Do Dblput() to send new
       values to VMS if changed.

  side: 

  Ret:   int  0 = success 
             -1 = Error

==============================================================================*/
LOCAL int cstrHdlrCpum(epicsBoolean * const dbUpdateNeeded_p)
{
  int       retStat = 0;
  vmsstat_t iss     = ASYNC_OKOK;

  long  epicStat = 0;
  long  nRequest = 1;      /* these are for fetching Epics PV's */
  long  options  = 0;
  float tmpFloat;

  /*------------------------------------- code -------------------------------*/

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrCpum\n"));

  *dbUpdateNeeded_p = epicsFalse;

  /* Get CPU from EPICS PV.
   * NOTE: We're allowing dbGet to fail without exiting with bad status 
   */

  tmpFloat = 0.0;
  /* DONT EVEN TRY TO READ THE CPU PV, it does not exist 
  if (addrCPU_s.precord) {                if pv exists, get value from Epics 
    if ((epicStat = dbGet(&addrCPU_s, DBR_FLOAT, &tmpFloat, &options, &nRequest, NULL))) {
      if (!record1_status) {                if first failure, save status 
        record1_status = epicStat;
	DEBUGPRINT(DP_ERROR, cstrHdlrFlag, ("CSTR_H: dbGet of CPU FAILED!!!! \n"));
      }    
    }
    else {  else data is good - use it. 
      
      DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: dbGet got IOC:CPU = %f \n",tmpFloat));
    }
  }
  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H:      *dbdataCPU_p = %d ******************\n",
									  (int)*DBPTR_FIRST(dbdataCPU_ps)));
  */

  /*
   * If it changed, store new value in list and dblput.  Set dbUpdateNeeded = true
   */
  if ((long)tmpFloat != *DBPTR_FIRST(dbdataCPU_ps)) { /* notice, one is float, one is long! */
    *DBPTR_FIRST(dbdataCPU_ps) = (long)tmpFloat;
    *dbUpdateNeeded_p = epicsTrue;
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag,
               ("CSTR_H: Sending new CPU = %d to VMS \n", (int)tmpFloat));
    if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblistCPU_ps,
                             (const dbheader_ts *const)dbdataCPU_ps, cstrJob))) {
      DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
                 ("CSTR_H: dblput FAILED in cstrHdlrCpum for CPU \n"));
      retStat = -1;
      goto egress;
    }
  }
  else {
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: CPU didn't change, so NOT SENDING \n"));
  }
  /* Get MEM from Epics PV and store as CSTR:RMX in SLC DB. 
   * NOTE: We're allowing dbGet to fail without exiting with bad status 
   */
  tmpFloat = 0.0;
  /* DONT EVEN TRY TO READ THE MEM pv, it does not exist 
  if (addrMEM_s.precord) {                if data exists 
    if ((epicStat = dbGet(&addrMEM_s, DBR_FLOAT, &tmpFloat, &options, &nRequest, NULL))) {
      if (!record2_status) {               if first failure, save status 
        record2_status = epicStat;
	DEBUGPRINT(DP_ERROR, cstrHdlrFlag, ("CSTR_H: dbGet of MEM FAILED!!!! \n"));
      }    
    }
    else {  else data is good - use it. 
      
      DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: dbGet got IOC:MEM = %f \n",tmpFloat));
    }
  }
  */
  /*
   * If it changed, store new value in list and dblput.  Set dbUpdateNeeded = true
   */
  if ((long)tmpFloat != *DBPTR_FIRST(dbdataRMX_ps)) {
    *DBPTR_FIRST(dbdataRMX_ps) = (long)tmpFloat;
    *dbUpdateNeeded_p = epicsTrue;
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag,
               ("CSTR_H: Sending new RMX = %d to VMS \n", (int)tmpFloat));
    if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblistRMX_ps,
                             (const dbheader_ts *const)dbdataRMX_ps, cstrJob))) {
      DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
                 ("CSTR_H: dblput FAILED in cstrHdlrCpum for RMX \n"));
      retStat = -1;
      goto egress;
    }
  }
  else {
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: RMX didn't change, so NOT SENDING \n"));
  }
  
 egress:
  
  return retStat;

} /* end cstrHdlrCpum */

/*==============================================================================

  Name: cstrHdlrInitMicrStatus()

  Abs:  Initialize data items and dblists associated with micro (ioc) status after startup. 

  Args: none

  Rem: This function is only called by another cstrHdlr routine.  So, it is static.

  side: 

  Ret: int  
       0  = success
      -1  = failure


==============================================================================*/
LOCAL int cstrHdlrInitMicrStatus(void)
{

  vmsstat_t iss;           /* vms status */
  vmsTimeStamp_ta       currTime_a; /* current time */

  /* These database items are for one time usage in this function.
   * They are allocated, used, and freeded here.
   */
  DBLIST(dblist_ps, dbhEntry_ts);      
  DBLIST(dbdata_ps, int);

  /*----------------------------------------- code --------------------------*/

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrInitMicrStatus\n"));

  /* 
   **********************************************************
   * Set up DB lists for CSTR,[this_micro],1:AMSK and MSTA. 
   **********************************************************
   */

  if (!SUCCESS(iss=DBALLOC(dblistAMSK_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataAMSK_ps, 1))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"AMSK",
			   (const dbheader_ts **const) &dblistAMSK_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistAMSK_ps,
                           (dbheader_ts **const)&dbdataAMSK_ps))) goto egress;

  if (!SUCCESS(iss=DBALLOC(dblistMSTA_ps, 1))) goto egress;
  if (!SUCCESS(iss=DBALLOC(dbdataMSTA_ps, 1))) goto egress;
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"MSTA",
			   (const dbheader_ts **const) &dblistMSTA_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblistMSTA_ps,
                           (dbheader_ts **const)&dbdataMSTA_ps))) goto egress;

  /*
   ********************************************************************************************
   * CSTR,[this_micro],1,[CRTS, CRTT, CRVn, CAM, NTIM, and TSTA, ] are set in the database once 
   * using dblget(),dblput(), dblistfree(). They are never set again.  
   * We don't need to call dbupdate here because caller calls slcAsyncMeterDbSend after this.
   *******************************************************************************************
   */
   
  /* CRTS ************************************************************/
  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CRTS",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /* CRTT *******************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CRTT",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /* CRV1 *****************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CRV1",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /* CRV2 *********************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CRV2",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /* CRV3 ***********************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CRV3",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /* CRV4 ************************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CRV4",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /* CRV5 **********************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CRV5",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;


  /* CRV6 *******************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CRV6",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /* CRV7 **********************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CRV7",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /* CAM **********************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"CAM ",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /* NTIM *****************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"NTIM",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /* TSTA *****************************************************/

  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"TSTA",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memset(DBPTR_FIRST(dbdata_ps), 0, DBLISTSIZE(dbdata_ps));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  /*************************************************************************
   *  The value of CSTR:VTIM is echoed to VMS in CSTR:MTIM as an indication 
   *  that the micro's database has been downloaded.  This is accomplished 
   *  by doing a dblistalloc()/dblist() for each secondary, dblget() for 
   *  CSTR:VTIM, dblput() putting the VTIM value to the database using 
   *  the MTIM list.
   */
  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR",
                           (const char *)"1   ",
                           (const char *)"VTIM",
                           (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  if (timeGetVMS(currTime_a)) {
    iss = MSG_BADTIME;
    slcCmlogLogMsg(iss, "BADTIME", "Init Failed because epicTimeGetCurrent Failed\n");
    goto egress;
  }
  /* Use VTIM only if the restart was due to an SLC IPL request */
  if (slcGetNotify()) {
    memcpy(currTime_a, DBPTR_FIRST(dbdata_ps), sizeof(vmsTimeStamp_ta));
  }  
  DBCLEAR(dblist_ps);
  DBCLEAR(dbdata_ps);
  if (!SUCCESS(iss=dblist ((const char *)"CSTR", 
			   (const char *)"1   ", 
			   (const char *)"MTIM",
			   (const dbheader_ts **const) &dblist_ps))) goto egress;
  if (!SUCCESS(iss=dblget ((const dbheader_ts *const)dblist_ps,
                           (dbheader_ts **const)&dbdata_ps))) goto egress;
  memcpy(DBPTR_FIRST(dbdata_ps), currTime_a, sizeof(vmsTimeStamp_ta));
  if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblist_ps,
                           (const dbheader_ts *const)dbdata_ps, cstrJob))) goto egress;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: *************************** MTIM (TIME) Succefully written to VMS at INIT TIME \n"  ));

egress:

  /* FREE all the one-time lists we obtained above */

  DBFREE(dblist_ps);
  DBFREE(dbdata_ps);

  if (!SUCCESS(iss)) {
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_A: Init FAILED in slcAsyncInitVerify !!!!!!!!!!!!\n"));
    return -1;
  }
  return 0;

} /* cstrHdlrInitMicrStatus */

/*==============================================================================

  Name: cstrHdlrExitMicrStatus()

  Abs:  

  Args: none

  Rem: 

  side: 

  Ret: int 
       0  = success
       -1 = failure

==============================================================================*/
LOCAL int cstrHdlrExitMicrStatus(void)
{
  
  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrExitMicrStatus\n"));

  DBFREE(dblistAMSK_ps);
  DBFREE(dbdataAMSK_ps);
  DBFREE(dblistMSTA_ps);
  DBFREE(dbdataMSTA_ps);

  return 0;
}

/*==============================================================================

  Name: cstrHdlrProcMicrSts()

  Abs:  Process AMSK and MSTA (while looking at JMSK) based on which
        jobs are active on this slcIoc.

  Args: 
       Type:         epicsBoolean
       Name:         dbUpdateNeeded_p
       Access:       W
       Description:  Indicates if dblput was done here.
                     If set, someone needs to do dbupdate later.

  Rem:

  side: 

  Ret:  int 
        0 = success
       -1 = failure

==============================================================================*/
LOCAL int cstrHdlrProcMicrSts(epicsBoolean * const dbUpdateNeeded_p)
{

  int           jobIdx;         /* loop counter */
  vmsstat_t     iss;            /* vms like status */
  int           retStat = 0;    /* return status */
  unsigned long newAMSK = 0;    /* new CSTR:AMSK here */
  short         newMSTA;        /* new CSTR:MSTA here */

  /*-------------------------------- code ------------------------------------*/
  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Entered cstrHdlrProcMicrSts\n"));
  *dbUpdateNeeded_p = epicsFalse;

  /* Go through slcJob_as and set a bit in AMSK for each active job.  
   * An active job has the active flag set in slcThread_as for all the threads that it has.    
   * At this time, that is only one thread (the Hdlr).  
   * If it's not  set, then the bit is not set in AMSK for that job.
   */

  /*					
Diane said Check this for active jobs:
slcJobs_as[jobIdx].hdlr_e != slcInvalid and
slcThreads_as[slcJobs_as[jobIdx].hdlr_e].active ==epicsTrue
  */

  for (jobIdx=0; jobIdx<MAX_SLC_JOBS; jobIdx++) 
  {
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: JobIx = %d  HDLR_E = %d, active = %d \n",  jobIdx,
					slcJobs_as[jobIdx].hdlr_e,
                                        slcThreads_as[slcJobs_as[jobIdx].hdlr_e].active));
    if ((slcJobs_as[jobIdx].hdlr_e  != slcInvalid) && 
	slcThreads_as[slcJobs_as[jobIdx].hdlr_e].active == epicsTrue) {
        newAMSK |= (CSTR_BIT0_MASK<<jobIdx);       /* set the bit in AMSK */
    }
  }

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: in cstrHdlrProcMicrSts AMSK is: %x \n", 
				      (unsigned int)newAMSK));

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: in cstrHdlrProcMicrSts JMSK is: %x \n", 
				      (unsigned int)cstrJmsk));

  /* 
   * If AMSK != JMSK, set MSTA to BAD(0)
   * ELSE set MSTA to ALIVE(1).
   */
  /* 
   * When it is noticed that a thread has exited (ie AMSK is different than JMSK): 
   * 
   * 1) It is desired to have users notice the problem and attempt a restart manually 
   *    (and if the thread that's dead is not critical, then they could time the restart 
   *    to be less invasive). So, no automatic restart will be attempted.  BTW, this 
   *    is the way EPICS works - when a task suspends or exits, it doesn't crash the IOC. 
   *    Instead people start noticing (sometimes subtle) problems and notify controls.
   *    It doesn't happen very often.
   *
   * 2) A message will be logged indicating the thread that has died. 
   *    logged every time through as a reminder that repair and restart are needed.
   * 
   * 3) An automatic SLC restart should not be attempted since the fatal error would 
   *    probably just happen again. 
   *
   */

  if (newAMSK != cstrJmsk) {
    newMSTA = 0;  /* bad */
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("AMSK != JMSK seen ProcMicrSts!!!!!!!!!!!!!! \n"));
    slcCmlogLogMsg(ASYNC_JOB_DEAD, "JOB_DEAD", "slcIOC Job unexpectly DEAD in AMSK.  AMSK = %x, JMSK = %x \n",
		   newAMSK, cstrJmsk);
  }
  else {
    newMSTA = 1;  /* good, alive */
  }

  /* Call dblput() to put local MSTA and AMSK. (do dblput() only if AMSK or MSTA changed)
   * Set dbupdateNeeded flag appropriately.
   */

  if (newMSTA != *DBPTR_FIRST(dbdataMSTA_ps)) {
    *DBPTR_FIRST(dbdataMSTA_ps) = newMSTA;
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("MSTA Change seen in ProcMicrSts \n"));
    *dbUpdateNeeded_p = epicsTrue;
    if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblistMSTA_ps,
                             (const dbheader_ts *const)dbdataMSTA_ps, cstrJob))) {
      DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
		("CSTR_H: dblput FAILED in cstrHdlr for MSTA \n")); 
      retStat = -1;
      goto egress;
    }
  }

  if (newAMSK != *DBPTR_FIRST(dbdataAMSK_ps)) {
    *DBPTR_FIRST(dbdataAMSK_ps) = newAMSK;
    DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("AMSK Change seen in ProcMicrSts \n"));
    *dbUpdateNeeded_p = epicsTrue;
    if (!SUCCESS(iss=dblput ((const dbheader_ts *const)dblistAMSK_ps,
                             (const dbheader_ts *const)dbdataAMSK_ps, cstrJob))) {
      DEBUGPRINT(DP_ERROR, cstrHdlrFlag,
		("CSTR_H: dblput FAILED in cstrHdlr for AMSK \n")); 
      retStat = -1;
      goto egress;
    }
  }


 egress:
  return retStat;
}



/*=============================================================================

  Name: cstrHdlrDoStop()

  Abs:  terminate this thread by setting the active flag to false

  Args: 
	none

  Rem:  setting active to false will break the thread out of it's loop and cause 
        it to begin the termination process.

  side: the slcThreads_as[cstrHdlr].active flag is set to false.

  Ret: none

===============================================================================*/
LOCAL void cstrHdlrDoStop(void)
{
  slcThreads_as[cstrHdlr].active = epicsFalse;
}

/*=============================================================================

  Name: cstrHdlrDoEcho()

  Abs:  reply to the TEST_ECHO message with a generic RESPONSE  message. That is
        a copy of the incoming message with the RESPONSE bit set.

  Args: 
       Type:        msgmail_ts *
       Name:        recv_ps
       Access:      RW
       Description: pointer to the receive buffer

  Rem:  

  side: a reply message is placed in the msgSend message queue

  Ret: vmsstat_t  returns error code from createOkReply function

===============================================================================*/
LOCAL vmsstat_t cstrHdlrDoEcho(const msgmail_ts* rcv_ps)
{
  int  i;
  unsigned long reqWords;
  msgmail_ts* snd_ps = NULL;
  vmsstat_t iss;
  
  /* get a buffer for the reply data */
  iss = msgQGetBuffer( sizeof(msgmail_ts),(void*)&snd_ps);
  if ( SUCCESS(iss) )
  {
    /* echo the data as is */
	reqWords = rcv_ps->msgheader.datalen;
	if ( (reqWords>0) && (reqWords <= (NETVAXMSGLEN-(sizeof(snd_ps->msgheader)/2))) )
	{
	  for (i = 0; i < reqWords; i++)  snd_ps->reqdata[i] = rcv_ps->reqdata[i];
	}
	else reqWords = 0;

    /* debug - print the message */
    slcPrintMsgheader(&snd_ps->msgheader, cstrHdlrFlag );

    /* send the message */
    iss = msgQSlcReply(rcv_ps->msgheader.func, (reqWords*2), (void*)rcv_ps, (void*)snd_ps);
  }
  return ( iss );
}

/*=============================================================================

  Name: cstrHdlrDoEchoMWord()

  Abs:  reply to the TEST_ECHO_MWORD with a copy of the incoming msg, with RESPONSE
        bit set.  This message has data in the reqdata portion

  Args: 
       Type:        msgmail_ts *
       Name:        recv_ps
       Access:      RW
       Description: pointer to the recieve buffer

  Rem:  

  side: a reply message is placed in the msgSend message queue

  Ret:  vmsstat_t  returns error code from sendOkReply function

===============================================================================*/
LOCAL vmsstat_t cstrHdlrDoEchoMWord(const msgmail_ts* rcv_ps)
{
  int  i;
  unsigned short reqWords;
  msgmail_ts* snd_ps = NULL;
  vmsstat_t iss;
  int hdrbytes = sizeof(msgheader_ts) + sizeof(fwd_hdr_ts);

  /* must calc the size of the outgoing message */
  reqWords = rcv_ps->reqdata[0];

  /* convert the count in reqdata[0] */
  cvtSSwap(&reqWords); 
  if ( reqWords >= NETMICLONGMSGLEN-hdrbytes  ) 
	reqWords = 0;
  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H:EchoMWord sending %u words of data to VMS\n", reqWords) );

  /* assume the data in reqdata is not swapped */
  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H:EchoMWord sending copies of 0x%.4x\n", rcv_ps->reqdata[1]));

  /* get a buffer for the reply data */
  iss = msgQGetBuffer( (reqWords*2 + hdrbytes), (void*)&snd_ps );
  
  /* now set up the reply message */
  if ( SUCCESS(iss) ) {

	/* echo the incoming reqdata[1], repeated with a count of reqdata[0] */
	for (i = 0; i < reqWords; i++)  
	  snd_ps->reqdata[i] = rcv_ps->reqdata[1];

	/* debug - print the message header*/
	slcPrintMsgheader(&snd_ps->msgheader, cstrHdlrFlag);

    /* send the message */
    iss = msgQSlcReply(rcv_ps->msgheader.func, (reqWords*2), (void*)rcv_ps, (void*)snd_ps);
  }
  return ( iss );
}

/*==============================================================================

  Name: cstrHdlrDoErrMeterReset()

  Abs:  TEMPORARY - TBD for now just reply all is OKOK

  Args: 
       Type:        msgmail_ts *
       Name:        recv_ps
       Access:      RW
       Description: pointer to the recieve buffer

  Rem: 

  side: TBD

  Ret: vmsstat_t the result of the sendReply function

==============================================================================*/
LOCAL vmsstat_t cstrHdlrDoErrMeterReset(msgmail_ts* rMsg_ps)
{
  vmsstat_t iss = ASYNC_OKOK;

  /* iss = err_meter_reset() this procedure is TBD */
  rMsg_ps->msgheader.datalen = 2;
  memcpy(&rMsg_ps->reqdata[0], &iss, 4);
  return ( cstrHdlrDoErrMeterReset(rMsg_ps) ); /* temporary */
}


/*==============================================================================

  Name: cstrHdlrExit()

  Abs:  cleanup the globals for this thread before shutdown

  Args: none

  Rem: clean up the slcThreads_as struct for this thread
       and release any resources 

  side: 

  Ret: none

==============================================================================*/
LOCAL void cstrHdlrExit(void)
{

  /*----------------------------------- code ---------------------------------*/

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: entering cstrHdlrExit \n"));

  /* Deallocate resources related to individual cycling functions */

  cstrHdlrExitChk1();
  cstrHdlrExitChk2();
  cstrHdlrExitChk3();
  cstrHdlrExitCpum();
  cstrHdlrExitMicrStatus();

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: Setting Task Active Flag to false \n"));

  messageQRelease(cstrHdlr);
  slcThreads_as[cstrHdlr].active = epicsFalse;
  slcThreads_as[cstrHdlr].tid_ps = NO_THREAD;

  DEBUGPRINT(DP_DEBUG, cstrHdlrFlag, ("CSTR_H: cstrHdlr Stopped and out of service \n"));


  return;
}
