/*=============================================================================
 
  Name: dbSendCtl.c
        dbSendWriteUpdates
		dbSendWaitCtlAck
		dbCtlSockConnect
		dbCtlSockDisconnect 
		dbCvtUpdatesSupHdr

  Abs:  Waits at message queue to receive inter-thread messages.
        Sends "database service" (DBS) type messages to DBEX port 
        on proxy server for routing to VMS Control System's DBEX process.

        Reference web pages on LCLS Controls | Facilities |SLC-Aware IOC | :
                        Database Service (DBS) Design Document 
                        Functional Requirements | SLC Database Service (DBS)
                        Database Input/Output Utilities
                        C Coding Standard

        Reference RMX_DBS dbmain.c 

  Auth:23 MAR 2005, drogind created
  Rev: 
 
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*----------------------------------------------------------------------------- 
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
#include <stdio.h>
int dbCtlFlag = DP_ERROR;
#endif
/* c includes */
#include     <stdio.h>
#include     <stdlib.h>    /* malloc, free */
#include     <string.h>    /* memcpy */

/* slc global vars */
#include     "slcGlob.h"

/* VMS Control system function/status codes  */
#include     "msgfunc.hc"  /* FUNC_MSG function code */
#include     "micrdef.hc"  /* MICR_EXNET_ codes */

/* slc common includes */
#include     "tcpUtil.h"   /* tcpGetConnection, tcpGeBuffer */
#include     "msgQUtil.h"  /* message queue utilities */
#include     "cvtUtil.h"   /* slc conversion utilities */
#include     "timeUtil.h"  /* slc time conversion utilities */
#include     "slcCmlog.h"  /* slcCmlogLogMsg */
#include     "slcUtil.h"   /* slcGetIOCName */ 
#include     "slcMsg.h"    /* DB_ACK_DBEX... */

/* dbs includes  */  
#include     "db_micro_struc.h"
#include     "dbUtil.h" 
#include     "dbmsgmail.h" 
#include     "dbGlob.h"  
#include     "dbdef.hc" 
#include     "dbSend.h"
#include     "dbSendCtl.h"

/* epics osi includes */
#include     "cantProceed.h"
/*****************************************************************************/
#define  MAX_RECEIPT_LOOP 5
#define  CTL_SOCK_WAIT 3
/*****************************************************************************/
/* local file scope data declarations and function prototypes */
LOCAL vmsstat_t dbCtlSockDisconnect (void);
LOCAL int dbCvtUpdatesSupHdr (const short supn, 
							  dbsndrcvbuf_ts  *dbsendbuf_ps);
LOCAL void dbSendComputeDiagTimes(void);

/* diagnostics */
/* dbSend thread diagnostics - var is public in dbSend.c */
extern dbSendDiags_ts dbSendDiags_s;
/*=============================================================================

  Name: dbCvtUpdatesSupHdr

  Abs:  Prepare dbsendbuf_ps with native supn header prior to sending
        ST 2 or 3 updates to dbex

  Args: Type		       Name	       Access	 Description
	-------------------  -----------  -------   ----------------------------
	    const short       supn          R         supertype (2 or 3) data to send
		dbsndrcvbuf_ts  * dbsendbuf_ps  W         file scoped global send buffer pointer

  Rem:  This procedure is called by dbSendWriteUpdates in order to
        convert designated supn VMS header residing in dbnodes_a
		to native and store in *dbsendbuf_ps  

  Side: dbsendbuf_ps must have specific supn header prepared for the update
        message to dbex. The existing VMS supn header residing in dbnodes_a is
		converted to native and stored in the nativeSB field of dbsendbuf_ps. This
		nativeSB is formatted, converted back to VMS in the supblk field of
		dbsendbuf_ps, prior to sending.

  Ret:  vmstat_t
Replace DB_CANTSEND with other error code message to indicate connection went down, then up
The socket down/up could be DBEX/CTL or both.
==============================================================================*/
LOCAL int dbCvtUpdatesSupHdr (const short supn, 
							  dbsndrcvbuf_ts  *dbsendbuf_ps) 
{
  int bytecnt = 0;

  /* convert VMS superblock header to native */
  bytecnt = dbVMStohSupHdr(dbnodes_a[supn],&dbsendbuf_ps->nativeSB);
  dbsendbuf_ps->nativeSB.blkbeg = (bytecnt /2) + 1;
  dbsendbuf_ps->nativeSB.id= (SUPERTYPE_ID_WMASK & dbsendbuf_ps->nativeSB.id)| SUPERTYPE_ID_ACKREQ ;
  return (bytecnt);
}

/*=============================================================================

  Name: dbSendWriteUpdates

  Abs:  This procedure is called by dbSendUpdates in order to send a job's 
        ST2 or ST3 updates that have accumulated in a job's
        dbupdateHiLo buffer.

  Args: Type		       Name	       Access	 Description
	-------------------  -----------  -------   ----------------------------
	    const slcJob_te   slcJobId_e    R         job enum for which to send updates
	    const short       supn          R         supertype index into hilo buffer
		                                          0=ST2; 1=ST3
		dbsndrcvbuf_ts  * dbsendbuf_ps  W         file scoped global send buffer pointer

  Rem:  dbSendWriteUpdates sends all items on calling job's 
        (slcJobId_e) list of pending supertype 2
        or 3 updates. This procedure is the result of another job thread issuing a dbupate().
		dbupdate sends a message to dbSend and waits at its mutex:
		slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateHiLoMutex_ps
        until it is signalled back by this routine.
		
		Following loop goes through slcJobId_e's pending supertype 3 update      
		items, sending each item separately as one or more messages (message size 
		will always be <= NETBUFLEN words (including database "superblock" header 
		which is part of every message sent from here)).  Each item (which may be 
		further divided, and which itself never includes any of the database      
		"superblock" header) is represented as a doubleton in wxa:  lower bound   
		and upper bound;  each bound is a 1-biased index into the micro's entire  
		supertype 3 block considered as a word array including its database       
		"superblock" header;  lower bound is index of first word to be            
		transmitted;  upper bound is index of last word to be transmitted.  Items 
		we cant send we retain on this subjob's list.                             

  Side: NOTE: Calling job thread is blocked waiting for an ack back from DBEX
        on the CTL port. 
		Ported from dbmain.c's  microdbsendb().
		Each job has a mutex around R/W of their update buffers (dbupdateSTHiLo[supnInd])
		The mutex, as well as the udpate buffer is accessed via
		slcJobs_as[slcJobId_e].dbupdate_ps pointer. The mutex is locked around
		The global dbRWMutex_ps, which protects all supertype data,  must also be 
		taken when reading dbnodes_a data and copying into dbsendbuf_ps. It doesn't need 
		to be taken around the superblock header copy, since this is only written once
		upon download, and only read thereafter.

  Ret:  vmstat_t
Replace DB_CANTSEND with other error code message to indicate connection went down, then up
The socket down/up could be DBEX/CTL or both.
==============================================================================*/
vmsstat_t dbSendWriteUpdates(const slcJob_te slcJobId_e,
							 const short supnInd,
							 dbsndrcvbuf_ts  *dbsendbuf_ps) {

#define MDBSENDB_ISS_MAXN  7
  dbhilo_job_ts *jbhilo_ps;
  epicsMutexId   jbhiloMutex_ps;      
  int            supblkbcVMS;    /* VMS superblock byte count */
  unsigned short dbmsgbc;         /* tot # bytes to send in this update */
  unsigned       iss_n, jj, wx, wc;
  vmsstat_t      iss_a[MDBSENDB_ISS_MAXN], iss,  issx = DB_OKOK;
  /*----------------------------------------------*/


  jbhilo_ps = &slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateSTHiLo[supnInd];
  jbhiloMutex_ps = slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateHiLoMutex_ps;

  /* anything to send? */
  if (jbhilo_ps->wxn==0) 
	return issx;
 
  if (jbhiloMutex_ps){
	DEBUGPRINT(DP_INFO,dbCtlFlag, ("dbSendUpdates: setting job mutex for slcJobId=%d.\n",
								   (int)slcJobId_e));		 
	epicsMutexMustLock(jbhiloMutex_ps);
  }
  else {
	DEBUGPRINT(DP_ERROR,dbCtlFlag,("dbupdateHiLo: bad dbupdateHiLoMutex_ps for slcJobId=%d \n",
								   (int)slcJobId_e));
	return (iss=DB_BADPTR);
  }
  DEBUGPRINT(DP_INFO,dbCtlFlag, ("dbSendUpdates: %lu updates in job update buffer for supnInd %hd for slcJobId=%d\n", 
								 jbhilo_ps->wxn, supnInd, (int)slcJobId_e));

  /* convert VMS dbnode[supn] supblk to sendbuf_ps->supblk.nativeSB */
  supblkbcVMS = dbCvtUpdatesSupHdr (supnInd+2, dbsendbuf_ps);

  for (iss_n = jj = 0;  jj < jbhilo_ps->wxn;)
	{
	  for (wx = jbhilo_ps->wxa[jj][0];  wx <= jbhilo_ps->wxa[jj][1];  wx += wc)
		{
		  wc = jbhilo_ps->wxa[jj][1] + 1 - wx <=
		    (sizeof(dbsendbuf_ps->supblk) - supblkbcVMS)/2 ?
			jbhilo_ps->wxa[jj][1] + 1 - wx :
			(sizeof(dbsendbuf_ps->supblk) -supblkbcVMS)/2;

		  dbsendbuf_ps->nativeSB.segbeg = wx;
		 
		  /* read data from correct supertype area; lock first */
		  if (dbRWMutex_ps)
			epicsMutexMustLock(dbRWMutex_ps);
		  else {
			DEBUGPRINT(DP_ERROR,dbCtlFlag,("dbSendCtl: bad dbRWMutex_ps for slcJobId=%d \n", (int)slcJobId_e));
			return (iss=DB_BADPTR);
		  }
		  memcpy(&dbsendbuf_ps->supblk.datw[supblkbcVMS/2],
				 &dbnodes_a[supnInd+2]->datw[wx-1], wc << 1);
		  if (dbRWMutex_ps)
			epicsMutexUnlock(dbRWMutex_ps);

		  /* use const for sbhdr size */
		  /* dbmsgbc = (wc << 1) + sizeof(dbsendbuf_ps->supblk.sbhdr);*/
		  dbmsgbc = dbhtoVMSSupHdr(&dbsendbuf_ps->nativeSB, &dbsendbuf_ps->supblk) 
		    + (wc << 1);

		  DEBUGPRINT(DP_INFO,dbCtlFlag, ("dbSendUpdates: wc=%u; dbmsgbc=%u\n", wc, dbmsgbc));		  
		  dbprinthSupHdr ("Native ST3 udpate header", &dbsendbuf_ps->nativeSB, dbmsgbc);	
		  /*dbprintVMSSupHdr ("VMS ST3 update header", &dbsendbuf_ps->supblk, dbmsgbc);*/
	  
		  if (!SUCCESS(issx = dbWriteDbs(slcSockets_as[sdDbex].sd,
						 dbsendbuf_ps->nativeSB.id,
						 &dbmsgbc, 
						 dbsendbuf_ps)))  goto exnetdbsendberr;
		  epicsTimeGetCurrent(&dbSendDiags_s.slcLastUpdTime);  /* update time of last sld update to dbex */
		  DEBUGPRINT(DP_DEBUG,dbCtlFlag, ("dbSendUpdates: Success writing to socket\n"));			  
		  jbhilo_ps->datwc += wc;
		  ++jbhilo_ps->msgc;
		  ++dbSendDiags_s.slcUpdCnt;
		}
	  memmove(jbhilo_ps->wxa[jj], jbhilo_ps->wxa[jj+1],
			  (jbhilo_ps->wxn-(jj+1))*sizeof(jbhilo_ps->wxa[0]));
	  --jbhilo_ps->wxn;
	  continue;
	  
	exnetdbsendberr:
	  if (iss_n < MDBSENDB_ISS_MAXN)
		iss_a[iss_n++] = issx;
	  ++jbhilo_ps->errc;
	  ++jj;
	  ++dbSendDiags_s.slcUpdDropCnt;
	  DEBUGPRINT(DP_ERROR,dbCtlFlag, ("dbSendUpdates: error waiting for ack from Ctrl socket; slcJobId=%d \n", 
									  (int)slcJobId_e));	
	}

  if (jbhiloMutex_ps) {
	DEBUGPRINT(DP_INFO,dbCtlFlag, ("dbSendUpdates: Resetting job mutex...\n"));	
	epicsMutexUnlock(jbhiloMutex_ps);
  }

  if (iss_n == 0)
	{
	  jbhilo_ps->difmin = MINDBSNDWSEP;
	  issx = DB_OKOK;
	}

  else  /* error on at least one of the individual updates */
	{
	  for (jj = 0;  jj < iss_n;  ++jj) {
		slcCmlogLogMsg(iss_a[jj], "DB_CANTSEND", "%s dbSendUpdates error for slcJobId=%d  \n", 
						  epicsThreadGetNameSelf(), (int)slcJobId_e);
	  }
	}
return issx;

#undef MDBSENDB_ISS_MAXN
}  
/*===========================================================

  Name: dbSendWaitCtlAck

  Abs:  Now wait here for ack back from DBEX, since we have a separate  
        connection for ack in this direction.  This could take a few    
        milliseconds so provide a timeout just in case this doesnt work 
        as planned (such as re-connect, proxy dead, etc).  If we see a  
        failure, then we do return bad status.                          
        If non-blocking socket does not have data, wait for 1 sec, then 
	    check .stop flag, loop up to 15 (sec) times
  
  Args: none

  Rem:  returns if this thread is told to stop
        
  Side: Called from dbWriteDbs() after it sent an ST update to DBEX

  Ret:  iss - the status code
                                     
Ret: vmsstat_t MICR_OKOK
     vmsstat_t MICR_EXNET_BADACK for bad ack
     vmsstat_t MICR_EXNET_RECVTMOUT if the call timed out without getting a ack after 15x1sec loop
	 vmsstat_t MICR_EXNET RECVFAIL if there was an error on the select call (-1)  
===========================================================*/ 
vmsstat_t dbSendWaitCtlAck(void)
{
  vmsstat_t iss = /*DB_TIMEOUT*/MICR_OKOK;
  int i;
  unsigned short       max_bytecount;        /* max # bytes we could receive */
  int                  act_bytecount;
  fwd_hdr_ts           receipt_header_s;     /* Y */ 

  max_bytecount = 0; /* not including fwd header */
  /* now begin, until connection fails, or we're told to quit */
  /* loop 15 times, waiting 1 sec each time to see if data has arrived at CTL socket */
  for (i = 0;  i < MAX_RECEIPT_LOOP;  ++i)  
	{
	  if (slcThreads_as[dbSend].stop){
		slcThreads_as[dbSend].active = epicsFalse;
		return (iss);
	  }

	  if (SUCCESS(iss =tcpSockReadyForRead(&slcSockets_as[sdCtl],CTL_SOCK_WAIT))) {
		
		if (!SUCCESS(iss = tcpGetBuffer(&slcSockets_as[sdCtl], 
										max_bytecount, 			      
						                &act_bytecount,
										&receipt_header_s,                      
										NULL)))  /* no data portion expected */
		  {   /* if socket error - go disconn, reconn, check stop flag */
			dbCtlSockDisconnect();
		  }
		
		DEBUGPRINT(DP_INFO,dbCtlFlag,
				   ("DB SEND:dbSendWaitCtlAck rec'ed byte0=%hd\n", receipt_header_s.user.byte0));
		dbSendComputeDiagTimes();
		epicsTimeGetCurrent(&dbSendDiags_s.ctlLastAckTime);  /* update time of last sld update to dbex */
		if (receipt_header_s.user.byte0 == dbsendReceipts) {
		  DEBUGPRINT(DP_INFO,dbCtlFlag,("DB SEND:dbSendWaitCtlAck success; count=%ld\n", 
										dbSendDiags_s.ctlAckCnt));
		  break;
		} 
		else  {
		  ++dbSendDiags_s.slcInvAckCnt;
		  iss = MICR_EXNET_BADACK;
		  DEBUGPRINT(DP_ERROR,dbCtlFlag,("DB SEND:dbSendWaitCtlAck err- byte0 mismatch\n"));
		  /* could be packets are out of order..*/
		  /* must break because ack has already been read; distinguish bet. timeout case */
		  /*break;*/
		}
	  } /* if ready */
	  else { 
		/* either timeout or socket failure.....
		   Returns from tcpSockReadyForRead: 
		   vmsstat MICR_OKOK if the socket has a message waiting
		   vmsstat MICR_EXNET_RECVTMOUT if the call timed out without getting a msg
		   vmsstat MICR_EXNET RECVFAIL if there was an error on the select call (-1)
		*/
		DEBUGPRINT(DP_INFO,dbCtlFlag,("DB SEND:dbSendWaitCtlAck waited 1 sec..\n"));
		/* NOTE - DMF try to just re-connect
		   every other time there is no msg - this will help if proxy goes down.
		   if the proxy is down, the dbCtlSockDisconnect will re-try connecting
		   until the proxy comes back 
		*/
		if ( (iss==  MICR_EXNET_RECVFAIL) || ((i%2) == 0)){
		  /* check specifically for socket failure case */ 
		  /* try to close socket and reconnect; also check stop flag */
		  dbCtlSockDisconnect();
		} /*if sock err */
	  } /* else */ 
	}/* for */
  /* timeout */
  if (iss==MICR_EXNET_RECVTMOUT){
	/* increment # times a job's updates timeout after 15 sec */
	++dbSendDiags_s.slcTimeoutAckCnt;
  }
  return iss;        
}
/*===========================================================

  Name: dbSendComputeDiagTimes

  Abs:  Calculates various diagnostic times for updates
        to transmit to Alpha on dbex port, and for the Alpha to ack
		back on ctl port.

  Args: none

  Rem:  All times are epics; times are in seconds
        
  Side: 

  Ret:  None
===========================================================*/ 
LOCAL void dbSendComputeDiagTimes(void ) {
  float deltaTime = 0.0;
  /* update time of last sld update to dbex */
  
  epicsTimeGetCurrent(&dbSendDiags_s.ctlLastAckTime);
  /* update total ack count; right or wrong */
  ++dbSendDiags_s.ctlAckCnt;
  /* calc delta send update to ack back time */
  deltaTime=epicsTimeDiffInSeconds(&dbSendDiags_s.ctlLastAckTime, &dbSendDiags_s.slcLastUpdTime); 
  /* update total time for average calc */
  dbSendDiags_s.totUpdToAckTime+=deltaTime;
  /* compute average delta time */
  dbSendDiags_s.avgUpdToAckTime = dbSendDiags_s.totUpdToAckTime/dbSendDiags_s.ctlAckCnt;
  /* updtae max delat time */ 
  if (deltaTime > dbSendDiags_s.maxUpdToAckTime) {
	dbSendDiags_s.maxUpdToAckTime = deltaTime;
  }
}

/*===========================================================

  Name: dbCtlSockConnect

  Abs:  Get a Ctl socket connection for dbSend. 
        dbSend is in charge of managing the CTL, read only,
		nonblocking socket

  Args: none

  Rem:  the tcpSockNonblockingConnect routine loops forever
        until either the connection is made, or this thread
		is told to stop via its .stop flag
		If stop flag is set upon return from that routine,
		dbSendCleanup() is called and thread exits.
		If socket was successful after a non-zero elapsed time,
		the global dbsendAllUpdates flag is set such that dbSend
		flag will send out all updates for all jobs. This takes care
		of sending out all updates that have accumulated while 
		the connection was down.
        
  Side: Passes slcSockets_as[sdCtl] as argument
        Sets dbsendAllUpdates global flag

  Ret:  iss - the status code
===========================================================*/ 
vmsstat_t dbCtlSockConnect(void)
{
  vmsstat_t      iss= MICR_OKOK;
  unsigned short proxy = PROXIES_HOST_MSG_PORT;    /* Proxy port number */
  unsigned short hostctl =  MICROS_HOST_CTL_PORT ; /* our 'porttag'*/
  unsigned long  elapsed_count = 0;
  float          wait_time = 1.0;

  DEBUGPRINT(DP_DEBUG,dbCtlFlag,("DB SEND: get ctl socket non blocking connection..\n")); 
  if (slcSockets_as[sdCtl].sd==INVALID_SOCKET) {
	iss = tcpSockNonblockingConnect(dbSend, proxy, hostctl, wait_time, &elapsed_count, &slcSockets_as[sdCtl] );  
	if (SUCCESS(iss)){
	  dbSendDiags_s.ctlSockConnStatus =  slcSockets_as[sdCtl].sd;;
	  ++dbSendDiags_s.ctlSockConnCnt;
	  if (elapsed_count >0){
		dbSendDiags_s.ctlLastDisconnTime = elapsed_count * wait_time;
		if ( (slcGetDbExists()) && (slcThreads_as[dbSend].active)){
		  DEBUGPRINT(DP_INFO,dbCtlFlag,("DB SEND: dbCtlSockConnect setting dbsendAllUpdates flag!\n"));
		  dbsendAllUpdates = epicsTrue;
		}
	  }
	}
  } 
  return (iss);
}
/*===========================================================

  Name: dbCtlSockDisconnect

  Abs:  Disconnects CTL socket
        Then attempts to re-establish connection
		if thread is not stopping

  Args: none

  Rem: Called when a socket error is detected when
       trying to receive data at CTL socket either on a 
	   tcpSockReadyForRead() call, or tcpGetBuffer() call.
	   Closes the socket, then immed. tries to reconnect.

  Side: access slcSockets_as[sdCtl]

  Ret:  iss - the status code
===========================================================*/ 
LOCAL vmsstat_t dbCtlSockDisconnect (void){
  vmsstat_t     iss = MICR_OKOK;

  tcpSockClose(&slcSockets_as[sdCtl]);
  dbSendDiags_s.ctlSockConnStatus =  slcSockets_as[sdCtl].sd;
  DEBUGPRINT(DP_ERROR,dbCtlFlag,("DB SEND:dbSendCtlSockDisconnect disconnecting Ctl socket due to error detected; will try to re-establish connection..\n"));
  iss=dbCtlSockConnect();
  return iss;
}


