/*=============================================================================
 
  Name: dbUtil.c

        dbHdlrThread -   main thread entry point
        dbprinthSupHdr   debug print routine for host super header
        dbprintVMSSupHdr debug print routine for VMS super header
        dbVMStohSupHdr   convert super header from VMS to host format 
        dbhtoVMSSupHdr   convert super header from host to VMS
        dbVMStohST0      convert ST0 block from VMS to host format

  Abs:  Waits at message queue to receive inter-thread messages.
        Sends "database service" (DBS) type messages to DBEX port 
        on proxy server for routing to VMS Control System's DBEX process.

        Reference web pages on LCLS Controls | Facilities |SLC-Aware IOC | :
                        Database Service (DBS) Design Document 
                        Functional Requirements | SLC Database Service (DBS)
                        Database Input/Output Utilities
                        C Coding Standard 
        Reference RMX_DBS dbmain.c 

  Auth:30 AUG 2004, drogind created 
  Rev: 16 DEC-2004, drogind modified prototype code to production standards
 
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*----------------------------------------------------------------------------- 
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
#include <stdio.h>
int dbUtilFlag = DP_ERROR;
#endif
#include     <stdio.h>
#include     <string.h>    /* memcpy */

/* slc global vars */
#include     "slcGlob.h"
#include     "msgTypes.h"

/* slc common includes */
#include     "cvtUtil.h"   /* slc conversion utilities */
#include     "slcCmlog.h"  /* slcCmlogLogMsg */
#include     "slcUtil.h"   /* slcGetIOCName */ 
#include     "slcMsg.h"    /* DB_ACK_DBEX... */

/* dbs includes  */  
#include     "db_micro_struc.h"
#include     "dbUtil.h" 
#include     "dbGlob.h"
#include     "dbdef.hc"   

/*=============================================================================
 
  Name: dbprintSupTypeHdr

  Abs:  Debug routine to print the host supertype header
  
  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	const char *        bufname       R        input print message
	const dbsuptypehdr_ts *sbuf_ps    R        pointer to supertype header
	const int           bytecnt       R        bytecount to print

  Rem:  print the host supertype block pointed to by pointer, along with contents of
        bufname and bytecnt

  Side: 
 
  Ret:  void
  
=============================================================================*/
void dbprinthSupHdr (const char *bufname, const dbsuptypehdr_ts *sbuf_ps, const int bytecnt)
{
  
  char *micr;  /* since microname is stored here, convert to string */
  micr = (char *)&sbuf_ps->micr;
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("DB UTIL: %s\n", bufname));
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tbyte count : %d\n", bytecnt));
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tnetwork:  %u\n", sbuf_ps->network));
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tnetlsn:  %u\n", sbuf_ps->netlsn));
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tid:  %x\n", sbuf_ps->id));
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tlen:  %ld\n", sbuf_ps->len));
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tmicr:  %s\n", micr));
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tblkbeg:  %ld\n", sbuf_ps->blkbeg));
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tsegbeg:  %ld\n", sbuf_ps->segbeg));
    
}
/*=============================================================================
 
  Name: dbprintSupTypeHdr

  Abs:  Debug routine to print the incoming VMS supertype header
  
  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	const char *        bufname       R        input print message
	const dbsuptype_tu  *sbuf_pu      R        pointer to supertype header
	const int           bytecnt       R        bytecount to print

  Rem:  print the VMS supertype block pointed to by pointer, along with contents of
        bufname and bytecnt

  Side: 
 
  Ret:  void
  
=============================================================================*/
void dbprintVMSSupHdr (const char *bufname, const dbsuptype_tu *sbuf_pu, const int bytecnt)
{
  /*  char *micr;          since microname is stored here, convert to string */
  unsigned short network, netlsn, id;
  unsigned long len, lmicr, blkbeg, segbeg;

  DEBUGPRINT(DP_INFO,dbUtilFlag, ("DB UTIL: %s\n", bufname));
  memcpy(&network,&sbuf_pu->datw[0],2);
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tnetwork:  %u\n", network));
 
  memcpy(&netlsn,&sbuf_pu->datw[1],2);
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tnetlsn:  %u\n", netlsn));
 
  memcpy(&id,&sbuf_pu->datw[2],2);
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tid:  %x\n", id));

  memcpy(&len,&sbuf_pu->datw[3],4);
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tlen:  %ld\n", len));

  memcpy(&lmicr,&sbuf_pu->datw[5],4);
  /*micr = (char *)lmicr;*/
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tmicr:  %lx\n", lmicr));

  memcpy(&blkbeg,&sbuf_pu->datw[7],4);
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tblkbeg:  %ld\n", blkbeg));

  memcpy(&segbeg,&sbuf_pu->datw[9],4);  
  DEBUGPRINT(DP_INFO,dbUtilFlag, ("\tsegbeg:  %ld\n", segbeg));
    
}
/*=============================================================================
 
  Name: dbVMStohSupType

  Abs:  Convert incoming VMS superblock to native format

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	const dbsuptype_tu    *supblk_pu      R        pointer to VMS supertype header
	const dbsuptypehdr_ts *suphdr_ps      W        pointer to native recipient header        

  Rem:  Convert the input VMS supertype block supblk_tu and output to native suphdr_ps
        by swapping short and long words from little endian to big endian, when
        implemented on a big endian architecture.

  Side: Long data falling on even 4 byte boundaries is now accessed as long, without a
        memcpy to a temp long var occurring. If the order or type of the header data 
        ever changes causing the long data to be at an odd offset within the stream, 
        this code must be changed (to copy to temp long var, swapped, memcpy'ed back to stream). 
        The stream can be accessed via word offsets on odd boundaries. 
 
  Ret:  integer byte count of bytes converted.  
=============================================================================*/
    
int dbVMStohSupHdr(const dbsuptype_tu *const supblk_pu,dbsuptypehdr_ts *suphdr_ps)
{
  int bytecnt = 0;
  /* 1st 3 struc members are unsigned short */
  cvtSSwapMove(&supblk_pu->datw[0], &suphdr_ps->network); 
  cvtSSwapMove(&supblk_pu->datw[1], &suphdr_ps->netlsn);
  cvtSSwapMove(&supblk_pu->datw[2], &suphdr_ps->id );
  /* last 4 struc members are unsigned long */
  cvtLSwapMove(&supblk_pu->datw[3], &suphdr_ps->len);
  /* don't swap the 4-byte micro string */
  memcpy(&suphdr_ps->micr, &supblk_pu->datw[5], 4);
  cvtLSwapMove(&supblk_pu->datw[7], &suphdr_ps->blkbeg);
  /* VMS blkbeg is 2 bytes (1 word) shorter; so decrement to point
     to correct starting point for data in incoming data stream */
  cvtLSwapMove(&supblk_pu->datw[9], &suphdr_ps->segbeg);
  bytecnt = 22;
  return bytecnt;  
}

/*=============================================================================

  Name: htoVMSupHdr

  Abs:  Put host db super type header passed as input arg into VMS byte-order
        and output 

   Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	const dbsuptypehdr_ts *suphdr_ps      R        pointer to native supertype header   
	const dbsuptype_tu    *supblk_pu      W        pointer to converted VMS supertype header

  Rem:  Convert the input host supertype block suphdr_ts and output to VMS supblk_pu
        by swapping short and long words from big endian to little endian, when
        implemented on a big endian architecture.

  Side: 

  Ret:  integer byte count of bytes converted

==============================================================================*/        
int dbhtoVMSSupHdr (const dbsuptypehdr_ts * const suphdr_ps, dbsuptype_tu * supblk_pu)
{
  int bytecnt = 0;
  /* 1st 3 struc members are unsigned short */
  cvtSSwapMove(&suphdr_ps->network, &supblk_pu->datw[0]);
  bytecnt += 2; 
  cvtSSwapMove(&suphdr_ps->netlsn, &supblk_pu->datw[1]);
  bytecnt += 2;   
  cvtSSwapMove(&suphdr_ps->id, &supblk_pu->datw[2]);
  bytecnt += 2; 
  /* last 4 struc members are unsigned long */
  cvtLSwapMove(&suphdr_ps->len, &supblk_pu->datw[3]);
  bytecnt += 4; 
  memcpy (&supblk_pu->datw[5], &suphdr_ps->micr, 4); /*string - don't swap*/
  bytecnt += 4;
  cvtLSwapMove(&suphdr_ps->blkbeg, &supblk_pu->datw[7]);
  bytecnt += 4;  
  cvtLSwapMove(&suphdr_ps->segbeg, &supblk_pu->datw[9]);
  bytecnt += 4;
  return bytecnt;
}
/*=============================================================================
 
  Name: dbVMStohDiag

  Abs:  Convert incoming VMS diagnostic message - both supertype header and data
        that follows, to native format

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	const dbsuptype_tu    *supblk_pu      R        pointer to VMS supertype header
	const dbsuptypehdr_ts *suphdr_ps      W        pointer to native recipient header        

  Rem:  Convert the input VMS supertype block supblk_tu and output to native suphdr_ps
        by swapping short and long words from little endian to big endian, when
        implemented on a big endian architecture.

  Side:
 
  Ret:  integer byte count of bytes converted (unpadded) 
=============================================================================*/
int dbVMStohDiag (const dbsuptype_tu * const supblk_pu, dbsuptypehdr_ts * suphdr_ps)
{
  int bytecnt = 0;
  /* 1st 3 struc members are unsigned short */
  cvtSSwapMove(&supblk_pu->datw[0], &suphdr_ps->network); 
  cvtSSwapMove(&supblk_pu->datw[1], &suphdr_ps->netlsn);
  cvtSSwapMove(&supblk_pu->datw[2], &suphdr_ps->id);
  /* last 4 struc members are unsigned long */
  cvtLSwapMove(&supblk_pu->datw[3], &suphdr_ps->len);
  /* don't swap the 4-byte micro string */
  memcpy(&suphdr_ps->micr, &supblk_pu->datw[5], 4);
  cvtLSwapMove(&supblk_pu->datw[7], &suphdr_ps->blkbeg);
  /* VMS blkbeg is 2 bytes (1 word) shorter; so decrement to point
     to correct starting point for data in incoming data stream */
  cvtLSwapMove(&supblk_pu->datw[9], &suphdr_ps->segbeg);
  bytecnt = 22;
  return bytecnt;  
}

/*=============================================================================
  Name: VMStohST0Blk

  Abs:  VMStohST0Blk transforms incoming supertype 0 structure to native
        by swapping words/long words from little endian to big endian

  Args: pointer to dbsuptypehdr_ts structure to transform  

  Rem:  Rules for architecture independence:
        1) The ST0 must remained packed (like VMS sent it) as links point to 
        offsets within the same block. 
        2) Data of type long within byte stream that doesn't start on a 4 byte 
        boundary must be indexed via word (or byte) offset, the 4 bytes mem-copied into 
        a temp long, the temp long is swapped for endianness, and the temp long 
        copied back into the byte stream. 
        3) Due to padding and byte alignment issues, structures overlaid on top of 
        the byte stream (like is done with dirtyzero.c) will not work due to architecture
        byte alignment differences. 

  Side: Reads and writes to memory pointed to by dbnodes_a[0] 

  Ret:  byte count (always unpadded)

==============================================================================*/
int dbVMStohST0 (void)
{
  unsigned short hshl = 0; 
  int4u off_hctbl = 0;
  int2u i;
  int debugcnt =0;

  dbsuptype_tu *supblk;        /* pointer to supertype header */
  unsigned short *upsblk;      /* pointer to a upsblk structure */
  char *subblk;                /* pointer to a subblk structure */
  unsigned short *offs_ptr;    /* offset ptr within ups blk */

  int2u s;   /* subblk loop index */
  int2u cnt; /* subblk item count */
  unsigned long maxlen;        /* length of ST0 block */
  unsigned long curCnt;        /* current word index into ST0 */
  unsigned long offset;        /* supertype word offset for current ups */
  unsigned long longword;      /* temp storage for long words */
  vmsstat_t iss = DB_BADST0CVT;

  supblk = dbnodes_a[0];
  DEBUGPRINT(DP_DEBUG,dbUtilFlag,("DBUTIL: Processing ST0 to swap ints and longs ...\n"));
  /* 1st 3 struc members are unsigned short */
  cvtSSwap(&supblk->datw[0]);           /* network */
  cvtSSwap(&supblk->datw[1]);           /* netlsn */
  cvtSSwap(&supblk->datw[2]);           /* id */  

  /* next 4 struc members are unsigned long */
  /* which happen to fall on 4 byte boundary - so the cast to long works ok */
  /* if hdr is ever changed s t long falls on odd boundary, must then */
  /*  index in via words, memcpy to temp, swap temp for endianness, */
  /*  and then store back */
  cvtLSwap((const unsigned long *)&supblk->datw[3]);           /* len */
  memcpy (&maxlen,&supblk->datw[3],4); 
  /* don't swap the 4-byte micro string (datw[5])*/
  cvtLSwap((const unsigned long *)&supblk->datw[7]);           /* blkbeg */
  cvtLSwap((const unsigned long *)&supblk->datw[9]);           /* segbeg */

  /*printVMSSupHdr ("ST 0 header", supblk, sizeof(supblk));*/
  /* hshl */
  cvtSSwap(&supblk->datw[OFF_ST0_HSHL]);
  hshl = supblk->datw[OFF_ST0_HSHL];
  /*DEBUGPRINT(DP_DEBUG,dbUtilFlag,("\t Swap array of ptrs to listheads; hshl length = %d \n", hshl));*/
  /* cvtLSwap array of ptrs to listheads */
  for (i=0;i<hshl;i++)
    {
      /* word offset in st0 for each hash offset */
      cvtLSwap((const unsigned long *)&supblk->datw[OFF_ST0_HTBL + (i*2)]); 
    }
  /* word offset in st0 for beg of hash collision count table */
  off_hctbl = OFF_ST0_HTBL + (hshl * 2); 
  /*DEBUGPRINT(DP_DEBUG,dbUtilFlag,("\t Swap array of collision counts; offset to cnts = %ld\n", off_hctbl));*/
  /* cvtSSwap array of collision counts of nodes in lists*/
  for (i=0;i<hshl;i++)
    {
      cvtSSwap(&supblk->datw[off_hctbl + i]);
    }

  /* get first offset to first ups block (from top of superblock 0) */
  offset = off_hctbl + hshl; /* master offset into ST 0 */
  /*DEBUGPRINT(DP_DEBUG,dbUtilFlag,("\t Initial offset for upsblk ptr = %ld \n", offset));
    DEBUGPRINT(DP_DEBUG,dbUtilFlag,("\t Swap upsblk & subblks...\n"));*/
  while (epicsTrue)
    {
      debugcnt++;
      /*DEBUGPRINT(DP_DEBUG,dbUtilFlag,("\t New offset for upsblk ptr = %ld \n", offset));
		DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t UPS BLK: \n"));  */
      /* Point to top of  ups structure */
      upsblk = ((unsigned short *)supblk)+ offset;
      /*DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t upsblk ptr = 0x%p\n",upsblk));*/

      cvtLSwap ((const unsigned long *)upsblk);  /* flink */
      /*memcpy (&longword, upsblk, 4);         
		DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t longword flink = %ld\n", longword));*/

      offs_ptr = upsblk + OFF_UPS_NEXTP;    /* nextp */
      cvtLSwap ((const unsigned long *)offs_ptr);
      /*memcpy (&longword, offs_ptr, 4); 
      DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t upsblk ptr to nextp = 0x%p\n",offs_ptr));
      DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t longword nextp = %ld\n", longword));*/
      
      offs_ptr = upsblk + OFF_UPS_NEXTC;    /* nextc */
      cvtLSwap ((const unsigned long *)offs_ptr);
      /*memcpy (&longword, offs_ptr, 4);
      DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t upsblk ptr to nextc = 0x%p\n",offs_ptr));
      DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t longword nextc = %ld\n", longword));*/

      offs_ptr = upsblk + OFF_UPS_NLEN;      /* nlen */
      cvtLSwap ((const unsigned long *)offs_ptr); 
      /*memcpy (&longword, offs_ptr, 4);
      DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t upsblk ptr to nlen = 0x%p\n",offs_ptr));
      DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t longword nlen = %ld\n", longword));*/

      cvtSSwap (upsblk+OFF_UPS_CATN);
      DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t catn = %d\n", *(upsblk+OFF_UPS_CATN)));
      cvtSSwap (upsblk+OFF_UPS_UNIT);
      DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t unit = %d\n", *(upsblk+OFF_UPS_UNIT))); 
      cvtSSwap (upsblk+OFF_UPS_NSUB);
      /*DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t nsub = %d\n", *(upsblk+OFF_UPS_NSUB)));*/
    
      cnt = *(upsblk + OFF_UPS_NSUB); /* number of subblocks */
      /* debug */
      memcpy(&i,upsblk+OFF_UPS_CATN,2);
      DEBUGPRINT(DP_DEBUG,dbUtilFlag, 
		 ("DBUTIL: There are %d subblks in upsblk catn= %d w offset = %ld\n", cnt, i, offset));
      curCnt += SIZE_UPS;
      /* Point to first subblk structure belonging to this upsblk */
  
      subblk = (char *)(upsblk + SIZE_UPS);
      offset += SIZE_UPS;    /* update master offset */
      /* Process each subblk  */
      
      for (s=0; s<cnt; ++s)
		{   
          DEBUGPRINT(DP_DEBUG,dbUtilFlag,("\t\t subblk (char) ptr = 0x%p \n", subblk));
		  longword = (unsigned short *)subblk - (unsigned short *)supblk;
		  DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t subblk offset = %ld\n", longword ));
		  DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t subn = %d, catn = %d, unit = %d\n", 
					 (unsigned char)*subblk, i, *(upsblk+OFF_UPS_UNIT) ));
		  DEBUGPRINT(DP_DEBUG,dbUtilFlag, 
					 ("\t\t\t supertype = %d\n", (unsigned char) *(subblk+OFF_SUB_SUPN_B)));
		  DEBUGPRINT(DP_DEBUG,dbUtilFlag, 
					 ("\t\t\t slen = %d\n", (unsigned char) *(subblk+OFF_SUB_SLEN_B)));
		  DEBUGPRINT(DP_DEBUG,dbUtilFlag, 
					 ("\t\t\t fmt = %c\n", (unsigned char) *(subblk+OFF_SUB_FMT_B)));
		  cvtLSwap ((const unsigned long *)(subblk+OFF_SUB_SPTR_B));
		  memset(&longword, 0, sizeof(longword));          
		  memcpy (&longword, (subblk+4), 4);
		  DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("\t\t\t longword sptr = %ld\n", longword));

	  subblk = subblk + SIZE_SUB_BYTES;
	  offset += SIZE_SUB;
	}      
      if (offset > maxlen)
	{
	  DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("DBUTIL: ERROR - trying to access beyond ST0 memory!!!!\n"));
	  slcCmlogLogMsg(iss, "BADST0CVT", "Problem with ST0 offset conversion.\n", 
				   (char *)&dblongmicro);	
	  break;
	}
      else if (offset == maxlen)
	{
	  DEBUGPRINT(DP_DEBUG,dbUtilFlag, ("DBUTIL: Completed byte swapping ST0 memory!!!!\n"));
	  break;
	}
    }
  /* report  bytes processed */
  DEBUGPRINT(DP_DEBUG,dbUtilFlag,("DBUTIL: Processed %d ups blocks\n", debugcnt));  
  DEBUGPRINT(DP_DEBUG,dbUtilFlag,("DBUTIL: Processed %ld bytes in ST0\n", offset*2));
  return (offset*2);

}

