/*=============================================================================
 
  Name: dbcheck.c


  Abs:  Utilities which help the database service utilities 
        Reference web pages on LCLS Controls | Facilities |SLC-Aware IOC | :
                        Database Service (DBS) Design Document 
                        Functional Requirements | SLC Database Service (DBS)
                        Database Input/Output Utilities
                        C Coding Standard


  Auth: 20-JAN-2005, Debbie Rogind (DROGIND) created
  Rev:  dd-mmm-200y,
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*-----------------------------------------------------------------------------  
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
#include <stdio.h>
int dbcheckFlag = DP_ERROR;
#endif
/* c includes */
#include     <stdio.h>
#include     <stdlib.h>    /* malloc, free */
#include     <string.h>    /* memcpy */
#include     <cantProceed.h>

/* slc global vars */
#include     "slcGlob.h"

/* VMS Control system function/status codes  */
#include     "msgfunc.hc"  /* FUNC_MSG function code */
#include     "micrdef.hc"  /* MICR_EXNET_ codes */

/* slc common includes */
#include     "cvtUtil.h"   /* slc conversion utilities */
#include     "timeUtil.h"  /* slc time conversion utilities */
#include     "slcCmlog.h"  /* slcCmlogLogMsg */
#include     "slcUtil.h"   /* slcGetIOCName */ 

/* dbs includes  */  
#include     "db_micro_struc.h"
#include     "dbUtil.h" 
#include     "dbGlob.h"   
#include     "dbdef.hc"    /* DB_OKOK,,*/
#include     "dbdict.h"
#include     "freeList.h"
#include     "cantProceed.h"
/* dbdict */
#include      "dbdict.h"
/* dbget */
#include      "dbget.h"
/* dbhash */
#include      "dbhash.h"
/* dbcheck  prototype */
#include    "dbcheck.h"

/*=============================================================================
  Name: dbcheckHeader

  Abs:  This routine is called by utilities dbcheck() to check a list's header values;
        thereby checking the integrity of the list itself.

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	const dbheader_ts * const db_ps  R/W       pointer to list structure


   Rem:  The list must be passed as a pointer to a pointer so that it
         may be modified (via dbcheck() routine)
         
   Side: calls dbcheck()

   Ret:  vmsstat_t:
         DB_BADPTR          - bad pointer supplied

==============================================================================*/ 
vmsstat_t dbcheckFreeLists(void)
{ 
  vmsstat_t iss = DB_OKOK;

  if ( (!dblistSmFreeList_p) || (!dblistBigFreeList_p) )
	iss=DB_BADPTR;
  return iss;
}
/*=============================================================================
  Name: dbcheckWithWait

  Abs:  This routine is called by utilities dblist()and dblunits() in order to 
        wait, if necessary, for the database to be downloaded

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	const dbheader_ts **const db_pps  R/W    pointer to pointer to structure


   Rem:  The list must be passed as a pointer to a pointer so that it
         may be modified (via dbcheck() routine)
         Caller must determine how many elements it needs

   Side: calls dbcheck()

   Ret:  vmsstat_t:
         DB_BADPTR          - bad pointer supplied
         DB_NOHEAP          - insufficient HEAP memory available
         DB_BADVERS         - DEFINE: list version and current version don't match 

==============================================================================*/ 
vmsstat_t dbcheckWithWait (const dbheader_ts ** const db_pps)
{ 
  vmsstat_t iss = DB_OKOK;
  
/*----------------------------------------------------------------------------*/
  DEBUGPRINT(DP_DEBUG,dbcheckFlag, ("dbcheckWithWait\n")); 
  if (!slcGetDbExists()) 
    {
      /* wait on dbDownloadEvent_ps forever */
	  if (dbDownloadEvent_ps)
		epicsEventMustWait(dbDownloadEvent_ps);
	  else {
		return (iss=DB_BADPTR);
	  }
    }
  iss = dbcheck(db_pps); 
  return(iss); 
}
/*=============================================================================
  Name: dbcheck

  Abs:  This routine is called by utilities dblget(), dblput(), dbCheckWithWait()
        who no longer need to wait on the database to be downloaded.

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	const dbheader_ts **const db_pps  R/W    pointer to pointer to list structure


   Rem:  The list must be passed as a pointer to a pointer so that it
         may be modified (via dbcheck() routine)
         
   Side: calls dbcheck()

   Ret:  vmsstat_t:
         DB_BADPTR          - bad pointer supplied
         DB_NOHEAP          - insufficient HEAP memory available
         DB_BADCONTEXT      - DEFINE: list version and current version don't match 

==============================================================================*/ 
vmsstat_t dbcheck(const dbheader_ts ** const db_pps)
{ 
  vmsstat_t iss = DB_OKOK;
  
/*----------------------------------------------------------------------------*/
  DEBUGPRINT(DP_INFO,dbcheckFlag,("dbcheck\n" ));

  if (!db_pps)
    {
      /* log message; dbcheckHeader */
      return (iss = DB_BADPTR);      
    }
  if (!*db_pps)
	{
	  DEBUGPRINT(DP_INFO,dbcheckFlag,("dbcheck..calling dblistalloc() .\n" ));
	  if (!SUCCESS(iss = dblistalloc(1,4, db_pps)))
		{
		  DEBUGPRINT(DP_INFO,dbcheckFlag,("dbcheck: unsuccessful with dblistalloc; return \n" ));
		  return (iss);
		}
	}
  DEBUGPRINT(DP_INFO,dbcheckFlag,("dbcheck..db_pps =  %p; *db_pps returned = %p .\n", db_pps, *db_pps ));  
  if (SUCCESS(iss = dbcheckHeader(*db_pps, epicsFalse))) iss=dbcheckExists();    
  return(iss); 
}
/*=============================================================================
  Name: dbcheckHeader

  Abs:  This routine is called by utilities dbcheck() to check a list's header values;
        thereby checking the integrity of the list itself.

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	const dbheader_ts * const db_ps  R/W       pointer to list structure


   Rem:  The list must be passed as a pointer to a pointer so that it
         may be modified (via dbcheck() routine)
         
   Side: calls dbcheck()

   Ret:  vmsstat_t:
         DB_BADPTR          - bad pointer supplied

==============================================================================*/ 
vmsstat_t dbcheckHeader(const dbheader_ts * const db_ps, 
						const epicsBoolean errOnZeroCur)
{ 
  vmsstat_t iss = DB_OKOK;
/*----------------------------------------------------------------------------*/

  DEBUGPRINT(DP_DEBUG,dbcheckFlag, ("dbcheckHeader\n")); 
  if (!(SUCCESS(iss=dbcheckFreeLists()))) return iss;
  if (!db_ps)
    {
      /* log message */
      return (iss = DB_BADPTR);      
    }
  if (!SUCCESS(iss=dbcheckExists()))
    {
	  DEBUGPRINT(DP_ERROR,dbcheckFlag, ("dbcheckHeader: db no exist; return error \n")); 
      return (iss = DB_BADPTR);       
    }
  if (db_ps->cur > db_ps->max)
    {
      /* log message */
	  DEBUGPRINT(DP_ERROR,dbcheckFlag, ("dbcheckHeader: cur > max; return error \n")); 
      return (iss = DB_BADPTR);       
    }
  if (errOnZeroCur)
	/* error if list has zero contents, such as with a dblist of dblget or dblput*/
	{
	  if (db_ps->cur ==0) {
		DEBUGPRINT(DP_ERROR,dbcheckFlag, ("dbcheckHeader: cur=0; return error \n"));
 		return(iss=DB_BADPRIM);
	  }
		
	}
  /* dbheader_ts field dbVersion_a is being deprecated to be compatible with FECC */
  /*(iss=dbcheckVersion((char *)&db_ps->dbVersion_a);*/
  return (iss);
}

/*=============================================================================
  Name: dbcheckPrim

  Abs:  This routine is called by utilities dblist()and dblunits()(whose output
        argument is a list) to verify that a primary entry exists, and to allocate
        space for a list, if it hasn't been allocated already.

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
        const char * const   prim_a        R        ptr to name of primary
        dbheader_ts **const  db_pps        R/W      ptr to ptr to data list structure
        dbhEntry_ts **       ptr_pps       R/W      ptr to ptr to prim dict entry

   Rem:  Checks pointers passed in, as well as primary name.
         The list must be passed as a pointer to a pointer so that it
         may be modified (via dblistalloc() routine)
         The dict entry (ptr_pps)must be passed as a pointer to pointer so
         the pointer val may be populated via dbhFind()

   Side: May call dblistalloc() if list is empty

   Ret:  vmsstat_t:
         DB_BADPTR          - bad pointer supplied
         DB_BADPRIM         - bad primary name supplied
         DB_NOHEAP          - insufficient HEAP memory available

==============================================================================*/ 
vmsstat_t dbcheckPrim(const char * const prim_a, 
					  const dbheader_ts ** const db_pps, 
					  dbhEntry_ts ** dbhEntry_pps)

{ 
  vmsstat_t iss = DB_OKOK;
  /*----------------------------------------------------------------------------*/
  
  DEBUGPRINT(DP_DEBUG,dbcheckFlag, ("dbcheckPrim\n")); 
  if ( (!db_pps)||(!dbhEntry_pps) ) {
    /* log error */
    return (iss=DB_BADPTR);
  }
  *dbhEntry_pps = dbhFind(dbdict_p,(char *) prim_a, SZ_NAME_SGL);
  if (!*dbhEntry_pps) {
	DEBUGPRINT(DP_INFO,dbcheckFlag, ("dbcheckPrim: primary not found - return\n")); 
    return (iss=DB_BADPRIM);
  }  
  if (!*db_pps){
    /* default to requesting 1 element, 4 bytes long */
	DEBUGPRINT(DP_INFO,dbcheckFlag, ("dbcheckPrim: list empty - return\n")); 
	return (iss = DB_BADPTR);       
  }
  return(iss); 
}
/*=============================================================================
  Name: dbcheckExists

  Abs:  This routine returns an error if the database does not exist

  Args: None

   Rem:  

   Side: Reads global var dbExists

   Ret:  vmsstat_t:
         DB_BADPTR          - bad pointer supplied
         DB_BADPRIM         - bad primary name supplied
         DB_NOHEAP          - insufficient HEAP memory available

==============================================================================*/ 
vmsstat_t dbcheckExists (void)
{ 
  vmsstat_t iss = MICR_OKOK;
  
/*----------------------------------------------------------------------------*/
  DEBUGPRINT(DP_DEBUG,dbcheckFlag, ("dbcheckExists\n")); 
  if (!slcGetDbExists())
    { 
	  DEBUGPRINT(DP_ERROR,dbcheckFlag, ("dbcheckVersion: db doesn't exist!\n"));
      iss = DB_NOSERVICE;
    }
  return(iss); 
}
  /* dbheader_ts field dbVersion_a is being deprecated to be compatible with FECC */
/*=============================================================================
  DEPRECATED
  Name: dbcheckVersion

  Abs:  This routine returns an error if the db version contained in the list
        header does not match the current global dbVersion_a.

  Args: None

   Rem:  

   Side: Reads global var dbVersion_a

   Ret:  vmsstat_t:
         DB_BADPTR          - bad pointer supplied
         DB_BADPRIM         - bad primary name supplied
         DB_NOHEAP          - insufficient HEAP memory available

==============================================================================*/ 
/*LOCAL vmsstat_t dbcheckVersion(const char * version_a)
{ 
  vmsstat_t iss = MICR_OKOK;
  

  DEBUGPRINT(DP_DEBUG,dbcheckFlag, ("dbcheckVersion\n")); 
  if (strncmp(dbVersion_a, version_a, DBCURVERSIONBC)!=0)
    {
	  DEBUGPRINT(DP_ERROR,dbcheckFlag, 
				 ("dbcheckVersion: list ver=%s != cur ver=%s; error\n", version_a, dbVersion_a)); 
      iss = DB_BADVERSION;
	  slcCmlogLogMsg(iss, "BADVERSION", "%s has list vers=%s; cur ver = %s.\n", 
					  epicsThreadGetNameSelf(),version_a, dbVersion_a);
    }

  return(iss); 
} */
