/*=============================================================================
 
  Name: dbdict.c
        dbdictInitFreeLists 
		dbdictDelFreeLists 
		dbdictInsP 
        dbdictInsAllUnits
        dbdictInsSecnInAllUnits 
		dbdictInsPUS             local 

  Abs:  Utilities for dictionary useage
        Reference web pages on LCLS Controls | Facilities |SLC-Aware IOC | :
                        Database Service (DBS) Design Document 
                        Functional Requirements | SLC Database Service (DBS)
                        Database Input/Output Utilities
                        C Coding Standard
			General Purpose Utilities

  Auth: 1-DEC-2004, Debbie Rogind (DROGIND) created
  Rev:  dd-mmm-200y,
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*-----------------------------------------------------------------------------  
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
#include <stdio.h>
int dbdictFlag = DP_ERROR;
#endif

/* c includes */
#include     <stdio.h>
#include     <stdlib.h>    /* malloc, free */
#include     <string.h>    /* memcpy */
#include     <cantProceed.h>

/* slc global vars */
#include     "slcGlob.h"

/* VMS Control system function/status codes  */
#include     "msgfunc.hc"  /* FUNC_MSG function code */
#include     "micrdef.hc"  /* MICR_EXNET_ codes */

/* slc common includes */
#include     "cvtUtil.h"   /* slc conversion utilities */
#include     "timeUtil.h"  /* slc time conversion utilities */
#include     "slcCmlog.h"  /* slcCmlogLogMsg */
#include     "slcUtil.h"   /* slcGetIOCName */ 

/* dbs includes  */  
#include     "db_micro_struc.h"
#include     "dbUtil.h" 
#include     "dbmsgmail.h" 
#include     "dbGlob.h"   
#include     "dbdef.hc"    /* DB_OKOK,,*/
#include     "dbdict.h"
#include     "freeList.h"
#include     "dbhash.h"
#include     "dblist.h"
#include     "dbST0.h"
#include     "dbget.h"
/* dbdict */
#include      "dbdict.h"

LOCAL dbhEntry_ts * dbdictInsPUS (const char * prim_a, const char *unit_a, const char *secn_a, 
			    const short supn, const short count, 
			    const long sptr, const short width, const char fmt);

/*-----------------------------------------------------------------------
  Name: dbdictInitFreeLists 

  Abs:  Initializes the memory management free lists for all structures used in
        the dictionary. 

  Args: None

  Rem:  utilizes epics libCom freeListLib

  Side: 

  Ret:  void
----------------------------------------------------------------------*/

void dbdictInitFreeLists ( void)
{
  vmsstat_t iss = DB_BADDICT;

  if (!dbdictNameTrpFreeList_p) {
    freeListInitPvt(&dbdictNameTrpFreeList_p, sizeof(dbdictNameTriplet_ts),NENTRIES);
    if (!dbdictNameTrpFreeList_p) {
      slcCmlogLogMsgFile(iss, "BADDICT",__FILE__, __LINE__, "Freelist init failure.\n");
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInitFreeLists failure for dbdictNameTrpFreeList_p\n"));
    }
  }
  if (!dbdictNameSglFreeList_p) {
    freeListInitPvt(&dbdictNameSglFreeList_p, sizeof(dbdictNameSgl_ts),NENTRIES);
    if (!dbdictNameSglFreeList_p) {
      slcCmlogLogMsgFile(iss, "BADDICT",__FILE__, __LINE__, "Freelist init failure.\n");
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInitFreeLists failure for dbdictNameSglFreeList_p\n"));
    }
  }
  if (!dbdictPUSFreeList_p) {
    freeListInitPvt(&dbdictPUSFreeList_p, sizeof(dbdictPUS_ts),NENTRIES);
    if (!dbdictPUSFreeList_p) {
      slcCmlogLogMsgFile(iss, "BADDICT",__FILE__, __LINE__, "Freelist init failure.\n");
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInitFreeLists failure for dbdictPUSFreeList_p\n"));
    }
  }
  if (!dbdictPFreeList_p) {
    freeListInitPvt(&dbdictPFreeList_p, sizeof(dbdictP_ts),NENTRIES);
    if (!dbdictPFreeList_p) {
      slcCmlogLogMsgFile(iss, "BADDICT",__FILE__, __LINE__, "Freelist init failure.\n");
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInitFreeLists failure for dbdictPFreeList_p\n"));
    }
  }
  if (!dbdictUFreeList_p) {
    freeListInitPvt(&dbdictUFreeList_p, sizeof(dbdictU_ts),NENTRIES);
    if (!dbdictUFreeList_p) {
      slcCmlogLogMsgFile(iss, "BADDICT",__FILE__, __LINE__, "Freelist init failure.\n");
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInitFreeLists failure for dbdictUFreeList_p\n"));
    }
  }  
  if (!dbdictSFreeList_p) {
    freeListInitPvt(&dbdictSFreeList_p, sizeof(dbdictS_ts),NENTRIES);
    if (!dbdictSFreeList_p) {
      slcCmlogLogMsgFile(iss, "BADDICT",__FILE__, __LINE__, "Freelist init failure.\n");
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInitFreeLists failure for dbdictSFreeList_p\n"));
    }
  } 
}
/*-----------------------------------------------------------------------
  Name: dbdictDelFreeLists 

  Abs:  Frees the memory management free lists for all structures used in
        the dictionary. 

  Args: None

  Rem:  utilizes epics libCom freeListLib

  Side: 

  Ret:  void
----------------------------------------------------------------------*/
void dbdictDelFreeLists (void)
{
  void * temp_p;
  
  if (dbdictNameTrpFreeList_p) {
	temp_p = dbdictNameTrpFreeList_p;
	dbdictNameTrpFreeList_p = NULL;    
    freeListCleanup(temp_p);
  } else
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictDelFreeLists failure for dbdictTrpFreeList_p\n"));
  DEBUGPRINT(DP_INFO,dbdictFlag,("dbdictDelFreeLists: dbdictNameSglFreeList_p = %p\n",dbdictNameSglFreeList_p));
  if (dbdictNameSglFreeList_p) {
	temp_p = dbdictNameSglFreeList_p;
	dbdictNameSglFreeList_p = NULL;
    freeListCleanup(temp_p);
  } else
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictDelFreeLists failure for dbdictSglFreeList_p\n"));

  DEBUGPRINT(DP_INFO,dbdictFlag,("dbdictDelFreeLists: dbdictPUSFreeList_p = %p\n",dbdictPUSFreeList_p));
  if (dbdictPUSFreeList_p) {
	temp_p = dbdictPUSFreeList_p;
	dbdictPUSFreeList_p = NULL;
    freeListCleanup(temp_p);
  } else
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictDelFreeLists failure for dbdictPUSFreeList_p\n"));
  DEBUGPRINT(DP_INFO,dbdictFlag,("dbdictDelFreeLists: dbdictPFreeList_p = %p\n",dbdictPFreeList_p));
  if (dbdictPFreeList_p) {
	temp_p = dbdictPFreeList_p;
	dbdictPFreeList_p = NULL;
    freeListCleanup(temp_p);
  } else
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictDelFreeLists failure for dbdictPFreeList_p\n"));

  DEBUGPRINT(DP_INFO,dbdictFlag,("dbdictDelFreeLists: dbdictUFreeList_p = %p\n",dbdictUFreeList_p));
  if (dbdictUFreeList_p) {
	temp_p = dbdictUFreeList_p;
	dbdictUFreeList_p = NULL;
    freeListCleanup(temp_p);
  } else
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictDelFreeLists failure for dbdictUFreeList_p\n"));

  DEBUGPRINT(DP_INFO,dbdictFlag,("dbdictDelFreeLists: dbdictSFreeList_p = %p\n",dbdictSFreeList_p)); 
  if (dbdictSFreeList_p) {
	temp_p = dbdictSFreeList_p;
	dbdictSFreeList_p = NULL;
    freeListCleanup(temp_p);
  } else
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictDelFreeLists failure for dbdictSFreeList_p\n"));
}


/*-----------------------------------------------------------------------
  Name: dbdictInsPUS

  Abs:  Makes a primary unit secondary (PUS) entry in the dictionary.
        Each dictionary entry requires a pointer to a name and a pointer to data.


  Args: Type		   Name	     Access	   Description
	---------------   --------  ---------- ----------------------------
        const char *        prim_a      R     pointer to 4 char primary name (from PRIMARY.MAP)
        const char *        unit_a      R     pointer to 4 char unit name 
	                                      (unit # from dbST0units - converted to ASCII)
        const char *        secn_a      R     pointer to 4 char secondary name (from PRIMARY.MAP)
        const short         supn        R     supertype number stored in ST0 for this PUS
        const short         count       R     how many elements (length)for this PUS stored in ST0
        const long          sptr        R     secondary data offset into supertype number block
        const short         width       R     VMS datum width of data (from PRIMARY.MAP)
        const char          fmt         R     format char (I,A,R,Z,S,T)

  Rem:  For name, it gets a dbdictNameTriplet_t structure from its approp. free list,
        concatenates each of the 4 char input substrings (prim, unit, secn) into 12 chars 
        to store in dbdictNameTriplet_t and passes pointer to dbhAdd().

	For data, it gets a dictPUS_t structure from its approp. free list,
        stores the incoming arguments in the structure and passes pointer to dbhAdd().
 
  Side: uses dbhash facility adapted from epics libCom gphash

  Ret:  pointer to prim/unit/secn dbhEntry_ts hash entry 
----------------------------------------------------------------------*/
LOCAL dbhEntry_ts * dbdictInsPUS (const char *prim_a, const char *unit_a, const char *secn_a, 
								  const short supn, const short count, 
								  const long sptr, const short width, const char fmt)
{ 
  dbhEntry_ts * dbdict_ps;
  dbdictPUS_ts * pus_ps = NULL;
  dbdictNameTriplet_ts * name_ps = NULL;

  pus_ps = freeListCalloc (dbdictPUSFreeList_p); 
  if (!pus_ps){
    slcCmlogLogMsgFile(DB_BADDICT, "BADDICT",__FILE__, __LINE__, "dbdictInsPUS unable to malloc dbdictPUS buffer from freelist.\n");
    return (NULL);
  } 
  name_ps = freeListCalloc (dbdictNameTrpFreeList_p);
  if (!name_ps){
    slcCmlogLogMsgFile(DB_BADDICT, "BADDICT",__FILE__, __LINE__, "dbdictInsPUS unable to malloc dbdictNameTrp buffer from freelist.\n");
    return (NULL);
  }
  /* concatenate each of the three 4 char inputs into 12 char to use as name */
  dbgetCatPUSName(prim_a, unit_a, secn_a, (char *)name_ps); 
  DEBUGPRINT(DP_INFO,dbdictFlag,("dbdictInsPUS: %.4s %.4s %.4s concatenates to %.12s\n",
								 prim_a, unit_a, secn_a, (char *)&(name_ps[0])));
  /* store rest of meta data in dbdictPUS_ts */
  if (pus_ps) {
    pus_ps->supn = supn;
    pus_ps->count = count;
    pus_ps->sptr = sptr;
    pus_ps->width = width;
    pus_ps->format = fmt; }
  else {
    DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInsPUS: invalid pointer pus_ps \n"));
	slcCmlogLogMsgFile(DB_BADDICT, "BADDICT",__FILE__, __LINE__,
				   "dbdictInsPUS: invalid pointer pus_ps.\n" );
    return (NULL);
  }
  DEBUGPRINT(DP_DEBUG,dbdictFlag,("dbdictInsPUS: dbhAdding %s...\n", (char *)name_ps));
  dbdict_ps = dbhAdd((void *)dbdict_p, (char *)name_ps, sizeof(dbdictNameTriplet_ts) );
  if (dbdict_ps){
    dbdict_ps->usrPvt = pus_ps;
    dbdict_ps->kind_e = primUnitSecn;
  } 
  else {
    DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInsPUS: invalid or previously existing entry in dbhAdd\n"));
    slcCmlogLogMsgFile(DB_BADDICT, "BADDICT",__FILE__, __LINE__,
				   "dbdictInsPUS: invalid or previously existing entry in hash table (dbhAdd).\n" );
    return (NULL);    
  }
  return (dbdict_ps);
}
/*-----------------------------------------------------------------------
  Name: dbdictInsP

  Abs:  Makes a primary (P)- only entry in the dictionary.
        Each dictionary entry requires a pointer to a name and a pointer to data.
	For name, it gets a dbdictNameSgl_t structure from its approp. free list,
        stores, and passes pointer to dbhAdd().

	For data, it passes a pointer to a dictP_ts structure (which is a ELLLIST), 
        which = NULL if there are no units associated with this primary (as indicated
        by input arg hasunits).

  Args: Type		   Name	     Access	   Description
	---------------   --------  ---------- ----------------------------  
        const char *       prim_a     R        pointer to 4 char primary name (from PRIMARY.MAP)
        const epicsBoolean hasunits   R        flag whether primary has ass'ed units,
	                                       if so, prim entry points to a units elllist head
                                               if not, ponts to null 

  Rem:  

  Side: NOTE: 

  Ret:  pointer to dbhEntry_ts hash entry 
----------------------------------------------------------------------*/
dbhEntry_ts * dbdictInsP(const char *prim_a, const epicsBoolean hasunits )
{ 
  dbdictP_ts * p_ps = NULL;
  dbhEntry_ts * dbdictE_ps = NULL;
  dbdictNameSgl_ts * name_p = NULL;

  name_p = freeListCalloc (dbdictNameSglFreeList_p);
  if (name_p) {
    memcpy (name_p, prim_a, 4);
	DEBUGPRINT(DP_DEBUG,dbdictFlag,("dbdictInsP: calling dbhAdd \n"));
    dbdictE_ps = dbhAdd((void *)dbdict_p, (char *)name_p, sizeof(dbdictNameSgl_ts) ); 
  }
  else {
    slcCmlogLogMsgFile(DB_BADDICT, "BADDICT",__FILE__,__LINE__,
				   "dbdictInsP: unable to freeListCalloc dbdictNameSgl buffer from freelist.\n");
    DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInsP: freeListCalloc (Sgl) failed\n"));
	return (NULL);
  }
  if (dbdictE_ps){
      dbdictE_ps->kind_e = prim;   
  }
  else {
    DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInsP: dbhAdd (P) failed\n"));
    slcCmlogLogMsgFile(DB_BADDICT, "BADDICT",__FILE__, __LINE__,
				   "dbdictInsP: dbhAdd failed.\n" );
    return NULL;
  }

  /* if this primary has units,dbhEntry_ts usrPvt points to list head (dbdictP_ts) v NULL */
  if (hasunits){
    p_ps = freeListCalloc (dbdictPFreeList_p);
    if (p_ps) {    
      ellInit(&(p_ps->listUnits));
      dbdictE_ps->usrPvt = p_ps;    
    }
    else {
      slcCmlogLogMsgFile(DB_BADDICT, "BADDICT",__FILE__,__LINE__,
			 "dbdictInsP: unable to freeListCalloc dbdictP buffer from freelist.\n");
      
      DEBUGPRINT(DP_ERROR,dbdictFlag,("dbdictInsP: freeListCalloc (P) failed\n"));
      return (NULL);  
    }
  }  

  return dbdictE_ps;
}

/*-----------------------------------------------------------------------
  Name: dbdictInsertU

  Abs:  For all units in the units list, it creates a linked list from primary. 
        Unit- only entries are linked list nodes off of the primary dictionary entry.
        Each unit node serves as a linked list head for its associated secondary linked list.
        The unit node has structure dictU_ts
        
        Each dictionary entry requires a pointer to a name and a pointer to data.
        For name, it gets a dbdictNameSgl_t structure from its approp. free list,
        and passes pointer to dbhAdd().
        For data, it gets a dictU_t structure from its approp. free list,
        stores the incoming arguments in the structure and passes pointer to dbhAdd().

  Args: Type		   Name	     Access	   Description
	---------------   --------  ---------- ----------------------------  
        const dbdictP_ts * const unitList_ps    R  contains sptr names 
	const dbST0header_ts * const dbunits_ps R  contains 4 char unit names 
  
  Rem:  

  Side: NOTE: unit is blank padded at end prior to dict. entry 

  Ret:  iss
----------------------------------------------------------------------*/
vmsstat_t dbdictInsAllUnits (const dbdictP_ts * const unitList_ps, 
			     const dbST0header_ts * const dbunits_ps)
{ 

  /* loops and calls dbdictInsU*/	
  int4u      *nextU_p;                 /* next unit location in dbunits list */
  short      cnt, loopi;
  short      unit;
  dbdictNameSgl_ts * unit_ps;         /* pointer to 4 char ascii unit name */
  dbdictU_ts   *dbdictU_ps;
  vmsstat_t iss = DB_BADDICT;

  if (!( (dbunits_ps) && (unitList_ps) )) {
    slcCmlogLogMsgFile(iss, "BADDICT",__FILE__,__LINE__,
		       "dbdictInsAllUnits:  Bad input pointer.\n");	
    return (iss );
  }
  cnt = dbunits_ps ->cur;               /* count of how many units */
  nextU_p = (int4u *)dbunits_ps + 2;   /* first unit location in buffer */
  if ( ((cnt == 0) || (nextU_p)) == 0 ){
    slcCmlogLogMsgFile(iss, "BADDICT",__FILE__,__LINE__,
		       "dbdictInsAllUnits:  Bad units list.\n");
    return (iss );
  }
  /* for ea unit (in dbunits list) add to units linked list  */
  for (loopi=0; loopi<cnt; loopi++)
    {
      /* get a unit dict structure */
      dbdictU_ps = freeListCalloc (dbdictUFreeList_p);
      if (!dbdictU_ps){
		slcCmlogLogMsgFile(DB_BADDICT, "BADDICT",__FILE__,__LINE__,
			   "dbdictInsAllUnits: unable to freeListCalloc dbdictU buffer from freelist.\n");
		return (iss);
      }
      /* link into primary's unit linked list */
      ellAdd((ELLLIST *)unitList_ps, (ELLNODE *)dbdictU_ps);
      /* link in name to dbdictU_ps->name */
      unit = *nextU_p++;
      DEBUGPRINT(DP_DEBUG,dbdictFlag,("\t dbdictInstAllUnits: unit from list =%d \n",unit));  
      unit_ps = freeListCalloc (dbdictNameSglFreeList_p); 
      if (!unit_ps){
		slcCmlogLogMsgFile(DB_BADDICT, "BADDICT",__FILE__,__LINE__,
					   "dbdictInsAllUnits: unable to freeListCalloc dbdictNameSgl buffer from freelist.\n");
		DEBUGPRINT(DP_ERROR,dbdictFlag,("\t dbdictInstAllUnits: unable to freeListCalloc for unit %d; ptr=%p \n",
										unit,unit_ps )); 
		return (iss);
      } 
      if (!SUCCESS (dbgetUnitAsString(unit, (char *)unit_ps))){
		slcCmlogLogMsgFile(DB_BADDICT, "BADDICT",__FILE__,__LINE__,
						   "dbdictInsAllUnits: unsuccessful unit to string conversion.\n");
		DEBUGPRINT(DP_ERROR,dbdictFlag,("\t dbdictInstAllUnits: unsuccessful unit to string conversion for unit %d; ptr=%p \n",
										unit,unit_ps )); 
		return (iss);
      }
	  
	  dbdictU_ps->name = unit_ps; /* store ptr returned from freeListCalloc*/
	  dbdictU_ps->unit = unit;
	  DEBUGPRINT(DP_INFO,dbdictFlag,("\t dbdictInstAllUnits: ASCII unit =%.4s-hex = 0x%lx; unit# =%hd; name ptr %p\n", 
									 (char *)&(dbdictU_ps->name[0]),  *( (unsigned long *)(dbdictU_ps->name)),
									 dbdictU_ps->unit, dbdictU_ps->name)); 

	}	  
  return (iss=MICR_OKOK);
}
/*-----------------------------------------------------------------------
  Name: dbdictInsSecnInAllUnits

  Abs:  Creates secondary linked list off of its associated unit linked list head 
        (unit linked list heads are off of each primary dictionary entry).
        Each unit node serves as a linked list head for its associated secondary linked list.
        Each secondary entry in the linked list has structure dictS_ts
        
        Each dictionary entry requires a pointer to a name and a pointer to data.
        For name, it gets a dbdictNameSgl_t structure from its approp. free list,
        and passes pointer to dbhAdd().
        For data, it gets a dictU_t structure from its approp. free list,
        stores the incoming arguments in the structure and passes pointer to dbhAdd().

  Args: Type		   Name	             Access	   Description
	---------------   --------       ---------- ----------------------------  
        const dbdictP_ts * const unitList_ps    R  contains ptr to head of unit linked list
		const dbST0header_ts * const dbunits_ps R  contains 4 char unit names 
        const char *      prim_a                R  pointer to 4 char primary name (from PRIMARY.MAP)
        const char *      secn_a                R  pointer to 4 char secondary name (from PRIMARY.MAP)
		const dbST0header_ts * dblist_ps        R  list of corr. ptrs to ST0 secn subblks for each    
        const short       count                 R  how many elements (length)for this PUS stored in ST0
        const short       width                 R  VMS byte count of each element (from PRIMARY.MAP)
        const cahr        fmt                   R  format char (I,A,R,Z,S,T)
  
  Rem:  

  Side: 

  Ret:  iss
----------------------------------------------------------------------*/

vmsstat_t dbdictInsSecnInAllUnits ( const dbdictP_ts * unitList_ps, const dbST0header_ts * dbunits_ps,
									const char * prim_a,
									const char * secn_a, 
									const dbST0header_ts * dblist_ps,  
									const short width, const char fmt, const short supn,
									dbST0header_ts **const dbdata_pp) 
{/* debug struc */	
struct subblk
    {
    unsigned char  subn;                /* Subtype number */
    unsigned char  supn;                /* Object number */
    unsigned char  len;                 /* Total data word length = width*count */
    unsigned char  fmt;                 /* Data format type */
    unsigned long  offset;              /* Word offset to data */
    };
 struct subblk subblk_struc;
 /* end debug */

  ELLNODE    *unitNode_ps;              /* unit node pointer */
  int4u      *next_p;                   /* next subblk location in dblist */
  int4u      *nextU_p;                  /* next unit location in dbunits list */
  int4u      cnt,cntU, loopi;
  unsigned long sptr;                   /* sptr for subblock (from dblist) */
  dbdictNameSgl_ts * secn_ps;           /* pointer to 4 char ascii secn name */
  dbhEntry_ts  *dbdictPUSEntry_ps;
  dbdictS_ts   *dbdictS_ps;
  dbdictU_ts   *dbdictU_ps;
  vmsstat_t    iss = DB_BADDICT;
  unsigned short unit;
  /* for accessing subblk info in ST0 */
  suptype0_ts  *sb0;                   /* pointer to supertype 0 header */
  unsigned short *sec_ptr;             /* pointer to secn subblk */
  unsigned short  count=0;
  
  
  /* init pointers */
  sb0 = (suptype0_ts *)dbnodes_a[0];
  
  if (!unitList_ps || !dbunits_ps || !dblist_ps) {
    slcCmlogLogMsgFile(iss, "BADDICT",__FILE__,__LINE__,
					   "dbdictInsSecnInAllUnits:  bad input ptr.\n");
    DEBUGPRINT(DP_ERROR,dbdictFlag,("tInsSecnInAllUnits - bad input ptr - exit\n"));
    return (iss);
  }
  cntU = dbunits_ps->cur;              /* count of how many units */
  cnt = dblist_ps->cur;                /* count of how many units */
  
  DEBUGPRINT(DP_INFO,dbdictFlag,("tInsSecnInAllUnits; unit cnt = %ld, list cnt = %ld \n", cntU, cnt));        
  /*dbST0printlist ((dbST0header_ts * const)dblist_ps);*/
  next_p = (int4u *)(dblist_ps);        
  nextU_p = (int4u *)dbunits_ps;    
  next_p += 2;                         /* first sec_offset location in buffer */
  nextU_p += 2 ;                       /* first unit location in buffer */
  if ( (cnt == 0) || !(nextU_p) || !(next_p) ){
    DEBUGPRINT(DP_ERROR,dbdictFlag,("tInsSecnInAllUnits - bad list - exit\n"));      
    slcCmlogLogMsgFile(iss, "BADDICT",__FILE__,__LINE__,
					   "dbdictInsSecnInAllUnits:  bad list.\n");
    return (iss);
  }
  unitNode_ps = ellFirst((ELLLIST *)unitList_ps); /* point to first unit of this prim to start  */
  if (!unitNode_ps){
    slcCmlogLogMsgFile(iss, "BADDICT",__FILE__,__LINE__,
					   "dbdictInsSecnInAllUnits:  bad unit linked list.\n");
    DEBUGPRINT(DP_ERROR,dbdictFlag,("InsSecnInAllUnits - bad unit linked list- exit\n"));      
    return (iss );
  } 
  
  /* for ea unit (in dbunits list) add to units linked list & as a PUS hash entry */
  for (loopi=0; loopi<cnt; loopi++)
    {
      sptr = *next_p++;
      DEBUGPRINT(DP_DEBUG,dbdictFlag,("tInsSecnInAllUnits - subblk offset from list = %ld\n", sptr));
      unit = *nextU_p++;
      /* error check unit list head to ensure proper unit # */
      dbdictU_ps = (dbdictU_ts *)unitNode_ps;
      if (unit != (int4u)dbdictU_ps->unit){
		DEBUGPRINT(DP_ERROR,dbdictFlag,
				   ("\t units don't align; have %d, list is %d \n",unit, (short) dbdictU_ps->unit ));
		slcCmlogLogMsgFile(iss, "BADDICT",__FILE__,__LINE__,
						   "dbdictInsSecnInAllUnits:  units don't align; have %d, list is %d.\n", unit, (short)dbdictU_ps->unit );
		return (iss); 
      } 
      else { 
		DEBUGPRINT(DP_INFO,dbdictFlag,
				   ("\t dbdictInsSecnInAllUnits processing prim %.4s, secn %.4s, unit %.4s; name ptr %p\n",
					prim_a, secn_a, (char *)&(dbdictU_ps->name[0]), dbdictU_ps->name ));
		if (strncmp(prim_a, "STPR",4)==0){

		  DEBUGPRINT(DP_INFO,dbdictFlag,
					 ("\t\t unit name contents =0x%lx\n", *( (unsigned long *)(dbdictU_ps->name)) ) );		  
		}
      }
	  
      /* point to top of secn subblk within ST0 */
      sec_ptr =(unsigned short *)(sptr + (short *)sb0);
      /*offset = *(unsigned long *)(sec_pc + OFF_SUB_SPTR_B);*/
      memcpy (&subblk_struc,sec_ptr, sizeof(subblk_struc));
      /*if (strncmp(secn_a, "STPR",4)==0) {
      DEBUGPRINT(DP_INFO,dbdictFlag,("\t\t\t subn = %hd\n", (unsigned int) subblk_struc.subn));
      DEBUGPRINT(DP_INFO,dbdictFlag,("\t\t\t supertype = %hd\n", (unsigned int) subblk_struc.supn));
      DEBUGPRINT(DP_INFO,dbdictFlag,("\t\t\t subblk.slen = %hd\n",(unsigned int) subblk_struc.len));
      DEBUGPRINT(DP_INFO,dbdictFlag,("\t\t\t fmt = %c\n", subblk_struc.fmt));
      DEBUGPRINT(DP_INFO,dbdictFlag,("\t\t\t subblk.sptr = %ld\n", (unsigned long) subblk_struc.offset));*/
      
      /* calculate the PUS dict count field via (slen*2)/file width */
      count = (subblk_struc.len * 2)/width; 
      if (count==0) {
		slcCmlogLogMsgFile(iss, "BADDICT",__FILE__,__LINE__,
						   "dbdictInsSecnInAllUnits count= 0 for prim %.4s, secn %.4s, unit %d\n",
						   prim_a, secn_a,(short)dbdictU_ps->unit);
        DEBUGPRINT(DP_ERROR,dbdictFlag,
				   ("\t dbdictInsSecnInAllUnits count= 0 for prim %.4s, secn %.4s, unit %d\n",
					prim_a, secn_a,(short)dbdictU_ps->unit));
	  }
	  
	  /* first insert the PUS to get the ptr back. Secn node will point to it */
	  /* dbdictPUSEntry_ps = dbdictInsPUS (prim_a, (char *)dbdictU_ps->name, secn_a,
		 subblk_struc.supn, count, subblk_struc.offset, width, fmt);*/
	  dbdictPUSEntry_ps = dbdictInsPUS (prim_a, (char *)dbdictU_ps->name, secn_a,
										supn, count, subblk_struc.offset, width, fmt);
      /* now get a secn node structure to link in the secn linked list */
	  dbdictS_ps = freeListCalloc (dbdictSFreeList_p);
	  if (!dbdictS_ps){
		slcCmlogLogMsgFile(iss, "BADDICT",__FILE__,__LINE__,
						   "dbdictInsSecnInAllUnits unable to freeListCalloc dbdictS buffer from freelist.\n"); 
		return (iss);
	  }  
	  secn_ps = freeListCalloc(dbdictNameSglFreeList_p);
	  if (!secn_ps){
		slcCmlogLogMsgFile(iss, "BADDICT",__FILE__,__LINE__,
						   "dbdictInsSecnInAllUnits unable to malloc dbdictNameSgl buffer from freelist.\n");
		return (iss);
	  } 
	  memcpy(secn_ps, secn_a, 4);
	  dbdictS_ps->dbdictEntry = dbdictPUSEntry_ps;
	  dbdictS_ps->name = (dbdictNameSgl_ts *)secn_ps;
	  
	  /* now add secn node into unit ell head at the bottom*/
	  ellAdd( (ELLLIST *)&dbdictU_ps->listSecns, (ELLNODE *)&dbdictS_ps->node );
	  /* advance to next unit in unit list */
	  unitNode_ps = ellNext(unitNode_ps);    
	}
return (iss = MICR_OKOK);
}
