/*=============================================================================
 
  Name: dbdump.c
        dbdumpDump
        dbdumpDisp

  Abs:  Utilities which read and write database lists 
        Reference web pages on LCLS Controls | Facilities |SLC-Aware IOC | :
                        Database Service (DBS) Design Document 
                        Functional Requirements | SLC Database Service (DBS)
                        Database Input/Output Utilities
                        C Coding Standard


  Auth: 14-FEB-2005, Debbie Rogind (DROGIND) created
  Rev:  dd-mmm-200y,
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*-----------------------------------------------------------------------------  
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
#include <stdio.h>
  int dbdumpFlag = DP_ERROR;
#endif
/* c includes */
#include     <stdio.h>
#include     <stdlib.h>    /* malloc, free */
#include     <string.h>    /* memcpy */
#include     <cantProceed.h>

/* slc global vars */
#include     "slcGlob.h"

/* VMS Control system function/status codes  */
#include     "msgfunc.hc"  /* FUNC_MSG function code */
#include     "micrdef.hc"  /* MICR_EXNET_ codes */

/* slc common includes */
#include     "cvtUtil.h"   /* slc conversion utilities */
#include     "timeUtil.h"  /* slc time conversion utilities */
#include     "slcCmlog.h"  /* slcCmlogLogMsg */
#include     "slcUtil.h"   /* slcGetIOCName */ 

/* dbs includes  */  
#include     "db_micro_struc.h"
#include     "dbUtil.h" 
#include     "dbGlob.h"   
#include     "dbdef.hc"    /* DB_OKOK,,*/
#include     "dbdict.h"
#include     "freeList.h"
#include     "cantProceed.h"
/* dbdict */
#include      "dbhash.h"
#include      "dbdict.h"
/* dbget */
#include      "dbget.h"
/* dblist  prototypes */
#include    "dblistUtil.h"
#include    "dblist.h"
#include    "dbl.h"
#include    "dbcheck.h"
#include    "dbdump.h"
LOCAL vmsstat_t dbdumpGet (const char * const prim_a, 
						   const dbheader_ts * const dbunits_ps,
						   const dbheader_ts * const dbsecns_ps,
						   const dbheader_ts ** const dblist_pps);
LOCAL void dbdumpFormat (const void * const dbdata_p,
						 const char * const secn_p,
						 const char format,
						 const int width, 
						 const int count);
/*=============================================================================
  Name: dbdumpDump

  Abs:  Setup routine prior to dumping data to console to configure lists.


  Args:     Type		        Name	 Access	   Description
        ------------------- -----------	---------- ----------------------------
	    const char *   const   prim_pa       R     ptr to 4 ASCII chars w prim
	    const char *   const   unit_pa       R     ptr to 4 ASCII chars w units
		const char *   const   secn_pa       R     ptr to 4 ASCII chars w secns  

  Rem:  Allocates and populates unit list (list of unit(s), ea 4 ASCII chars) 
        and secondary list (list of secn(s), ea 4 ASCII chars, dep. on prim) 
		based on input arguments and database. Also allocates data list.
        Calls dbdumpDisp to retrieve from db and output to console the data 
		for each prim:unit:secn configured in unit and secn lists.
		
  Side: Memory allocation (down in list utils when list is too short)

  Ret:  vmsstat_t:

==============================================================================*/
vmsstat_t dbdumpDump (const char * const prim_pa, 
					  const char * const unit_pa,
                      const char * const secn_pa)
{
  vmsstat_t iss =MICR_OKOK; 
  dbheader_ts * dblist_ps = NULL;
  dbheader_ts * dbunits_ps = NULL;
  dbheader_ts * dbsecns_ps = NULL;
  short length = SZ_NAME_SGL;
  
  if (strncmp(unit_pa, "    ",4)!=0)
	length+=SZ_NAME_SGL;
  if (strncmp(secn_pa, "    ",4)!=0)
	length+=SZ_NAME_SGL;
  /* get list to hold data for this prim */ 
  if (!SUCCESS(iss = dblistalloc(4,20,(const dbheader_ts ** const)&dblist_ps)))
	{
	  printf ("Error with list\n");
	  return (iss=DB_BADPTR);		  
	}
  /* get list to hold secns for this prim */
  if (!SUCCESS(iss = dblistalloc(4,20,(const dbheader_ts ** const)&dbsecns_ps)))
	{
	  printf ("Error with list\n");
	  return(iss=DB_BADPTR);		  
	}
  /* get list to hold units for this prim */
  if (!SUCCESS(iss = dblistalloc(4,20,(const dbheader_ts ** const)&dbunits_ps)))
	{
	  printf ("Error with list\n");
	  return (iss=DB_BADPTR);		  
	}	  
  /* primary only */
  if (length==SZ_NAME_SGL){
	/* get all units, all secns     */
	DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdump: prim only \n"));
	if (!SUCCESS(iss = dblunits(prim_pa,(dbheader_ts ** const)&dbunits_ps))){
	  return (iss);
	}
    if (dbunits_ps->cur == 0){
	  return(iss=DB_NOTALLUNIT);
	}     
	iss = dblsecns(prim_pa,(dbheader_ts ** const)&dbsecns_ps); 
  }
  /* primary, unit, no secn */
  else if (length==2*SZ_NAME_SGL){
	if (strncmp(unit_pa,"ALL*",4)==0){
	  /* all units, all secns */
	  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdump: prim ALL* ALL*\n"));
	  if (!SUCCESS(iss = dblunits(prim_pa,(dbheader_ts ** const)&dbunits_ps))){
		return (iss);
	  }
	  if (dbunits_ps->cur == 0){
		return(iss=DB_NOTALLUNIT);
	  }  
	  iss = dblsecns(prim_pa,(dbheader_ts ** const)&dbsecns_ps); 
	}
	else {
	  /* one unit, all secns */
	  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdump: prim unit ALL* \n"));
	  dblistAdd( 4, 1, (void *)unit_pa,(const dbheader_ts ** const)&dbunits_ps);
	  if (!SUCCESS(iss = dblsecns(prim_pa,(dbheader_ts ** const)&dbsecns_ps))){
		return (iss);
	  }        	  
	}
  }
  /* prim, unit, secn */
  else if (length == 3*SZ_NAME_SGL){
	if (strncmp(unit_pa, "ALL*", 4)==0){
	  /* all units, one secn */
	  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdump: prim ALL* secn\n"));
	  if (!SUCCESS(iss = dblunits(prim_pa,(dbheader_ts ** const)&dbunits_ps))){
		return (iss);
	  }
	  if (dbunits_ps->cur/sizeof(SZ_NAME_SGL) == 0){
		return(iss=DB_NOTALLUNIT);
	  } 
	  dblistAdd( 4, 1, (void *)secn_pa,(const dbheader_ts ** const)&dbsecns_ps); 	  
	}
	else {
	  /* one unit, one secn */
	  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdump: prim unit secn\n"));
	  dblistAdd( 4, 1, (void *)unit_pa,(const dbheader_ts ** const)&dbunits_ps);
	  dblistAdd( 4, 1, (void *)secn_pa,(const dbheader_ts ** const)&dbsecns_ps); 
	}	
  }
  if (!SUCCESS(iss=dbdumpGet(prim_pa, dbunits_ps, dbsecns_ps, (const dbheader_ts **const) &dblist_ps))){
	return iss;
  }
  
  if (dblist_ps)
	iss = dblistfree(dblist_ps);
  if (dbunits_ps)
	iss = dblistfree(dbunits_ps);
  if (dbsecns_ps)
	iss= dblistfree(dbsecns_ps);
  return (iss);
}
/*=============================================================================
  Name: dbdumpGet

  Abs: For the given primary, return the corresponding list of
       units.  

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	    const char *   const     prim_a       R       ptr to 4 ASCII chars w prim
        const dbheader_ts *const dbunits_ps   R       ptr to pop'ed list of units
        const dbheader_ts *const dbsecns_ps   R       ptr to pop'ed list of secns
        dbheader_ts ** const     dblist_pps  R/W      ptr to ptr of data list

  Rem:  Using input lists, retrieves data from db and 
	    displays PRIM:MICR, UNIT
	      SECNx:   data        
             ...
	      SECNy:   data      
  Side: Memory allocation of dblist_pps (down in list utils) if list too short
          'R' means real
          'I' or 'Z' and width =2 means short int
          'I', 'Z', and width=4 means long int
          'A' means 4-byte chars array
          'T' means VMS time value
          'S' means char array
  Ret:  vmsstat_t

==============================================================================*/
vmsstat_t dbdumpGet (const char * const prim_a, 
					 const dbheader_ts * const dbunits_ps,
					 const dbheader_ts * const dbsecns_ps,
					 const dbheader_ts ** const dblist_pps)
{
  vmsstat_t iss =MICR_OKOK; 
  unsigned short cnt, scnt, dcnt;
  unsigned short secnCnt, unitCnt;
  unsigned long * next_p; 
  unsigned long * snext_p;
  dbheader_ts * dbdata_ps = NULL;
  char format; 
  int width; 
  int count;
  int supn;
  char * dbdata_p;
  dbheader_ts * temp_p;
  unsigned short nameSize = WSZ_NAME_SGL;  

  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdumpGet\n"));
  if (!prim_a || !dbunits_ps || !dbsecns_ps || !dblist_pps || !*dblist_pps)
	{
	  DEBUGPRINT(DP_ERROR,dbdumpFlag,("dbdumpGet: list error; exit \n"));
	  return (iss = DB_BADPTR);
	}
  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdumpGet: secns list cur=%u \n", dbsecns_ps->cur ));
  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdumpGet: units list cur=%u \n", dbunits_ps->cur ));

  secnCnt = (dbsecns_ps->cur)/nameSize;
  unitCnt = (dbunits_ps->cur)/nameSize;
 
  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdumpGet: %u secns in dbsecns list\n", secnCnt)); 
  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdumpGet: %u units in dbunits list\n", unitCnt));  

  /* form list of dict ptrs in dblist_pps  */
  /* for ea unit */ 
  next_p = (unsigned long *)(dbunits_ps) + sizeof(dbheader_ts)/sizeof(unsigned long);
  for (cnt=0; (cnt < unitCnt); cnt++, next_p++){
	DEBUGPRINT(DP_DEBUG,dbdumpFlag,
		  ("dbdumpGet: processing unit %u out of %u\n", cnt, unitCnt));
	/* for each secn - start at beg of secn list */
	snext_p = (unsigned long *)(dbsecns_ps);
	snext_p += sizeof(dbheader_ts)/sizeof(long);
	
    for (scnt=0; (scnt < secnCnt); scnt++, snext_p++){
	  if (!SUCCESS(iss=dblist ((const char *)prim_a, 
						   (const char *)next_p, 
						   (const char *)snext_p,
						   dblist_pps))){
		return(iss);
	  }
	} /* for every secn in list */
  } /* for every unit in list */

  temp_p= (dbheader_ts *)*dblist_pps;
  DEBUGPRINT(DP_DEBUG,dbdumpFlag,
			 ("dbdumpGet: dblist list cur=%u after for loops; ptr=%p\n", temp_p->cur, temp_p ));
  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdumpGet: dblistalloc\n"));    

  /* given dblist of dict ptrs, get data for all */
  if (!SUCCESS(iss = dblistalloc( (((dbheader_ts*)*dblist_pps)->cur)*2, 2,
								  (const dbheader_ts ** const)&dbdata_ps ))) {
	  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdumpGet: dblistalloc error\n"));
	  return iss;
  }
  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdumpGet: dblgetting..w dblist ptr=%p.\n", *dblist_pps));  

  if (!SUCCESS(dblget( (const dbheader_ts * const)*dblist_pps, &dbdata_ps))) {
	if (iss == DB_BADSUP) printf ("Error: cannot access supertype 4 data from IOC.\n"); 
	else { 
	  DEBUGPRINT(DP_DEBUG,dbdumpFlag,("dbdumpGet: dblget error\n"));
	  /* debug - print out the formed list */
	  dblistPrint((const dbheader_ts * const)*dblist_pps);
	}
	return iss;
  }
  
  /* given data, loop through again and display */
  /* for ea unit */
  dbdata_p = (char *)dbdata_ps + sizeof(dbheader_ts); 
  next_p = (unsigned long *)(dbunits_ps) + sizeof(dbheader_ts)/sizeof(unsigned long);
  for (cnt=0; (cnt < unitCnt); cnt++){
	printf("\n%.4s: %.4s %.4s", slcGetIOCName(), prim_a, (char *)next_p);
	/* for each secn */
	/*snext_p = (unsigned long *)(dbsecns_ps) + 4;*/
	snext_p = (unsigned long *)dbsecns_ps;
	snext_p += sizeof(dbheader_ts)/sizeof(unsigned long); 

    for (scnt=0; (scnt < secnCnt); scnt++){
	  dbgetMeta ((const char *)prim_a, 
				 (const char *)next_p, 
				 (const char *)snext_p,
				 &format, 
		         &width, 
				 &count, &supn);

	  if (format=='A') {
		for (dcnt=0; dcnt<count/4; dcnt++) {
		  if (dcnt % 14 == 0) printf("\n    %.4s", (char *)snext_p);
		  dbdumpFormat ((const void * const )dbdata_p, (const char * const)snext_p, 
						format, width, count);
		  dbdata_p+=4;
		}	  
	  }
	  else if (format == 'T'){
		for (dcnt=0; dcnt<count; dcnt++) {
		  if (dcnt % 2 == 0) printf("\n    %.4s", (char *)snext_p);
		  dbdumpFormat ((const void * const )dbdata_p, (const char * const)snext_p, 
						format, width, count);
		  dbdata_p+=width;
		}			
	  }
	  else{
		for (dcnt=0; dcnt<count; dcnt++) {
		  if (dcnt % 4 == 0) printf("\n    %.4s", (char *)snext_p);
		  dbdumpFormat ((const void * const )dbdata_p, (const char * const)snext_p, 
						format, width, count);
		  dbdata_p+=width;
		}
	  }
	  snext_p++;
	}
	next_p++;
  }	 
  printf("\n");
  if (dbdata_ps){
	dblistfree(dbdata_ps);
  }

  return (iss);
}

/*=============================================================================
  Name: dbdumpFormat

  Abs: For the given primary, return the corresponding list of
       units.  

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	    const char *   const     prim_a       R       ptr to 4 ASCII chars w prim
        const dbheader_ts *const dbunits_ps   R       ptr to pop'ed list of units
        const dbheader_ts *const dbsecns_ps   R       ptr to pop'ed list of secns
        dbheader_ts ** const     dblist_pps  R/W      ptr to ptr of data list

  Rem:  Using input lists, retrieves data from db and 
	    displays PRIM:MICR, UNIT
	      SECNx:   data        
             ...
	      SECNy:   data      
  Side: Memory allocation of dblist_pps (down in list utils) if list too short
          'R' means real
          'I' or 'Z' and width =2 means short int
          'I', 'Z', and width=4 means long int
          'A' means 4-byte chars array
          'T' means VMS time value
          'S' means char array
  Ret:  vmsstat_t


==============================================================================*/
LOCAL void dbdumpFormat (const void * const dbdata_p,
						 const char * const secn_p,
						 const char format,
						 const int width,
						 const int count)
{
  long lint;           /* for accessing 8 byte 'T' as 2 long words */
  unsigned long ulint;
  short sint; 
  float tfloat;
  char * char_p;

  char_p = (char *)dbdata_p;       /* for width = 4 type operations */
  switch (format){
  case 'I':
	if (width==4){
	  memcpy (&lint, char_p, width);
	  printf (" %ld",lint);
	}
	else {
	  memcpy (&sint, char_p, width);
	  printf (" %d",sint);
	}
	break;
  case 'A':
	/* count is total # of chars; space between every 4 chars */
	printf ( " %.4s", char_p); 
	break;
  case 'R':	
	memcpy (&tfloat, char_p, width);
	printf (" %12.6g",tfloat);
	break;
  case 'Z':
	if (width==4){
	  memcpy (&lint, char_p, width);
	  printf (" %.8lx",lint);	  
	}
	else {
	  memcpy (&sint, char_p, width);
	  printf (" %.4hx",sint);
	}
	break;
  case 'S':
	printf ( " %.*s", count, char_p); 		
	break;
  case 'T':
	memcpy (&ulint, char_p, width);
	printf ( " [0]: 0x%.8lx", ulint);
	char_p += 4;
	memcpy (&ulint, char_p, width);
	printf ( " [1]: 0x%.8lx", ulint);   
	break;
  }

}
