/*=============================================================================
 
  Name: dbedit.c
        dbeditEdit
        dbeditDisp

  Abs:  Utilities which write to a database PUS
        Reference web pages on LCLS Controls | Facilities |SLC-Aware IOC | :
                        Database Service (DBS) Design Document 
                        Functional Requirements | SLC Database Service (DBS)
                        Database Input/Output Utilities
                        C Coding Standard


  Auth: 7-MAR-2005, Debbie Rogind (DROGIND) created
  Rev:  dd-mmm-200y,
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*-----------------------------------------------------------------------------  
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
#include <stdio.h>
  int dbeditFlag = DP_ERROR;
#endif
/* c includes */
#include     <stdio.h>
#include     <stdlib.h>    /* malloc, free */
#include     <string.h>    /* memcpy */
#include     <cantProceed.h>

/* slc global vars */
#include     "slcGlob.h"

/* VMS Control system function/status codes  */
#include     "msgfunc.hc"  /* FUNC_MSG function code */
#include     "micrdef.hc"  /* MICR_EXNET_ codes */

/* slc common includes */
#include     "cvtUtil.h"   /* slc conversion utilities */
#include     "timeUtil.h"  /* slc time conversion utilities */
#include     "slcCmlog.h"  /* slcCmlogLogMsg */
#include     "slcUtil.h"   /* slcGetIOCName */ 

/* dbs includes  */  
#include     "db_micro_struc.h"
#include     "dbUtil.h" 
#include     "dbGlob.h"   
#include     "dbdef.hc"    /* DB_OKOK,,*/
#include     "dbdict.h"
#include     "freeList.h"
#include     "cantProceed.h"
/* dbdict */
#include      "dbdict.h"
/* dbget */
#include      "dbget.h"
/* dblist  prototypes */
#include    "dblistUtil.h"
#include    "dblist.h"
#include    "dbl.h"
#include    "dbedit.h"

/*=============================================================================
  Name: dbeditEdit

  Abs:  Setup routine prior to editing data.


  Args:     Type		        Name	 Access	   Description
        ------------------- -----------	---------- ----------------------------
	    const char *   const   prim_pa       R     ptr to 4 ASCII chars w prim
	    const char *   const   unit_pa       R     ptr to 4 ASCII chars w units
	    const char *   const   secn_pa       R     ptr to 4 ASCII chars w secns  

  Rem:  Allocates and populates unit list (list of unit(s), ea 4 ASCII chars) 
        and secondary list (list of secn(s), ea 4 ASCII chars, dep. on prim) 
		based on input arguments and database. Also allocates data list.
        Calls dbdumpDisp to retrieve from db and output to console the data 
		for each prim:unit:secn configured in unit and secn lists.
		
  Side: Memory allocation (down in list utils when list is too short)

  Ret:  vmsstat_t:

==============================================================================*/
vmsstat_t dbeditEdit (const char * const prim_pa, 
					  const char * const unit_pa,
                      const char * const secn_pa,
					  const char * const val_p)
{
  vmsstat_t iss =MICR_OKOK; 
  dbheader_ts * dblist_ps = NULL;
  dbheader_ts * dbdata_ps = NULL;
  char * nextData_p;
  short sint;
  long lint;
  float flt;
  unsigned long ulint;
  unsigned short usint;
  char format = ' '; 
  int  width = 0;
  int  count = 0;
  int  supn = 0;
  slcJob_te iocJobId_e;
  char name_a[5] = {'I', 'O', 'C', 'S', '\0'};

  if (! SUCCESS(iss = dbgetMeta(prim_pa, unit_pa, secn_pa, &format, &width, &count, &supn))) 
	{
    return iss;		
	}

  /* get list to hold data for this prim */ 
  if (!SUCCESS(iss = dblistalloc(4,1,(const dbheader_ts ** const)&dblist_ps)))
	{
	  printf ("Error with dblist allocation\n");
	  return (iss);		  
	}
  if (!SUCCESS(iss = dblistalloc(4,1,(const dbheader_ts ** const)&dbdata_ps)))
	{
	  printf ("Error with dblist allocation\n");
	  return (iss);		  
	}
  nextData_p = (char *)dbdata_ps;
  nextData_p += sizeof(dbheader_ts);

  if (!SUCCESS(iss=dblist ((const char *)prim_pa, 
						   (const char *)unit_pa, 
						   (const char *)secn_pa,
						   (const dbheader_ts **const) &dblist_ps)))
	{
	  printf ("Error with dblist...\n");
	  return (iss);		  
	}	  

  switch (format){
  case 'I':
	DEBUGPRINT(DP_INFO,dbeditFlag, ("dbedit: 'I'\n")); 
	if (width==4){
	  sscanf(val_p, "%ld", &lint);
	  memcpy(nextData_p, &lint, width);
	  printf ("Input: %ld" ,lint);  
	}
	else {
	  sscanf(val_p, "%hd", &sint);	
	  memcpy(nextData_p, &sint, width);  
	  printf ("Input: %hd",sint);  
	}
	break;
  case 'A':
  case 'S':
	DEBUGPRINT(DP_INFO,dbeditFlag, ("dbedit: 'A' or 'S'\n"));
    memcpy(nextData_p, val_p, width);
	printf ("Input: %s",val_p);
	break;
  case 'R':
	sscanf(val_p, "%f", &flt);
	memcpy(nextData_p, &flt, width);
	printf ("Input: %f",flt);
	break;
  case 'Z':
	if (width==4){
	  sscanf(val_p, "%lx", &ulint);
      memcpy(nextData_p, &ulint, width);
	  printf ("Input: %lx" ,ulint);
	}
	else {
	  sscanf(val_p, "%hx", &usint);
      memcpy(nextData_p, &usint, width);	  
	  printf ("Input: %hx",usint); 
	}
	break;
  case 'T':
	printf ("T format is not implemented.\n");
	break;
  }

  iocJobId_e =  slcGetJobEnumByName ((const char * const) name_a);

  /* update db data list current byte count */
  /* NOTE: when editing arrays, the first element in array will be edited;
      the rest of the array will be zeroed out. */
  dbdata_ps->cur += width;
  /* prim, unit, secn */
  if (supn==1) {
	if (!SUCCESS(iss=dblputST1  ((const dbheader_ts * const) dblist_ps,  
								 (const dbheader_ts * const) dbdata_ps,
								 (int)iocJobId_e))) return iss;
  }
  else {
	if (!SUCCESS(iss=dblput  ((const dbheader_ts * const) dblist_ps,  
							  (const dbheader_ts * const) dbdata_ps,
							  (int)iocJobId_e))) return iss;
  } 
  printf(" was written.\n"); 
  
  if (dblist_ps)
	iss = dblistfree(dblist_ps);
  if (dbdata_ps)
	iss = dblistfree(dbdata_ps);

  return (iss);
}
