/*=============================================================================
 
  Name: dbget.c
        dbgetUnitAsString
		dbgetCatPUSName
		dbgetFormat
		dbgetWidth 
		dbgetCount
		dbgetMeta 
		dbgetVersion
                dbgetUnitCount
		for getting dbExists, use slcGetDbExists()

  Abs:  Utilities which return info about the database or individual entries
        in the dict/db 
        Reference web pages on LCLS Controls | Facilities |SLC-Aware IOC | :
                        Database Service (DBS) Design Document 
                        Functional Requirements | SLC Database Service (DBS)
                        Database Input/Output Utilities
                        C Coding Standard


  Auth: 20-JAN-2005, Debbie Rogind (DROGIND) created
  Rev:  dd-mmm-200y,
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*-----------------------------------------------------------------------------  
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
#include <stdio.h>
  int dbgetFlag = DP_ERROR;
#endif
/* c includes */
#include     <stdio.h>
#include     <stdlib.h>    /* malloc, free */
#include     <string.h>    /* memcpy */
#include     <cantProceed.h>

/* slc global vars */
#include     "slcGlob.h"

/* VMS Control system function/status codes  */
#include     "msgfunc.hc"  /* FUNC_MSG function code */
#include     "micrdef.hc"  /* MICR_EXNET_ codes */

/* slc common includes */
#include     "cvtUtil.h"   /* slc conversion utilities */
#include     "timeUtil.h"  /* slc time conversion utilities */
#include     "slcCmlog.h"  /* slcCmlogLogMsg */
#include     "slcUtil.h"   /* slcGetIOCName */ 

/* dbs includes  */  
#include     "db_micro_struc.h"
#include     "dbUtil.h" 
#include     "dbGlob.h"   
#include     "dbdef.hc"    /* DB_OKOK,,*/
#include     "dbdict.h"
#include     "freeList.h"
#include     "cantProceed.h"
/* dbdict */
#include      "dbdict.h"
/* dbget */
#include      "dbget.h"

#include      "dbcheck.h"

/*-----------------------------------------------------------------------
  Name: dbgetNullToBlank

  Abs:  Replaces any blank in given 4 char input string with nulls
        

  Args: Type		   Name	     Access	   Description
	---------------   --------  ---------- ----------------------------
        char *            name_pa     W     pointer to 4 char primary, secn, unit name


  Rem:  Input string may be null-terminated (unit will be if <4 chars, as it
        is converted from int to ASCII via "sprintf()". Input string should be
        4 chars long.

  Ret:  none
----------------------------------------------------------------------*/
void dbgetNullToBlank (unsigned long *name_pa)
{ 
  char name[SZ_NAME_SGL]= {' ',' ',' ',' '};
  size_t cnt = 0;

  DEBUGPRINT(DP_ERROR, dbgetFlag,("dbgetNullToBlank- name_pa = %.4s; 0x%.8lx; ptr = %p\n", 
								  (char *)name_pa, *name_pa,  &name_pa));
  cnt = strspn((char *)name_pa,"ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_");
  if (cnt<=SZ_NAME_SGL)
	memcpy (&name, name_pa, cnt);
  else 
	memcpy (&name, name_pa, SZ_NAME_SGL);
  DEBUGPRINT(DP_ERROR, dbgetFlag,("dbgetNullToBlank - cnt = %d\n", cnt));
  memcpy (name_pa, &name, SZ_NAME_SGL);
 

}
/*-----------------------------------------------------------------------
  Name: dbgetBlkPad

  Abs:  Pads given 4 char input string with trailing blanks,
        rendering a left justified, blank padded, non-null terminated char[4]
        

  Args: Type		   Name	     Access	   Description
	---------------   --------  ---------- ----------------------------
        char *            name_pa     W     pointer to 4 char primary, secn, unit name


  Rem:  Input string may be null-terminated (unit will be if <4 chars, as it
        is converted from int to ASCII via "sprintf()". Input string should be
        4 chars long.

  Ret:  none
----------------------------------------------------------------------*/
void dbgetBlkPad (char *name_pa)
{ 
  char name[SZ_NAME_SGL]= {' ',' ',' ',' '};
  size_t cnt = 0;

  cnt = strspn(name_pa,"ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_");
  if (cnt<=SZ_NAME_SGL)
	memcpy (&name, name_pa, cnt);
  else 
	memcpy (&name, name_pa, SZ_NAME_SGL);
  memcpy (name_pa, &name, SZ_NAME_SGL); 
}

/*=============================================================================
  Name: dbgetUnitAsString

  Abs:  This routine 

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
	const dbheader_ts **const db_pps  R/W    pointer to pointer to structure


   Rem: 

   Side: Blank pads at end of the 4 chars (unit is left justified)

   Ret:  vmsstat_t:
         DB_OKOK            - good status     


==============================================================================*/ 
vmsstat_t dbgetUnitAsString(const unsigned short unit, char * const unit_pa)
{ 
  vmsstat_t iss = DB_OKOK;
  char      temp_ca[ SZ_NAME_SGL+1]; /* leave from for 4 chars + null term from sprintf */
  /*----------------------------------------------------------------------------*/

  if (unit > 9999){
	return (iss= DB_BADUNIT);
  }
  /* don't sprinf with unit_pa; will overwrite next free space unit with null */
  /*   term in 5th char for unit numbers with 4 significant digits */
  memcpy (&temp_ca[0],unit_pa, SZ_NAME_SGL);
  sprintf(&temp_ca[0], "%d", unit); /* convert to ASCII */
  memcpy (unit_pa, &temp_ca[0],  SZ_NAME_SGL);
  dbgetBlkPad(unit_pa);
  return(iss); 
}

/*-----------------------------------------------------------------------
  Name: dbgetFormat

  Abs:  Given prim, unit, secn ASCII names, this procedure looks up the
        meta data for that channel in the dictionary and returns the format char

  Args: Type		   Name	             Access	   Description
	---------------   --------       ---------- ----------------------------  
        const char *   prim_a              R  pointer to 4 char primary name 
        const char *   secn_a              R  pointer to 4 char secondary name  
  
  Rem:  Calls dbgetMeta() with unit = ALL* to get the format char

  Side: 

  Ret:  format character (I,A,R,Z,S,T)
        If dbgetMeta() fails, it logs an error and ' ' (blank) will be 
		returned to caller by dbgetFormat().
----------------------------------------------------------------------*/
char dbgetFormat (const char * const prim_a, const char * const secn_a)
{
  char format = ' ';
  int width = 0;
  int count = 0;
  int supn = 0;

  dbgetMeta(prim_a, "ALL*", secn_a, &format, &width, &count, &supn);
  return format;  
}
/*-----------------------------------------------------------------------
  Name: dbgetWidth

  Abs:  Given prim, unit, secn ASCII names, this procedure looks up the
        meta data for that channel in the dictionary and returns the format char

  Args: Type		   Name	             Access	   Description
	---------------   --------       ---------- ----------------------------  
        const char *   prim_a              R  pointer to 4 char primary name 
        const char *   secn_a              R  pointer to 4 char secondary name  
  
  Rem:  Calls dbgetMeta() with unit = ALL* to get the width of data
        associated with this PUS

  Side: 

  Ret:  width        
        If dbgetMeta() fails, it logs an error and -1 will be returned 
            to caller by dbgetWidth().
----------------------------------------------------------------------*/
int dbgetWidth (const char * const prim_a, const char * const secn_a)
{
  char format = '0';
  int width = -1;
  int count = -1;
  int supn = 0;

  dbgetMeta(prim_a, "ALL*", secn_a, &format, &width, &count, &supn);
  return width;  
}
/*-----------------------------------------------------------------------
  Name: dbgetCount

  Abs:  Given prim, unit, secn ASCII names, this procedure looks up the
        meta data for that channel in the dictionary and returns the format char

  Args: Type		   Name	             Access	   Description
	---------------   --------       ---------- ----------------------------  
        const char *   prim_a              R     pointer to 4 char primary name 
        const char *   unit_a              R     pointer to 4 char unit name
        const char *   secn_a              R     pointer to 4 char secondary name  
  

  Rem:  Calls dbgetMeta() to get the count for this PUS
        Error if unit = ALL*
  Side: 

  Ret:  count       
        If dbgetMeta() fails, it logs an error and -1 will be returned 
            to caller by dbgetcount().
----------------------------------------------------------------------*/
int dbgetCount   (const char * const prim_a, 
				  const char * const unit_a, 
				  const char * const secn_a)
{
  char format = ' ';
  int width = -1;
  int count = -1;
  int supn = 0;

  if (strncmp(unit_a, "ALL*", SZ_NAME_SGL)==0){
	DEBUGPRINT(DP_ERROR, dbgetFlag,("dbgetCount - cannot pass in ALL* for units! \n"));
	return count;
  }
  dbgetMeta(prim_a, unit_a, secn_a, &format, &width, &count, &supn);
  return count;  
}

/*-----------------------------------------------------------------------
  Name: dbgetMeta

  Abs:  Given prim, unit, secn ASCII names, this procedure looks up the
        meta data for that channel in the dictionary.

  Args: Type		   Name	             Access	   Description
	---------------   --------       ---------- ----------------------------  
        const char *   prim_a              R  pointer to 4 char primary name 
        const char *   unit_a              R  pointer to 4 char unit name
        const char *   secn_a              R  pointer to 4 char secondary name  
		char *         format              W  pointer to return value for format
        int *          width               W  pointer to return value for width; =
		                                      how many elements (length)for this PUS stored in ST0
        int *          count               W  pointer to return value for count; =
		                                      VMS byte count of each element (from PRIMARY.MAP)
  
  Rem:  if unit=ALL*, first get primary-only dict entry, which points to its unit list
        in dict. The first unit in the list is then concatenated with input args
        prim_a and secn_a to form the complete 12 char PUS entry. 
        if unit != ALL*, the prima_a, unit_a, and secn_a args are concatenated to 
        form the 12 char PUS entry.
        The PUS entry is then found in the dictionary, and its meta data extracted
        and stored in the locations pointed to by the output arguments of format, width
        and count.

  Side: 

  Ret:  iss
        DB_BADPRIM, DB_BADUNIT, DB_BADSECN, DB_BADPTR
----------------------------------------------------------------------*/
vmsstat_t dbgetMeta (const char * const prim_a, 
		       const char * const unit_a, 
		       const char * const secn_a, 
		       char * format, 
		       int * width, 
		       int * count,
			   int * supn)
{ 
  vmsstat_t iss = DB_OKOK;
  dbhEntry_ts *  dbhEntry_ps;     /* pointer to dict entry in hash tbl */
  dbdictP_ts  *  dbdictP_ps;      /* pointer to dbdictP primary only entry in dict */     
  dbdictU_ts  *  dbdictU_ps;      /* pointer to dbdictU unit entry */
  dbdictPUS_ts * dbdictPUS_ps;    /* pointer to PUS entry in dict */
  ELLLIST        listUnits;       /* unit list head for input arg prim_a when units=ALL* */
  char           *unit_pa;
  char           primUnitSecn_a[SZ_NAME_PUS]; /* contains concatenated prim  unit secn name */

  /*----------------------------------------------------------------------------*/

  if (strncmp(unit_a, "ALL*", SZ_NAME_SGL)==0) {
  
	dbhEntry_ps = dbhFind(dbdict_p,(char *) prim_a, SZ_NAME_SGL);
	if (!dbhEntry_ps) {
	  DEBUGPRINT(DP_INFO,dbgetFlag,("dbgetMeta - invalid prim \n"));
	  return (iss=DB_BADPRIM);
	}  
 
	/* progress linearly through units linked list */
	dbdictP_ps = (dbdictP_ts *)dbhEntry_ps->usrPvt;
	if (!dbdictP_ps){
	  /* if no dbdictP_ts entry, then no units ass'ed w this prim */
	  DEBUGPRINT(DP_INFO,dbgetFlag,("dbgetMeta - valid prim, but invalid secn (no units) \n"));
	  return (iss=DB_BADSECN);
	}
	
	listUnits = dbdictP_ps->listUnits;
	if (!listUnits.count >0){
	  DEBUGPRINT(DP_INFO,dbgetFlag,("dbgetMeta - valid prim, but no units \n"));
	  return (iss=DB_BADUNIT);
	}
	
	dbdictU_ps = (dbdictU_ts *)ellFirst(&(listUnits));
	unit_pa = (char *)(dbdictU_ps->name);
	dbgetCatPUSName (prim_a, unit_pa, secn_a, (char *)&primUnitSecn_a);
  }
  else {
	dbgetCatPUSName (prim_a, unit_a, secn_a, (char *)&primUnitSecn_a);
	DEBUGPRINT(DP_DEBUG,dbgetFlag,("dbgetMeta - PUS name: %.12s \n", primUnitSecn_a));	
  }

  dbhEntry_ps = dbhFind(dbdict_p, (char *)&primUnitSecn_a, SZ_NAME_PUS);
  if (!dbhEntry_ps) {
	DEBUGPRINT(DP_INFO,dbgetFlag,("dbgetMeta - couldn't find %.12s in dict \n", primUnitSecn_a));
	return (iss=DB_NOSUCHNODE);
  }  
  dbdictPUS_ps = (dbdictPUS_ts *)dbhEntry_ps->usrPvt;
  if (!dbdictPUS_ps) {
	DEBUGPRINT(DP_INFO,dbgetFlag,("dbgetMeta - bad dict entry for %.12s \n", primUnitSecn_a));
	return (iss=DB_BADPTR);			
  }
  
  *format = dbdictPUS_ps->format;
  *width =  dbdictPUS_ps->width; 
  *count =  dbdictPUS_ps->count;
  *supn =   dbdictPUS_ps->supn;
  return(iss); 
}
/*=============================================================================
  Name: dbgetSTAddr - not used?

  Abs:  This routine takes input arg ptr to dict hash table entry
        and forms the address of the STx data ptr 

  Args:      Type		       Name	      Access	   Description
	    ------------------- -----------	---------- ----------------------------
	const dbhEntry_ts * const dbhEntry_ps  R/W    pointer to dict hash entry


   Rem: 

   Side: 

   Ret:  address of dictionary entry's ST data    


==============================================================================*/ 
/*void * dbgetSTAddr (const char * const prim_a, 
						 const char * const unit_a, 
						 const char * const secn_a)*/
LOCAL void * dbgetSTAddr (const dbhEntry_ts * const dbhEntry_ps)
{
  void * st_p = NULL;
  dbdictPUS_ts * dbdictPUS_ps;

  if ( (!dbhEntry_ps)||(!dbhEntry_ps->usrPvt) )
	return (NULL);
  dbdictPUS_ps = dbhEntry_ps->usrPvt;
  st_p = (void *)( (unsigned short *)dbnodes_a[dbdictPUS_ps->supn]+dbdictPUS_ps->sptr);
  return (st_p);
   
}
/*-----------------------------------------------------------------------
  Name: dbgetVersion

  Abs:  Return pointer to the global db Version string

  Args: None
  
  Rem:  

  Side: 

  Ret:  Return a pointer to the global space where global string 
        dbVersion_a is stored.
----------------------------------------------------------------------*/
const char * dbgetVersion (void)
{
  return ((const char *)&dbVersion_a);
}
/*-----------------------------------------------------------------------
  Name: dbgetExists

  Abs:  Return global dbExists value

  Args: None
  
  Rem:  

  Side: Access global dbExists

  Ret:  epicsTrue or epicsFalse - value of dbExists
----------------------------------------------------------------------*/  
epicsBoolean dbgetExists(void)
{
  return (dbExists);
}
/*-----------------------------------------------------------------------
  Name: dbgetCatPUSName

  Abs:  Makes a primary unit secondary (PUS) entry in the dictionary.
        Each dictionary entry requires a pointer to a name and a pointer to data.


  Args: Type		   Name	     Access	   Description
	---------------   --------  ---------- ----------------------------
        const char *        prim_a      R     pointer to 4 char primary name 
        const char *        unit_a      R     pointer to 4 char unit name 
        const char *        secn_a      R     pointer to 4 char secondary name
        char *             primUnitSecn W     ptr to concatenated prim unit secn name string

  Rem:  concatenates each of the 4 char input substrings (prim, unit, secn) into 12 chars 
        and stores in location pointed to by primUnitSecn_pa
 
  Side: It is assumed that incoming prim_a, unit_a, and secn_a are left justified, and
        blank padded to the right (end).  

  Ret: void
----------------------------------------------------------------------*/
void dbgetCatPUSName (const char * prim_a, const char * unit_a, const char * secn_a,
		   char * primUnitSecn_pa)
{
  char name_pa[SZ_NAME_PUS] = {' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '};

  /* concatenate each of the three 4 char inputs into 12 char to use as name */
  memcpy(name_pa, prim_a, 4);
  memcpy(&name_pa[4], unit_a, 4);
  memcpy(&name_pa[8], secn_a, 4);
  memcpy(primUnitSecn_pa, name_pa, 12);
}

/*-----------------------------------------------------------------------
  Name: dbgetUnitCount

  Abs:  Given prim return the total number of units.

  Args: Type               Name              Access        Description
        ---------------   --------       ---------- ----------------------------

  Rem:  

  Side:

  Ret:  int count of units. 0 = no units or invalid primary.
----------------------------------------------------------------------*/
int dbgetUnitCount (const char * const prim_a)
{
  int		ucount = 0;      /* Count of units found */
  dbhEntry_ts * dbhEntry_ps;     /* pointer to dict entry in hash tbl */
  dbdictP_ts  * dbdictP_ps;      /* pointer to dbdictP primary only entry in dict */
/*---------------------------------------------------------------------*/

  dbhEntry_ps = dbhFind(dbdict_p,(char *) prim_a, SZ_NAME_SGL);
  if (!dbhEntry_ps) {
    DEBUGPRINT(DP_INFO,dbgetFlag,("dbgetUnitCount - invalid prim \n"));
    goto egress;
  }
  /* progress linearly through units linked list */
  dbdictP_ps = (dbdictP_ts *)dbhEntry_ps->usrPvt;
  if (!dbdictP_ps){
    /* if no dbdictP_ts entry, then no units ass'ed w this prim */
    DEBUGPRINT(DP_INFO,dbgetFlag,
               ("dbgetUnitCount - NULL dictionary pointer this primary\n"));
    goto egress;
  }
/*  listUnits = dbdictP_ps->listUnits; */
  ucount = dbdictP_ps->listUnits.count;
  if (!ucount >0)
    DEBUGPRINT(DP_INFO,dbgetFlag,("dbgetUnitCount - valid prim, but no units.\n"));
egress:
  return ucount;
}
