/*=============================================================================
 
  Name: dblistUtil.c
        dblistInitFreeLists  Init small and big list free lists
        dblistDelFreeLists   Free small and big list free lists
		dblistNew            Create a new list
		dblistEnlarge        Make an existing list larger
		dblistAdd            Add data into a list

  Abs:  Utilities which manage database lists 
        Reference web pages on LCLS Controls | Facilities |SLC-Aware IOC | :
                        Database Service (DBS) Design Document 
                        Functional Requirements | SLC Database Service (DBS)
                        Database Input/Output Utilities
                        C Coding Standard


  Auth: 3-JAN-2005, Debbie Rogind (DROGIND) created
  Rev:  dd-mmm-200y,
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*-----------------------------------------------------------------------------  
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
#include <stdio.h>
  int dblistUFlag = DP_ERROR;
#endif
/* c includes */
#include     <stdio.h>
#include     <stdlib.h>    /* malloc, free */
#include     <string.h>    /* memcpy */
#include     <cantProceed.h>

/* slc global vars */
#include     "slcGlob.h"

/* VMS Control system function/status codes  */
#include     "msgfunc.hc"  /* FUNC_MSG function code */
#include     "micrdef.hc"  /* MICR_EXNET_ codes */

/* slc common includes */
#include     "cvtUtil.h"   /* slc conversion utilities */
#include     "timeUtil.h"  /* slc time conversion utilities */
#include     "slcCmlog.h"  /* slcCmlogLogMsg */
#include     "slcUtil.h"   /* slcGetIOCName */ 

/* dbs includes  */  
#include     "db_micro_struc.h"
#include     "dbUtil.h" 
#include     "dbGlob.h"   
#include     "dbdef.hc"    /* DB_OKOK,,*/
#include     "dbdict.h"
#include     "freeList.h"
#include     "cantProceed.h"
/* dbdict */
#include      "dbdict.h"

/* dblist  prototype */
#include    "dblist.h"
#include    "dblistUtil.h"
#include    "slcMain.h"   /* slcfreelistUtil */
/* ============================================================================= 
   Name: dblistInitFreeLists

   Abs:   Initializes big and small list free lists

   Args:  None

   Rem:   

   Side:  Utilizies epics libCom freeListLib 

   Ret:  void

 ==============================================================================*/
vmsstat_t dblistInitFreeLists (void)
{
  vmsstat_t iss = MICR_OKOK;

  if (!dblistSmFreeList_p){
    freeListInitPvt(&dblistSmFreeList_p,(DBL_SZSM), DBL_NSM);
	if (!dblistSmFreeList_p){
	  DEBUGPRINT(DP_ERROR,dblistUFlag, ("dblist: freeListSmFreeList ptr = %p\n", dblistSmFreeList_p));
	  iss = DB_BADPTR;
	}
  }
  if (!dblistBigFreeList_p){
    freeListInitPvt(&dblistBigFreeList_p,(DBL_SZBIG), DBL_NBIG);
	if (!dblistBigFreeList_p){
	  DEBUGPRINT(DP_ERROR,dblistUFlag, ("dblist: freeListBigFreeList ptr = %p\n", dblistBigFreeList_p));
	  iss = DB_BADPTR;
	}
  }  
  return iss;

}
/* ============================================================================= 
   Name: dblistDelFreeLists

   Abs:  Removes big and small list free lists

   Args: None

   Rem:   

   Side: Utilizies epics libCom freeListLib
         The freeListCleanup rte does not set ptr to NULL - so do it here

   Ret:  void

 ==============================================================================*/
void dblistDelFreeLists (void)
{
  void * temp_p;

  DEBUGPRINT(DP_INFO,dblistUFlag, ("dblist: freeListSmFreeList ptr = %p\n", dblistSmFreeList_p));
  DEBUGPRINT(DP_INFO,dblistUFlag, ("dblist: freeListBigFreeList ptr = %p\n", dblistBigFreeList_p));
  if (dblistSmFreeList_p){
    temp_p = dblistSmFreeList_p;

    dblistSmFreeList_p = NULL;
	DEBUGPRINT(DP_INFO,dblistUFlag, ("dblist: temp_p ptr = %p\n", temp_p));
	DEBUGPRINT(DP_INFO,dblistUFlag, ("dblist: cleaning up freeListSmFreeList \n"));
    freeListCleanup(temp_p);

  }
  if (dblistBigFreeList_p){
    temp_p = dblistBigFreeList_p;
    dblistBigFreeList_p = NULL;
	DEBUGPRINT(DP_INFO,dblistUFlag, ("dblist: cleaning up freeListBigFreeList \n"));
    freeListCleanup(temp_p);
  }  
}
/*=============================================================================
   Name: dblistNew 

   Abs:  This routine initializes a data list from
         either of two free list memory pools, small and big. It uses
         input argument bytes_to_allocate to determine from which pool
         to choose. If bytes_to_allocate is larger than the big pool
         size, then it allocates from the heap.

   Args:     Type		    Name	         Access	   Description
	     ---------------    -----------     ---------- ----------------------------
		 const unsigned long bytes_to_allocate  R     Number of bytes initially to be in list
		 const dbheader_ts **const dbdata_pps   W     pointer to pointer of list

   Rem:  The list must be passed as a pointer to a pointer so that it
         may be modified.

   Side: Uses epics freeList to manage big and small pool.
         Adjusts dbheader field max.

   Ret:  dblist_p - pointer to list


==============================================================================*/
vmsstat_t dblistNew (const unsigned long bytes_to_allocate, 
					 const dbheader_ts ** const dbdata_pps)  
   
{
  vmsstat_t iss = MICR_OKOK;
  dbheader_ts *   dbdata_ps;

  if (bytes_to_allocate < DBL_SZSM ) {
	/*DEBUGPRINT(DP_INFO,dblistUFlag, ("dblistNew: getting small list diagnostics...\n")); 
	  slcfreelistUtil(dblistSmFreeList_p,  DBL_NSM, DBL_SZSM, "dblist small");*/	
	dbdata_ps = freeListCalloc(dblistSmFreeList_p);
	DEBUGPRINT(DP_INFO,dblistUFlag, ("dblistNew: freeListCalloc ptr = %p\n", dbdata_ps)); 
	if (!dbdata_ps){
	  iss = MICR_MALLOC;
	  DEBUGPRINT(DP_ERROR,dblistUFlag, ("dblistNew: UNABLE to freeListCalloc db small list\n")); 
	  slcCmlogLogMsg(iss, "MALLOC",
					 "%s unable to freeListCalloc small dblist buffer from freelist.\n", 
					 epicsThreadGetNameSelf());
	  return (iss);
	}
	dbdata_ps->max = DBL_WSZSM;  /* word size of list */
  }
  else if (bytes_to_allocate < DBL_SZBIG) {
	DEBUGPRINT(DP_INFO,dblistUFlag, ("dblistNew: getting big list diagnostics...\n")); 	
	/*slcfreelistUtil(dblistBigFreeList_p, DBL_NBIG, DBL_SZBIG, "dblist big");*/
	dbdata_ps = freeListCalloc(dblistBigFreeList_p);
	if (!dbdata_ps){
	  iss = MICR_MALLOC;
	  DEBUGPRINT(DP_ERROR,dblistUFlag, ("dblistNew: UNABLE to freeListCalloc db large list\n"));
	  slcCmlogLogMsg(iss, "MALLOC",
					 "%s unable to freeListCalloc big dblist buffer from freelist.\n", 
					 epicsThreadGetNameSelf());
	  return (iss);
	}
	dbdata_ps->max = DBL_WSZBIG;  /* word size of list */
  }
  else {
      dbdata_ps = callocMustSucceed(1, bytes_to_allocate, "dblistNew"); 
      dbdata_ps->max = bytes_to_allocate/2;  /* word size of list */
  }

  dbdata_ps->cur = 0;  

  DEBUGPRINT(DP_INFO,dblistUFlag, ("dblistNew: max = %u; ptr returning = %p\n", 
								   dbdata_ps->max, dbdata_ps));
  *dbdata_pps = dbdata_ps;
  return iss;
} 
/*=============================================================================
   Name: dblistEnlarge 

   Abs:  This routine enlarges a data list. The pointer to pointer of list
         must be valididated prior to calling this routine.

   Args:     Type		    Name	         Access	   Description
	     ---------------    -----------     ---------- ----------------------------
		 const unsigned long bytes_to_allocate  R     Number of bytes to add to list
		 dbheader_ts **const dbdata_pps   W     pointer to pointer of list

   Rem:  This routine enlarges a data list. If it was small (from small list free
         list) and needs a big (from big list free list), it grabs a big, 
		 copies small into big, then frees small.
         If it was big, it mallocs from the heap, copies big into alloc'ed
         memory, then frees big.
         If it was a heap list, it alloc's another twice its size, copies old
         heap list to new heap list, and frees the old heap list.
         The list must be passed as a pointer to a pointer so that it
         may be modified.

   Side: Uses epics freeList to manage big and small pool.
         May allocate from heap memory for super large lists.

   Ret:  vmsstat_t

==============================================================================*/
vmsstat_t dblistEnlarge (const unsigned long bytes_to_allocate, 
						 dbheader_ts ** const dbdata_pps)       
{
  vmsstat_t iss = MICR_OKOK;
  dbheader_ts * new_ps;
  dbheader_ts * old_ps;

  old_ps = *dbdata_pps;
  /*  if dbVersion has changed, flag error */
  /*  if small list, go to big list  */
  if ( old_ps->max == DBL_WSZSM  ) {
    new_ps = freeListCalloc(dblistBigFreeList_p);
	if (!new_ps){
	  iss = MICR_MALLOC;
	  slcCmlogLogMsg(iss, "MALLOC",
					 "%s unable to freelistCalloc big dblist buffer from freelist.\n", 
					 epicsThreadGetNameSelf());
	  return (iss);
	}
    memcpy (new_ps, old_ps, DBL_SZSM);
    freeListFree (dblistSmFreeList_p, (void *)old_ps);
	DEBUGPRINT(DP_DEBUG,dblistUFlag, 
			   ("dblistEnlarge: freeing small list; new list ptr=%p\n",new_ps ));
    new_ps->max = DBL_WSZBIG;   
  }
  /*  if big list, go to heap, get twice size of big list */
  else if (old_ps->max == DBL_WSZBIG  ) {
    new_ps = callocMustSucceed(1,DBL_SZBIG*2, "dblistEnlarge");
    memcpy (new_ps, old_ps, DBL_SZBIG);
    freeListFree (dblistBigFreeList_p, (void *)old_ps);
    new_ps->max = DBL_WSZBIG*2;    
	DEBUGPRINT(DP_DEBUG,dblistUFlag, 
			   ("dblistEnlarge: freeing big list; heap list max = %u words\n", new_ps->max));
  } 
  /*  if heap, realloc more from heap */
  else {    
    /* old_ps = realloc(old_ps, bytes_to_allocate);*/
    new_ps = callocMustSucceed(1, old_ps->max*4, "dblistEnlarge");
    memcpy (new_ps, old_ps, old_ps->max*4);	
	new_ps->max = old_ps->max*2;  
    free(old_ps);
	DEBUGPRINT(DP_DEBUG,dblistUFlag, 
			   ("dblistEnlarge: freeing old heap list; new heap list max = %u words\n", 
				new_ps->max));
  }
  *dbdata_pps = new_ps;
  return iss;
} 

/*=============================================================================
  Name: dblistAdd

  Abs: For the given database terms return the corresponding list of
      "pointers" These pointers may be passed to a subsequent routine
      (see for example "dblget") to retrieve quickly the database
      information they point to.

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
        const int           bytes_in_elem  R       # of bytes of element being added to list
        const int           num_elem       R       # of elements being added to list
        const void *        addMe_p        R       points to data to be added to list
		dbheader_ts ** const db_pps       R/W      ptr to ptr to db list which will contain
		                                           addMe_p contents


  Rem:  

  Side: Memory allocation if *db_pps is not populated, or if list is not long enough to
        hold incoming data.
         Adjusts dbheader field cur.

  Ret:  vmsstat_t:
         DB_OKOK            - good status
         DB_BADPTR          - bad pointer supplied
         DB_NOHEAP          - insufficient HEAP memory available; returned from dblistalloc

==============================================================================*/
vmsstat_t dblistAdd (const unsigned short bytes_in_elem,
					 const unsigned short num_elems,
					 const void * addMe_p,
					 const dbheader_ts ** const db_pps)
{
  vmsstat_t iss =MICR_OKOK; 
  unsigned long bytes_to_add = 0;
  unsigned short max, cur;
  char *   next_ps = NULL;

  /*DEBUGPRINT(DP_DEBUG,dblistUFlag, 
			("dblistAdd: bytes in elem: %d, num elems: %d, addme_p: %p, db_pps: %p, *db_pps; %p\n", 
			bytes_in_elem, num_elems, addMe_p, db_pps, *db_pps));*/
  if (!(*db_pps)){
    if (!SUCCESS(iss=dblistalloc(bytes_in_elem, num_elems, db_pps))) 
      return iss;
  }

  if (!addMe_p) 
    return (iss=DB_BADPTR);

  bytes_to_add = bytes_in_elem * num_elems;
  max = ((dbheader_ts *)(*db_pps))->max;
  cur = ((dbheader_ts *)(*db_pps))->cur;

  if ( (bytes_to_add/2) > (max - (cur + (sizeof(dbheader_ts)/2) ) ) ){
    /* or call dblistEnlarge directly: ? */
	if (!SUCCESS(iss=dblistalloc(bytes_in_elem, num_elems, db_pps))) 
	  return iss;      
  }   

  next_ps = (char *)(*db_pps) + sizeof(dbheader_ts) + (cur * 2); 
  DEBUGPRINT(DP_DEBUG,dblistUFlag, 
			 ("dblistAdd:to list cur=%u; max=%u; bytes: %ld\n", cur, max, bytes_to_add)); 
  memcpy (next_ps, addMe_p, bytes_to_add);
  ((dbheader_ts *)(*db_pps))->cur += bytes_to_add/2 ;
  DEBUGPRINT(DP_DEBUG,dblistUFlag,("dblistAdded contents = %p, to list ptr %p\n",(long *) *(long *)next_ps, next_ps));
  return (iss);
  
}
/*=============================================================================
  Name: dblistPrint

  Abs:  For debug purposes, print out the pointer dblist input.

  Args: Type		    Name	Access	   Description
	------------------- -----------	---------- ----------------------------
        const dbheader_ts * const dblist_ps   R    pointer to list of dict data ptrs

  Rem:  

  Side: Memory allocation.

  Ret:  vmsstat_t:

==============================================================================*/
vmsstat_t  dblistPrint  (const dbheader_ts * const dblist_ps )
{
  vmsstat_t iss = MICR_OKOK;  
  unsigned long   *next_p;            /* next dict ptr location in dblist */
  unsigned short  curElems=0;         /* current # of elements in dblist (ptrs to hash entries) */
  unsigned short  loop;               /* current # bytes in  data list; loop cnter */
  dbhEntry_ts     *dbhEntry_ps;       /* ptr elements in dblist_ps */
  dbdictPUS_ts    *dbdictPUS_ps;      /* dbhEntry_ps->usrPvt points to this element */

  /*  if (!SUCCESS(iss=dbcheckHeader((dbheader_ts * const)dblist_ps, epicsTrue))) {
    return (iss);}
  */
  curElems = dblist_ps->cur/ ( sizeof(dbhEntry_ts *)/2 );
  next_p = (unsigned long *)dblist_ps;
  next_p += sizeof(dbheader_ts)/sizeof(long);      
                                      
  DEBUGPRINT(DP_DEBUG,dblistUFlag,("dblistPrint: dblist elements =%u; max =%u, ptr =%p\n", 
								   curElems,dblist_ps->max, dblist_ps));
  for (loop=0;loop<curElems;loop++){
    dbhEntry_ps = (dbhEntry_ts *)*next_p;        /* get hash entry for this P-U-S */
    DEBUGPRINT(DP_DEBUG,dblistUFlag,("dblistPrint: dbhEntry_ps %u=%p \n",loop, dbhEntry_ps)); 
	if (!dbhEntry_ps){
      DEBUGPRINT(DP_ERROR,dblistUFlag,("dblistPrint: bad dbhEntry_ps dblist ptr \n"));
      DEBUGPRINT(DP_ERROR,dblistUFlag,("\t dblist ptr=%p; next_p=%p \n", dblist_ps, next_p));
      /*return (iss= DB_BADPTR);*/
    }
    else {
      dbdictPUS_ps = dbhEntry_ps->usrPvt;
      if (!dbdictPUS_ps){
		DEBUGPRINT(DP_ERROR,dblistUFlag,("dblistPrint: bad dict ptr \n"));
      }
	  DEBUGPRINT(DP_DEBUG,dblistUFlag,
	    ("\t PUS=%.12s with format=%c, width=%hd, count=%hd, supn=%hd, list ptr=%p \n", 
		(char *)dbhEntry_ps->name,dbdictPUS_ps->format, dbdictPUS_ps->width, dbdictPUS_ps->count,
				  dbdictPUS_ps->supn, next_p));
		  
	  next_p++;               /* next dict addr */
     
    }/*else*/     

  }/*for*/ 

  return (iss);
}
