/*=============================================================================
 
  Name: dbupdate.c
        dbupdate
		dbupdateAll
  
  Abs:  Utilities which write to a database PUS
        Reference web pages on LCLS Controls | Facilities |SLC-Aware IOC | :
                        Database Service (DBS) Design Document 
                        Functional Requirements | SLC Database Service (DBS)
                        Database Input/Output Utilities
                        C Coding Standard


  Auth: 14-MAR-2005, Debbie Rogind (DROGIND) created
  Rev:  dd-mmm-200y,
-----------------------------------------------------------------------------*/
#include "copyright_SLAC.h"    
/*-----------------------------------------------------------------------------  
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
#include <stdio.h>
  int dbupdFlag = DP_ERROR;
#endif
/* c includes */
#include     <stdio.h>
#include     <stdlib.h>    /* malloc, free */
#include     <string.h>    /* memcpy */
#include     <cantProceed.h>

/* slc global vars */
#include     "slcGlob.h"

/* VMS Control system function/status codes  */
#include     "msgfunc.hc"  /* FUNC_MSG function code */
#include     "micrdef.hc"  /* MICR_EXNET_ codes */

/* slc common includes */
#include     "slcMsg.h"    /* DB_UPDATE message define */
#include     "cvtUtil.h"   /* slc conversion utilities */
#include     "timeUtil.h"  /* slc time conversion utilities */
#include     "slcCmlog.h"  /* slcCmlogLogMsg */
#include     "slcUtil.h"   /* slcGetIOCName */ 
#include     "msgQUtil.h"  /* message queue utilities */

/* dbs includes  */  
#include     "db_micro_struc.h"
#include     "dbUtil.h" 
#include     "dbmsgmail.h" 
#include     "dbGlob.h"   
#include     "dbdef.hc"    /* DB_OKOK,,*/
#include     "dbdict.h"
#include     "freeList.h"
#include     "cantProceed.h"
/* dblist  prototypes */
#include    "dbupdate.h"

/* dbupdate diagnostics global variable */
dbupdateJobDiags_ts  dbupdateJobDiags_as[MAX_SLC_JOBS] = { { 0, 0, 0, {0, 0}}, 
 { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}}, 
 { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}}, 
 { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}}, 
 { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}}, 
 { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}}, 
 { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}}, 
 { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}}, 
{ 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}},  { 0, 0, 0, {0, 0}} };

LOCAL vmsstat_t dbupdateSend(const slcJob_te slcJobId_e, 
							 const epicsBoolean waitFlag);
/*=============================================================================
  Name: dbupdate

  Abs:  Called to send ST 2 or 3 data updates from calling job's dbupdateSTHiLo buffer 
        to DBEX.


  Args:     Type		        Name	 Access	   Description
        ------------------- -----------	---------- ----------------------------
	    const slcJob_te        slcJobId_e    R     job thread enumeration
	    const short            supn          R     supertype number 

  Rem:  Error checks input parameters, then calls dbupdateSend
        for rest of logic.

  Side: 

  Ret:  vmsstat_t:

==============================================================================*/
vmsstat_t dbupdate(const slcJob_te slcJobId_e)
{
  vmsstat_t iss = DB_OKOK;
 
  if ((slcJobId_e<0) || (slcJobId_e > MAX_SLC_JOBS) || (slcJobId_e==invalidJob) )
	{
	  return (iss=DB_CANTSEND);
	}  

  dbupdateSend(slcJobId_e, epicsTrue);

  return iss;
}
/*=============================================================================
  Name: dbupdateNoWait

  Abs:  Called to send ST 2 or 3 data updates from calling job's dbupdateSTHiLo buffer 
        to DBEX. Does not wait on job's event prior to exit. This procedure is
        used when 'slcdbUpdate <jobName>' is issued from the ioc shell.


  Args:     Type		        Name	 Access	   Description
        ------------------- -----------	---------- ----------------------------
	    const slcJob_te        slcJobId_e    R     job thread enumeration
	    const short            supn          R     supertype number 

  Rem:  Error checks input parameters, then calls dbupdateSend
        for rest of logic.

  Side: 

  Ret:  vmsstat_t:

==============================================================================*/
vmsstat_t dbupdateNoWait(const slcJob_te slcJobId_e)
{
  vmsstat_t iss = DB_OKOK;
 
  if ((slcJobId_e<0) || (slcJobId_e > MAX_SLC_JOBS) || (slcJobId_e==invalidJob) )
	{
	  return (iss=DB_CANTSEND);
	}  

  dbupdateSend(slcJobId_e, epicsFalse);

  return iss;
}
/*=============================================================================
  Name: dbupdateAll

  Abs:  Called to send ST 2 or 3 data updates for all jobs to DBEX.
 

  Args:     Type		        Name	 Access	   Description
        ------------------- -----------	---------- ----------------------------
	    const slcJob_te        slcJobId_e    R     job thread enumeration
	    const short            supn          R     supertype number 

  Rem:  Sends the DB_UPDATE_ALL message to dbSend in order for it to 
        cycle through all the jobs in order to send updates.

  Side: Called from 'slcdbUpdate ALL*' ioc shell command

  Ret:  vmsstat_t:

==============================================================================*/
vmsstat_t dbupdateAll(void)
{
  vmsstat_t iss = MICR_OKOK;
  dbmsgmail_ts   dbSendMsgBuf_s;    /* message buffer */

  /* does not call dbupdateSend for each job, cuz don't want to wait on events */
  dbSendMsgBuf_s.func = DB_UPDATE_ALL;
  if (!SUCCESS(iss= messageQSend(dbSend, epicsTrue, sizeof(dbSendMsgBuf_s),(void *)&dbSendMsgBuf_s)) )
	{
	  DEBUGPRINT(DP_DEBUG,dbupdFlag,("dbupdateAll: DB_UPDATE message failed\n"));
	  /* return if error ? */
	}
  else 
	DEBUGPRINT(DP_DEBUG,dbupdFlag,("dbupdateAll: sent DB_UPDATE_ALL to dbSend\n")); 
  return iss;
}
/*=============================================================================
  Name: dbupdateSend

  Abs:  Called by dbupdate for purpose of sending ST 2 or 3 data updates from 
        calling job's dbupdateSTHiLo buffer to DBEX. Implemented with a message
        being sent to dbSend in order to send the data.

  Args:     Type		        Name	 Access	   Description
        ------------------- -----------	---------- ----------------------------
	    const slcJob_te        slcJobId_e    R     job thread enumeration
	    const short            supn          R     supertype number 
		const epicsBoolean     waitFlag      R     whether or not to wait on jobs' 
		                                           ack event (autom. true when 
												   being called via dbupdate())

  Rem: Reads job's dbupdateSTHiLo[supn] buffer to see if there is any data waiting
       to be sent. If so, sends DB_UPDATE message to dbSend. If waitFlag is true,
       it waits on job's dbupdateHiLoAckEvent, and upon being released, returns
	   dbudpateHiLoAckStatus.

  Side: 

  Ret:  vmsstat_t:

==============================================================================*/
LOCAL vmsstat_t dbupdateSend(const slcJob_te slcJobId_e,
							 const epicsBoolean waitFlag)
{
  vmsstat_t      iss = MICR_OKOK;
  dbhilo_job_ts *jbhiloST2_ps; 
  dbhilo_job_ts *jbhiloST3_ps;  
  epicsMutexId   jbhiloMutex_ps;
  dbmsgmail_ts   dbSendMsgBuf_s;    /* message buffer */
  int            cntST2, cntST3 = 0;
  
  if (!(slcJobs_as[slcJobId_e].dbupdate_ps)){
	DEBUGPRINT(DP_ERROR,dbupdFlag,
			   ("dbupdateSend: no dbupdate struct for this job enum=%d! \n", slcJobId_e));
	return (iss = DB_CANTSEND);  
  }
  DEBUGPRINT(DP_INFO,dbupdFlag,("dbupdateSend: job enum=%d\n", slcJobId_e));
  /* ST2 start at zero offset in array; ST3, index=1 */
  jbhiloST2_ps = &(slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateSTHiLo[0]);
  jbhiloST3_ps = &(slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateSTHiLo[1]);
  jbhiloMutex_ps = slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateHiLoMutex_ps;
  
  /* read the hilo buffer count; put mutex around the read */
  if (jbhiloMutex_ps)		 
    epicsMutexMustLock(jbhiloMutex_ps);
  else {
    DEBUGPRINT(DP_ERROR,dbupdFlag,("dbupdateSend: bad dbupdateHiLoMutex_ps \n"));
    return (iss=DB_BADPTR);
  }
  cntST2 = jbhiloST2_ps->wxn;
  cntST3 = jbhiloST3_ps->wxn;
  if (jbhiloMutex_ps)		 
	epicsMutexUnlock(jbhiloMutex_ps);
  
  if ((cntST2 >0)||(cntST3>0))         /* if there are updates waiting to be sent */
    {      
      dbSendMsgBuf_s.jobId = (int)slcJobId_e;
      dbSendMsgBuf_s.func = DB_UPDATE;
	  /* update num of updates sent and current update time for this job */
	  dbupdateJobDiags_as[slcJobId_e].updToVMSCnt++;
	  epicsTimeGetCurrent(&(dbupdateJobDiags_as[slcJobId_e].lastUpdTime) );
      if (!SUCCESS(iss= messageQSend(dbSend, epicsTrue, sizeof(dbSendMsgBuf_s),(void *)&dbSendMsgBuf_s)) )
		{
		  DEBUGPRINT(DP_ERROR,dbupdFlag,("dbupdateSend: DB_UPDATE message failed\n"));
		  /* return if error ? */
		}
      else 
		DEBUGPRINT(DP_WARN,dbupdFlag,
				   ("dbupdateSend: sent DB_UPDATE to dbSend for jobenum=%d\n",slcJobId_e)); 
      if (waitFlag) { 
		DEBUGPRINT(DP_INFO,dbupdFlag,
				   ("dbupdateSend: waiting on dbupdateHiLoAckEvent....\n"));  
		epicsEventMustWait(slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateHiLoAckEvent_ps);
		DEBUGPRINT(DP_INFO,dbupdFlag,
				   ("dbupdateSend: dbupdateHiLoAckEvent succesfully signalled.\n"));
		if (!SUCCESS(iss=slcJobs_as[slcJobId_e].dbupdate_ps->dbudpateHiLoAckStatus_s)){
		  dbupdateJobDiags_as[slcJobId_e].VMStoSlcAckDumpCnt++;
		}
		else {
		  dbupdateJobDiags_as[slcJobId_e].VMStoSlcAckGoodCnt++;
		}
		return (iss); 
      }
    }
  else {
    DEBUGPRINT(DP_WARN,dbupdFlag,
			   ("dbupdateSend: ST2/3 hilo bufs empty; no data sent. \n"));
    /* should status be set to no data ? */
  }
    return iss;
}
/*=============================================================================
  Name: dbupdateHiLo

  Abs:  Store lower & upper bounds of secondary just updated   
        in supertype 2or3 (supn) on behalf of specified subjob.  

  Args:     Type		        Name	 Access	   Description
        ------------------- -----------	---------- ----------------------------
	    const slcJob_te        slcJobId_e    R     job thread enumeration
	    const short            supn          R     supertype number 
		const unsigned long    sptrLo        R     low offset into dbnodes for supn
        const unsigned long    sptrHi        R     high offset into dbnodes for supn

  Rem:  Called from dblput().
        Item bounds stored may get combined (by dbupdateCompress)  
              into fewer items, either if list containing stored     
              item bounds is already full when dbupdateHiLo is      
              called or just before items are actually sent by       
              dbSend.  

  Side:  Global areas slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateSTHiLo[supnInd]) and
         slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateHiLoMutex_ps
		 are written.
		 Ported from dbmain.c's  dbhilo_update. 

  Ret:  vmsstat_t:
        MICR_OKOK, DB_CANTSEND,
		DB_TRASHED
==============================================================================*/ 
vmsstat_t dbupdateHiLo(const slcJob_te slcJobId_e, 
				  const short supn,
				  const unsigned long sptrLo,
				  const unsigned long sptrHi)

{  
#define DBUP_MAXLOOK  4
  vmsstat_t      iss = MICR_OKOK;
  dbhilo_job_ts *jbhilo_ps;
  epicsMutexId   jbhiloMutex_ps;
  unsigned       jj;
  short          supnInd;

  /*----------------------------------------------*/
  supnInd = supn-2; /* ST2 start at zero offset in array */
  if (!(supnInd==0 || supnInd==1 )) {
	DEBUGPRINT(DP_ERROR,dbupdFlag,("dbupdateHiLo: bad supn \n"));
	iss = DB_CANTSEND;
	slcCmlogLogMsg(iss, "DB_CANTSEND", "%s dbupdateHiLo supn out of range error\n", 
				   epicsThreadGetNameSelf()); 
	return iss;
  }
  if (!(slcJobs_as[slcJobId_e].dbupdate_ps)){
	DEBUGPRINT(DP_ERROR,dbupdFlag,("dbupdateHiLo: no dbupdate struct for this job! \n"));
	return (iss = DB_CANTSEND);  
  }
  DEBUGPRINT(DP_DEBUG,dbupdFlag,("dbupdateHiLo: supn=%d, supnInd=%d, job enum=%d\n", 
								 supn,supnInd, slcJobId_e));
  jbhilo_ps = &(slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateSTHiLo[supnInd]);

  jbhiloMutex_ps = slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateHiLoMutex_ps;

  DEBUGPRINT(DP_INFO,dbupdFlag,("dbupdateHiLo: sptrLo=%ld; sptrHi=%ld \n", sptrLo, sptrHi));
  if (!(1 <= sptrLo && sptrLo <= sptrHi))
	{
	  iss = DB_TRASHED;
	  slcCmlogLogMsg(iss, "DB_TRASHED", "%s dbupdateHiLo sptr errors\n", epicsThreadGetNameSelf());
	  DEBUGPRINT(DP_ERROR,dbupdFlag,("dbupdateHiLo: sptrLo=%ld; sptrHi=%ld \n", sptrLo, sptrHi));
	  return iss;
	}
  else
	{
	  if (jbhiloMutex_ps){
		DEBUGPRINT(DP_INFO,dbupdFlag,("dbupdateHiLo: Locking jbhiloMutex.. \n"));		 
		epicsMutexMustLock(jbhiloMutex_ps);
	  }
	  else {
		DEBUGPRINT(DP_ERROR,dbupdFlag,("dbupdateHiLo: bad dbupdateHiLoMutex_ps \n"));
		return (iss=DB_BADPTR);
	  }  
	  
	   /* If given update item is near enough to any of DBUP_MAXLOOK items */
	   /* most recently added to this subjob's list, then combine given    */
	   /* update item into existing list item and return.                  */
	   
	   for (jj = jbhilo_ps->wxn;  jj > 0 &&  jj + DBUP_MAXLOOK > jbhilo_ps->wxn;  )
		 {
		   --jj;
		   if (jbhilo_ps->wxa[jj][1] + jbhilo_ps->difmin >= sptrLo &&
			   sptrHi + jbhilo_ps->difmin >= jbhilo_ps->wxa[jj][0])
			 {
			   if (jbhilo_ps->wxa[jj][0] > sptrLo)
				 jbhilo_ps->wxa[jj][0] = sptrLo;
			   if (jbhilo_ps->wxa[jj][1] < sptrHi)
				 jbhilo_ps->wxa[jj][1] = sptrHi;
			   DEBUGPRINT(DP_INFO,dbupdFlag,("dbupdateHiLo: unlocking jbhiloMutex & return from for loop.. \n"));
			   if (jbhiloMutex_ps)		 
				 epicsMutexUnlock(jbhiloMutex_ps);
			   return iss;

			 } /*if*/
		 } /*for*/
	   
	   /* If this subjob's list is already full, then try to compress it */
	   /* or at least find a pair of update items as close to each other */
	   /* as any other pair and combine that pair to one.                */
	   
	   if (jbhilo_ps->wxn >= MAXDBSNDPJOB) {
		 dbupdateCompress (slcJobId_e,supn, 1);
	   }
	   
	   /* Add given update item to this subjob's list. */
	   
	   if (jbhilo_ps->wxn < MAXDBSNDPJOB)
		 {
		   jbhilo_ps->wxa[jbhilo_ps->wxn][0] = sptrLo;
		   jbhilo_ps->wxa[jbhilo_ps->wxn][1] = sptrHi;
		   ++jbhilo_ps->wxn;
		   DEBUGPRINT(DP_WARN,dbupdFlag,("dbupdateHiLo: update count for job enum %d = %ld\n", 
										 slcJobId_e, jbhilo_ps->wxn));
		   /*sndctrl_erchc[sizeof(sndctrl_erchc)-2] = '2';
			 errchk(sndctrl_erchc, sizeof(sndctrl_erchc)-1, &rqexc);*/
		 }
	   else
		 {
		   ++jbhilo_ps->errc;
		   iss = DB_CANTSEND;
		   slcCmlogLogMsg(iss, "DB_CANTSEND", "%s dbupdateHiLo MAXDBSNDPJOB error\n", 
						  epicsThreadGetNameSelf()); 
		 }
	   DEBUGPRINT(DP_INFO,dbupdFlag,("dbupdateHiLo: unlocking jbhiloMutex.. \n"));
	   if (jbhiloMutex_ps)		 
		 epicsMutexUnlock(jbhiloMutex_ps);
	 }
  return iss;
}                                                 
/*=============================================================================
  Name: dbupdateCompress

  Abs:  Compress lower & upper bounds of calling subjobs' pending list of   
        supertype 2or3 (supn) updates.  

  Args:     Type		        Name	 Access	   Description
        ------------------- -----------	---------- ----------------------------
	    const slcJob_te        slcJobId_e    R     job thread enumeration
	    const short            supn          R     supertype number 
        const short            xxxf          R     Flag:  1 - if after attempted
                                                   compaction the list is still full   
                                                   then combine two closest items;     
                                                   0 = dont.

  Rem:  In any subjob's jbhilo element, difmin is minimum      
          separation (in words) between update items for         
            deciding to send update items separately. 

  Side:  Global area slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateSTHiLo[supnInd])
		 are written.
		 Assume we are already in  slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateHiLoMutex_ps 
		 Ported from dbmain.c's microdbsendc

  Ret:   None
==============================================================================*/   
void dbupdateCompress(const slcJob_te slcJobId_e,
					  const short supn,
					  const short xxxf)
{ 
  dbhilo_job_ts *jbhilo_ps;  
  unsigned       jj, jk; 
  unsigned       svjj = 0; 
  unsigned       svjk = 0;
  unsigned long  ddmin, ddt;
  short          supnInd = supn-2; /* ST2 start at zero offset in array */

  /*----------------------------------------------*/
  if (!(supnInd==1 ||supnInd==2 )) {
	DEBUGPRINT(DP_ERROR,dbupdFlag,("dbupdateHiLo: bad supn \n"));
	return;
  }
  jbhilo_ps = &slcJobs_as[slcJobId_e].dbupdate_ps->dbupdateSTHiLo[supnInd]; 
  /* Following double loop looks at all pairs of update items pending for  */
  /* given subjob (where each update item is a doubleton:  lower bound and */
  /* upper bound;  each bound is a 1-biased index into the micro's entire  */
  /* supertype 3 block considered as a word array including its database   */
  /* superblock header;  lower bound is index of first word to be          */
  /* transmitted;  upper bound is index of last word to be transmitted).   */
 
  for (jj = 0;  jj + 1 < jbhilo_ps->wxn;  ++jj)
	{
	  for (jk = jj + 1;  jk < jbhilo_ps->wxn;  )
		{
		  /* For each pair of update items, if upper end of jj's item is below */
		  /* lower end of jk's item by more than (currently selected) minimum  */
		  /* distance (in words), or similarly with jj and jk the other way    */
		  /* around, then dont combine the items in this pair.  Otherwise, do  */
		  /* combine them (into jj's item) and shorten the list (eliminate     */
		  /* jk's item) and restart jk.                                        */
		  
		  if (jbhilo_ps->wxa[jj][1] + jbhilo_ps->difmin >= jbhilo_ps->wxa[jk][0] &&
			  jbhilo_ps->wxa[jk][1] + jbhilo_ps->difmin >= jbhilo_ps->wxa[jj][0])
			{
			  if (jbhilo_ps->wxa[jj][0] > jbhilo_ps->wxa[jk][0])
				jbhilo_ps->wxa[jj][0] = jbhilo_ps->wxa[jk][0];
			  if (jbhilo_ps->wxa[jj][1] < jbhilo_ps->wxa[jk][1])
				jbhilo_ps->wxa[jj][1] = jbhilo_ps->wxa[jk][1];
			  memmove(jbhilo_ps->wxa[jk], jbhilo_ps->wxa[jk+1],
					  (jbhilo_ps->wxn-(jk+1))*sizeof(jbhilo_ps->wxa[0]));
			  --jbhilo_ps->wxn;
			  jk = jj + 1;
			}
		  else
			{
			  ++jk;
			}
		}
	}
  
  if (xxxf & 1 && jbhilo_ps->wxn >= MAXDBSNDPJOB)
	{
	  /* At this moment we know that no pair of update items in this    */
	  /* subjob's list overlaps, but this subjob's list is still full.  */
	  /* Find the minimum distance separating any pair of update items; */
	  /* then combine a pair separated by that distance.  Remember that */
	  /* distance (plus a little) for subsequent compaction attempts,   */
	  /* until after the next call to micro_dbsend on behalf of this    */
	  /* subjob.                                                        */
	  
	  ddmin = 0x7FFFFFFF;
	  for (jj = 0;  jj + 1 < jbhilo_ps->wxn;  ++jj)
		{
		  for (jk = jj + 1;  jk < jbhilo_ps->wxn;  ++jk)
			{
			  if (((int) (ddt = jbhilo_ps->wxa[jk][0] - jbhilo_ps->wxa[jj][1]) >= 0 ||
				   (int) (ddt = jbhilo_ps->wxa[jj][0] - jbhilo_ps->wxa[jk][1]) >= 0) &&
				  ddmin > ddt)
				{
				  ddmin = ddt;
				  svjj = jj;
				  svjk = jk;
				}
			}
		}
	  
	  if (ddmin < 0x7FFFFFFF)
		{
		  if (jbhilo_ps->wxa[svjj][0] > jbhilo_ps->wxa[svjk][0])
			jbhilo_ps->wxa[svjj][0] = jbhilo_ps->wxa[svjk][0];
		  if (jbhilo_ps->wxa[svjj][1] < jbhilo_ps->wxa[svjk][1])
			jbhilo_ps->wxa[svjj][1] = jbhilo_ps->wxa[svjk][1];
		  memmove(jbhilo_ps->wxa[svjk], jbhilo_ps->wxa[svjk+1],
				  (jbhilo_ps->wxn-(svjk+1))*sizeof(jbhilo_ps->wxa[0]));
		  --jbhilo_ps->wxn;
		  jbhilo_ps->difmin = ddmin;
		}
	}
}                                              

