/*=============================================================================
 
  Name: iocCmlog.h

  Abs:  IOC CMLOG Interface Public Functions

  Auth: 05-Dec-2004, James Silva (JSILVA):
  Rev:  20-Jan-2005, S. Allison  (SAA)

-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  (newest to oldest)  
        20-Sep-2006, Ron MacKenzie:
           Removed several functions.
 
=============================================================================*/

#ifndef IOC_CMLOG_H  /* guard macro */
#define IOC_CMLOG_H  /* guard macro */

#include <stdarg.h>  /* va_list */

#define IOC_CMLOG_NOPRINTF            0
#define IOC_CMLOG_PRINTF              1
#define IOC_CMLOG_USE_VMS_SEVERITY    -1

/*
  Arguments:
  Name	           Description
  -------------    --------------------------------------------------------
  facility_c       Facility string (if NULL, "EPICS" is used)
  status	   Status code (EPICS or VMS)
  severity	   EPICS severity code (-1: status code is VMS
                                         4: informational)
  doPrintf	   0=LOG only, 1=printf and LOG
  status_c	   Status string (if NULL, defaults appropriate to status code)
  destination_c    Destination string (can be NULL)
  filename_c	   not used
  lineno	   not used
  format_c 	   Format string for message text
*/
int iocCmlogLogMsg  (const char * const  facility_c, 
                     const unsigned long status,
                     const int           severity,
                     const int           doPrintf,
		     const char * const  status_c, 
		     const char * const  destination_c,
		     const char * const  filename_c,  
		     const unsigned long lineno,  
		     const char * const  format_c, ...);
 
int iocCmlogVLogMsg (const char * const  facility_c, 
                     const unsigned long status, 
                           int           severity,
                     const int           doPrintf,
		     const char * const  status_c, 
		     const char * const  destination_c,             
		     const char * const  filename_c,  
		     const unsigned long lineno,           
		     const char * const  format_c,  
		     va_list format_args_p);


#endif /* IOC_CMLOG_H */
