/*==============================================================================
 
  Abs:  Magnet function codes (VAX, RMX, RTEMS and Linux)
 
  Name: mgntFunc.hc
 
  Rem:  Below is a description of the message request
        function word decoded.  The lower byte of the
        message function, bits 1-8 inclusive, hold a
        numeric code describing the type of magnet  
        operation being requested.  The second byte 
        of the function, bits 9-15 inclusive, indicate
        the micro job identiciation code. The magnet job 
        id is 2, and therefore ALL magnet function must 
        be within the range 200 - 2ff (hex). The job id
        tells the message service which micro job is to 
       receive the message request. The most significant
        bit, bit 16, is always clear upon the receipt of
        a message request. However, upon sending a reply
        message, to the initial request, this bit is then 
        set.
 
         BIT   Message Function                   BIT
 
         0001  DAC Zero    - 'ZERO'               0001
         0002  Standardize - 'STDZ'               0002
         0004  Calibrate   - 'CALB'               0004
         0008  Check       - 'CHCK'               0008
         0010  Trim        - 'TRIM'               0010
         0020  Perturb     - 'PTRB'               0020
         0040  Diagnostic Calibrate 'DCAL'        0040
         0080  Magnet Ramping - 'RAMP'            0080
         0100   J                                 0100
         0200    O                                0200
         0400     B                               0400
         0800            I                        0800
         1000              D                      1000
         2000                      N              2000
         4000                        O            4000
         8000  Response flag                      8000
 
        When adding new function codes be aware that
        some function are setup to be a combination of
        other functions as are those listed below:
  
        Touchup           - 'TOUC' = MGNT_ZERO .OR. MGNT_TRIM
        Reset             - 'RSET' = all functions are set
        Actual to Desired - '_ACT' = MGNT_CHCK .OR. MGNT_DCAL

  Side: If this file is modified please update
        corresponding VAX and RMX Fortran includes
            magfunc.txt
            magfunc.int
            magfunc.hc
 
  Auth: 26-Nov-1988, Nan Phinney (NAN):
  Revw: dd-mmm-yyyy, Reviewer's Name (.NE. Author's Name)
 
--------------------------------------------------------------------------------
 
  Mod:  
        14-Jul-2005, K. Luchini       (LUCHINI):
            ported from the Alph. renamed from magfunc.hc to mgntFunc.hc
        14-Jun-2001, Lee Ann Yasukawa (LEEANN):
            Adding function definition for MGNT_INIT.
        04-Jan-1996, Lee Ann Yasukawa (LEEANN):
            Added new RAMP function: MGNT_RAMP.
        16-Aug-1995, Kristi L. Luchini (LUCHINI):
            Chg MGNT_ACT_TO_DES to MGNT__ACT because 
            of conflict w/ magnet msg code of same name
        15-Aug-1995, Kristi L. Luchini (LUCHINI):
            Add MGNT_ACT_TO_DES = MGNT_CHCK | MGNT_DACL
        06-Sep-1994, Alan Cheilek (CHEILEK):
	    Add MGNT_DAOC message for DAOC restore on crate init
	26-May-1992, T. Gromme (TEG):
            Translated from MSGFUNC.TXT.
        16-Jan-1989, Nan Phinney (NAN):
            Get rid of STON       
        16-Nov-1988,  HMK:
            Redefined function STDZ = STDZ + TRIM, and added new func
            SZON = STANDARDIZE ONLY.
 
==============================================================================*/
#ifndef mgntFunc_HC
#define mgntFunc_HC

#define  MGNT_CALB  0x0204
#define  MGNT_STDZ  0x0202
#define  MGNT_CHCK  0x0208
#define  MGNT_TRIM  0x0210
#define  MGNT_PTRB  0x0220
#define  MGNT_ZERO  0x0201
#define  MGNT_RSET  0x02FF
#define  MGNT_TOUC  0x0211
#define  MGNT_DCAL  0x0240
#define  MGNT_DAOC  0x0234 
#define  MGNT__ACT  0x0248
#define  MGNT_RAMP  0X0280

#endif  /* mgntFunc_HC */
