/*=============================================================================

  Name: mgntMacros.h

  Abs:  Magnet macro definitions

  Auth: 10-Jul-2005, K. Luchini      (LUCHINI)
  Rev:  dd-mmm-yyyy, Reviewer's Name (USERNAME)

-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  (newest to oldest)  

        04-Oct-2005, K. Luchini (LUCHINI):
           add MGNT_LOCAL_FUNC

=============================================================================*/
#ifndef mgntMacro_H
#define mgntMacro_H 


#include "slcStructs.h"     /* for slcScpFunc_ts               */
#include "slcMsg.h"         /* IOC_ASYNC                       */
#include "mgntParam.h"      /* MGNT_CALB_TIMEOUT,etc           */
#include "mgntFunc.hc"      /* MGNT_CALB,MGNT_STDZ,etc         */
#include "mgntTypes.h"      /* ngntPrimSecn_ts,prim_ta         */


#define MGNT_FUNC_TIMEOUT \
  const unsigned long mgntFuncTimeout_a[MGNT_NUM_FUNCS] =\
     { MGNT_CALB_TIMEOUT,\
       MGNT_STDZ_TIMEOUT,\
       MGNT_CHCK_TIMEOUT,\
       MGNT_TRIM_TIMEOUT,\
       MGNT_PTRB_TIMEOUT,\
       MGNT_TOUC_TIMEOUT,\
       MGNT_DCAL_TIMEOUT,\
       MGNT_DAOC_TIMEOUT,\
       MGNT__ACT_TIMEOUT,\
       MGNT_ZERO_TIMEOUT,\
       MGNT_CHCK_TIMOEOUT }
/*
** The list of functions that can take a long time and therefore
** require that the Alpha be notified which function is currently
** being processed. This is done by the magnet handler setting
** CSTR.1.MAGF with magnet function code being executed.
*/
#define MGNT_FUNC_TO_ALPHA \
    const unsigned short mgntFuncToAlpha_a[MGNT_NUM_FUNCS]= \
                          {0,\
                           MGNT_CALB,\
                           MGNT_TRIM,\
                           MGNT_STDZ,\
                           (MGNT_STDZ|MGNT_TRIM),\
                           MGNT_DCAL,\
                           MGNT_ZERO,\
                            0,0,0,0}

/*
** This is the list of valid local magnet message functions, of which there
** is only one, with the exception of IOC_STOP. However, the IOC_STOP
** function is check specifically in code and not from this table of
** magnet local functions.
*/
#define MGNT_LOCAL_FUNC \
       const slcScpFunc_ts  mgnt_local_func_as[MGNT_NUM_LOCAL_FUNCS] = \
                  {{ "IOC_ASYNC", IOC_ASYNC, MGNT_CHCK_TIMEOUT, mgntCheck, NULL }}

#define MGNT_PRIM \
   const prim_ta  mgntPrim_ac[MGNT_NUM_PRIMS]= \
       { _LGPS,_QUAD,_XCOR,_YCOR,_SOLN,\
         _BEND,_BTRM,_QTRM,_SMPS,_SEXT }
/* above are defined in mgntParam.h file */

#define MGNT_PRIM_ENUM \
   const mgntPrim_te mgntPrim_ae[MGNT_NUM_PRIMS]=\
      { lgps_e,quad_e,xcor_e,ycor_e,soln_e,bend_e,btrm_e,qtrm_e,smps_e,sext_e } 

/*
** list of secondaries associated for each magnet primary.
*/
#define MGNT_PRIM_SECN \
   const mgntPrimSecn_ts  mgntPrimSecn_as[MGNT_NUM_PRIMS][MGNT_MAX_SECN]= \
    {{ _LGPS, 2, {"BDES","HSTA",0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0} },\
     { _QUAD, 2, {"BDES","HSTA",0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0} },\
     { _XCOR, 2, {"BDES","HSTA",0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0} },\
     { _YCOR, 2, {"BDES","HSTA",0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0} },\
     { _SOLN, 2, {"BDES","HSTA",0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0} },\
     { _BEND, 2, {"BDES","HSTA",0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0} },\
     { _BTRM, 2, {"BDES","HSTA",0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0} },\
     { _QTRM, 2, {"BDES","HSTA",0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0} },\
     { _SMPS, 2, {"BDES","HSTA",0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0} },\
     { _SEXT, 2, {"BDES","HSTA",0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0} }}

#endif /* mgntMacros_H */
