/*=============================================================================

  Name: mgntParam.h

  Abs:  Magnet parameters

  Auth: 10-Jul-2005, K. Luchini      (LUCHINI)
  Rev:  dd-mmm-yyyy, Reviewer's Name (USERNAME)

-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  (newest to oldest)  

        04-Oct-2005, K. Luchini (LUCHINI):
           add MGNT_NUM_LOCAL_FUNCS
		02-Mar-2007, D. Rogind 
		   changed HSTACTRL.RVAL to HSTACTRL since it is a longout
=============================================================================*/
#ifndef mgntParam_H
#define mgntParam_H 


#ifndef ERROR 
#define ERROR          -1
#endif
#ifndef OK
#define OK              0
#endif

/* Time between magnet check functions (adc update) in seconds */
#define MGNT_TIME_BETWEEN_CHECK  2.0

/* SLC database true magnet primaries supported by SLC-Aware IOC */
#define MGNT_NUM_PRIMS 10          /* number of primarires (supertype 1)      */
#define MGNT_MAX_SECN  10          /* maximum number of secondaries           */
#define _LGPS "LGPS\0"
#define _QUAD "QUAD\0"
#define _XCOR "XCOR\0"
#define _YCOR "YCOR\0"
#define _SOLN "SOLN\0"
#define _BEND "BEND\0"
#define _BTRM "BTRM\0"
#define _QTRM "QTRM\0"
#define _SMPS "SMPS\0"
#define _SEXT "SEXT\0"
/* #define _PHAS "PHAS"*/
            
#define _WILDCARD "ALL*"               /* Wildcard */

/* 
** Num of valid SLC-Aware Magnet functions.
** Note: this includes IOC_ASYNC which is a
** local function sent to the magnet handler
** to perform a MGNT_CHCK.
*/
#define MGNT_NUM_FUNCS  11   
#define MGNT_NUM_LOCAL_FUNCS  1

/* Timeouts for each function */
#define MGNT_DEFAULT_TIMEOUT 40 /* for 2 sec cycle time in prod */ 
#define MGNT_CALB_TIMEOUT  MGNT_DEFAULT_TIMEOUT
#define MGNT_DCAL_TIMEOUT  MGNT_DEFAULT_TIMEOUT
#define MGNT_ZERO_TIMEOUT  MGNT_DEFAULT_TIMEOUT
#define MGNT_TRIM_TIMEOUT  MGNT_DEFAULT_TIMEOUT
#define MGNT_PTRB_TIMEOUT  MGNT_DEFAULT_TIMEOUT
#define MGNT_CHCK_TIMEOUT  MGNT_DEFAULT_TIMEOUT
#define MGNT__ACT_TIMEOUT  MGNT_DEFAULT_TIMEOUT
#define MGNT_DAOC_TIMEOUT  MGNT_DEFAULT_TIMEOUT
#define MGNT_STDZ_TIMEOUT  300 /* for 2 sec cycle time in prod */
#define MGNT_TOUC_TIMEOUT  MGNT_DEFAULT_TIMEOUT


#define I_CALB   0
#define I_STDZ   1
#define I_CHCK   2
#define I_TRIM   3 
#define I_PTRB   4
#define I_TOUC   5
#define I_DCAL   6
#define I_DAOC   7
#define I__ACT   8
#define I_ZERO   9
#define I_ASYN  10

/* CTRL function menu indices (menu order)*/
#define CTRL_READY          0
#define CTRL_SNL_PERTURB    1
#define CTRL_TRIM           2
#define CTRL_CALB           3
#define CTRL_STDZ           4
#define CTRL_DEGAUSS        5
/* define functions handled in record processing here */
/* Note: the following order is important,            */
/* as affects mgntControl.db CTRLGO fanout order      */
#define CTRL_PERTURB        6
#define CTRL_DAC_ZERO       7
#define CTRL_RESET          8
#define CTRL_TURN_ON        9
#define CTRL_TURN_OFF      10
#define CTRL_SAVE_BDES     11
#define CTRL_LOAD_BDES     12
#define CTRL_UNDO_BDES     13
#define CTRL_FEEDBACK      14
#define CTRL_UNAVAILABLE   15

/* shared with mgntControl.h */
#define STATUS_DONE         0  /* req wait */   
#define STATUS_RAMPING      1
#define STATUS_WAITING      2
#define STATUS_STDZMINI     3  
#define STATUS_UNAVAILABLE  4
#define STATUS_RAMP_TIMEOUT 5
#define STATUS_ERROR        6
#define STATUS_TIMEOUT      7
#define STATUS_ABORT        8   
#define STATUS_DISABLED     9
#define STATUS_FEEDBACK    10
#define STATUS_SETTLING    11
#define STATUS_OK          20

#define TRIMTYPE_3          0
#define TRIMTYPE_LINEAR     1

#define FUNC_READY          0
#define FUNC_NOTUSED        1
#define FUNC_TRIM           2
#define FUNC_CALB           3
#define FUNC_STDZ           4
#define FUNC_DEGAUSS        5
#define FUNC_PERTURB        6
#define FUNC_DAC_ZERO       7
#define FUNC_RESET          8
#define FUNC_TURN_ON        9
#define FUNC_TURN_OFF      10
#define FUNC_SAVE_BDES     11
#define FUNC_LOAD_BDES     12
#define FUNC_UNDO_BDES     13
#define FUNC_FEEDBACK      14

/* CTRL function menu indices (menu order)*/
#define CTRL_INDEX_READY     0
#define CTRL_INDEX_TRIM      1
#define CTRL_INDEX_STDZ      2
#define CTRL_INDEX_PERTURB   3
#define CTRL_INDEX_ZERO      4
#define CTRL_INDEX_CALB      5
#define CTRL_INDEX_RESET     6


/* pv names */
/* straight epics (no slc equiv) */
/* readback for CTRL: */
#define PV_CTRL "CTRL.RVAL"
/* setpoint for CTRL: */
#define PV_CTRL_SETPT "FUNC" 
/* readback for CTRLSTATE (status):*/
#define PV_CTRLSTATE "CTRLSTATE"
/* slc to epics name map */
#define PV_EPICSHSTA "HSTA"
#define PV_SLCHSTA "HSTACTRL" 
#define PV_STAT "STAT"
#define PV_TTIM "TRIMTOD"

#define PV_BDES "BDES"
#define PV_BACT "BACT"
#define PV_BMON "BMON"
#define PV_IDES "IDES"
#define PV_IACT "IACT"
#define PV_RIPL "IRIPL"
#define PV_IPRV "IACTPREVOK"
#define PV_IMON "IMON"
#define PV_IMAX "IDES.HIGH"
#define PV_CTIM "CALBTOD" 
#define PV_KTIM "STDZTOD"
#define PV_ZTIM "STDZLOSTTOD" 
#define PV_ITRY "TRIMTRYCNT"

#define CTRL_IDLE 0 
#define CTRL_FBCK 1 

#define CB_CTRL 0
#define CB_HSTA 1
#define CB_BDES 2
#endif /* mgntParam_H */
