/*=============================================================================

  Name: mgntTypes.h

  Abs:  Magnet structures

  Side: None

  Auth: 10-Jul-2005, K. Luchini      (LUCHINI)
  Rev:  19-Oct-2005, Debbie Rogind   (DROGIND)
NOTE: This file cannot be included in a file that also includes CA
  monitor file "cadef.h"
-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  (newest to oldest)  

        DD-MMM-YYYY, First Lastname  (USERNAME):
           Changed such and such to so and so. etc. etc.

=============================================================================*/
#ifndef mgntTypes_H
#define mgntTypes_H 

#include <epicsTypes.h> /* epicsBoolean   */
#include <epicsTime.h>  /* epicsTimeStamp */
#include <dbAddr.h>     /* DBADDR         */
#include <epicsEvent.h> /* epicsEventId   */
  
#include "dblist.h"     /* dbheader_ts    */
#include "mgntParam.h"  /* MGNT_NUM_PRIMS */
#include "slcStructs.h" /* vmsTimeStamp_ta */

typedef char prim_ta [5];
typedef char unit_ta [5];
typedef char secn_ta [5];
typedef char pvName_ta[80];

/* Primary enums */
typedef enum {
  lgps_e=0,
  quad_e=1,
  xcor_e=2,
  ycor_e=3,
  soln_e=4,
  bend_e=5,
  btrm_e=6,
  qtrm_e=7,
  smps_e=8,
  sext_e=9
}mgntPrim_te;

/* For calls to mgntAsyncCheck(), or mgntAsyncCheckUnit(), determines */
/*   whether slc or epics is the "master" for BDES and HSTA changes */
typedef enum {
  slcBdesHstaMaster_e,
  epicsBdesHstaMaster_e
} mgntMaster_te;

typedef struct {
    prim_ta       prim_c;                  /* primary associated with this list of secondaries */
    unsigned      long nsecn;              /*  of secondaries for this primary                 */
    secn_ta       secn_ac[MGNT_MAX_SECN];  /* 4-char secondary name in SLC database            */
    unsigned long mon_a[MGNT_MAX_SECN];
}mgntPrimSecn_ts;

/* 
** This structure is the enum of PRIM:LOCA:MD:CTRL states 
*/
typedef enum {
  mgntCtrlOff,
  mgntCtrlOn,
  mgntCtrlTrim, 
  mgntCtrlPtrb, 
  mgntCtrlCal, 
  mgntCtrlDiagCal,
  mgntCtrlIactToBact,
  mcntCtrlDaoc,
  mgntCtrlZero,
  mgntCtrlReset
} mgntCtrl_te;

typedef enum {
  BDES,HSTA,CTRL,BUSY
}mgntSecn_te;

typedef struct {
  epicsBoolean     busy;        /* set whenever CTRL!=IDLE */
  epicsBoolean     scp;         /* set when waiting for scp function to complete */
  /* REMOVE ireq - not used **************** */
  unsigned long    ireq;        /* item number in the requst message               */
  vmsstat_t        iss;
  unsigned short   primIndex;   /* index of this unit's primary */
  unsigned short   unitIndex;   /* index of this unit */
 
  char             *prim_pc;    /* ptr to a NULL terminated primary name           */
  mgntPrim_te      prim_e;      /* primary emum  tells what type of unit this is   */
  char             unit_ac[5];  /* Null terminated 4 char ASCII (slc) unit number        */ 
  unsigned short   slcUnitsI;   /* offset into the master mgntInfo_s.dlist_s.slcAllUnits list */       
  /*
  **  EPICS db addresses for the secondaries 
  **  that are read and written upon SCP function request
  */
  /* note - .precord = NULL if doesn't exist */
  /* note - naming is in slc nomenclature; actual epics pv may have different name */
  DBADDR bdesAddr_s;
  DBADDR hstaAddr_s;              /* also read for fbck units */
  DBADDR hstaCtrlAddr_s;          /* for updating epics setpoint from slc */
  DBADDR ctrlAddr_s;              /* point to PRIM:LOCA:UNIT:CTRL.RVAL readback     */
  DBADDR ctrlSetptAddr_s;         /* CTRL setpoint */
  DBADDR ctrlStateAddr_s;         /* CTRLSTATE readback */

  /* epics values received via mgntAsync callback hdlr */
  unsigned short ctrlVal;         /* value from CTRL callback (readback)*/
  float          bdesVal;
  unsigned short hstaVal;

  /* read upon CTRL=IDLE transision in mgntAsync callback hdlr */
  /* all primaries have these secns: */
  /*  hstaAddr_s, */
  DBADDR statAddr_s;
  DBADDR ctrlstateAddr_s;  
  DBADDR ttimAddr_s;
  DBADDR itryAddr_s;
  /* LGPS,BEND,BTRM,QTRM,QUAD,XCOR,YCOR,SEXT?,SMPS,SOLN have this secn superset: */
  /*   bdesAddr_s, */
  DBADDR bactAddr_s;
  DBADDR bmonAddr_s;
  DBADDR idesAddr_s;
  DBADDR iactAddr_s;
  DBADDR riplAddr_s;
  DBADDR iprvAddr_s;
  DBADDR imonAddr_s;
  DBADDR imaxAddr_s;
  DBADDR ctimAddr_s;
  DBADDR ktimAddr_s;
  DBADDR ztimAddr_s;
  
  /* read from slc db for SCP function complete */
  /* each possible secn has both a "pointer" list and a data list */
  /*   written to slc db if different than hstaVal or desVals */
  /*  *******  All secn slc db pointer lists: *********   */
  DBLIST_DECLARE(dbhEntry_ts) hstaPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) bdesPlist_ps;

  DBLIST_DECLARE(dbhEntry_ts) statPlist_ps;

  DBLIST_DECLARE(dbhEntry_ts) ttimPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) itryPlist_ps;
  /* LGPS,BEND,BTRM,QTRM,QUAD,XCOR,YCOR,SEXT?,SMPS,SOLN have this secn superset: */
  /*   bdesPlist_ps, */
  DBLIST_DECLARE(dbhEntry_ts) bactPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) bmonPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) idesPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) iactPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) riplPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) iprvPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) imonPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) imaxPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) ctimPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) ktimPlist_ps;
  DBLIST_DECLARE(dbhEntry_ts) ztimPlist_ps;

  /* ***  All secn slc db data lists: ***************** */
  DBLIST_DECLARE(unsigned short) hstaDlist_ps;
  DBLIST_DECLARE(float) bdesDlist_ps;

  DBLIST_DECLARE(unsigned short) statDlist_ps;

  DBLIST_DECLARE(vmsTimeStamp_ta) ttimDlist_ps;
  DBLIST_DECLARE(unsigned short) itryDlist_ps;
  /* LGPS,BEND,BTRM,QTRM,QUAD,XCOR,YCOR,SEXT?,SMPS,SOLN have this secn superset: */
  /*   bdesDlist_ps, */
  DBLIST_DECLARE(float) bactDlist_ps;
  DBLIST_DECLARE(float) bmonDlist_ps;
  DBLIST_DECLARE(float) idesDlist_ps;
  DBLIST_DECLARE(float) iactDlist_ps;
  DBLIST_DECLARE(float) riplDlist_ps;
  DBLIST_DECLARE(float) iprvDlist_ps;
  DBLIST_DECLARE(float) imonDlist_ps;
  DBLIST_DECLARE(float) imaxDlist_ps;
  DBLIST_DECLARE(vmsTimeStamp_ta) ctimDlist_ps;
  DBLIST_DECLARE(vmsTimeStamp_ta) ktimDlist_ps;
  DBLIST_DECLARE(vmsTimeStamp_ta) ztimDlist_ps;

}mgntUnitStruct_ts;
/* Note: any magnet unit is capable of being any type of primary magnet; same structure for all */
 
/* Magnet Database information */
typedef struct {
  mgntPrim_te         prim_e;      /* primary emum & index into mgntPrim_s array */
  char                prim_ac[5];  /* primary name */
  unsigned short      slcUnitsI;   /* first unit index into the master unit list  */
                                   /*  use DBPTR(list,slcUnitsI) to access list */
  unsigned short      nunits;      /* number of units for this primary            */
  DBLIST_DECLARE (mgntUnitStruct_ts) units_ps;    
                                   /* ptr to dblist whose sole purpose is to contain */
                                   /*  heap memory for the units array using db free list */
} mgntPrimStruct_ts;

/* Magnet Diagnostic information */
typedef struct {
  epicsTimeStamp startTime_s;          /* time the mgntHdlr thread was started                                */

  /* message queue request time stamps */ 
  epicsBoolean   processingReq;          
  epicsTimeStamp lastReqTime_s;        /* time the last message request found in the mgntHdlr queue            */
  epicsTimeStamp lastReqStartTime_s;   /* time the last message request was processed in the mgntHdlr queue    */
  epicsTimeStamp lastReqFinishedTime_s;/* time the last message request was completed by the mgntHdlr queue    */
  epicsTimeStamp lastDbupdateTime_s;   /* time the last dbupdate was performed                                 */          

  /*
  ** Note that the last time a magnet check function was peformed. Remember
  ** that a magnet check is done before a magnet function request, and
  ** also periodically in the mgntAsync thread. We don't want to write over
  ** the timestamp that the last magnet request occurred...so we need to keep
  ** two timestamps.
  */
  epicsTimeStamp lastChckTime_s;       /* time the last time a mgntCheck() was done                           */

  double         maxDeltaRespTime;    /* maximum seconds for a request message tobe processed                  */
  double         totalDeltaRespTime;  /* total response time to process the last message                       */
  double         avgDeltaRespTime;    /* average response time to process a request message                    */
  double         lastDeltaRespTime;   /* last response time to process a request message                       */

  unsigned short lastFunc;            /* last function code processed byt the mgntHdlr                         */
  unsigned short curFunc;             /* the current function code being processed in the mgntHdlr             */

  double         reqCnt;              /* number of requests since the mgntHdlr started                         */
  double         reqStartCnt;         /* number of requests started since the mgntHdlr started                 */
  double         reqFinishedCnt;      /* number of requests completed since the mgntHdlr started               */
  double         reqDroppedCnt;       /* number of requests dropped sicne the mgntHdler started                */
  double         replyCnt;            /* number of replys since the mgntHdlr started                           */
  double         replyFailCnt;        /* number of reply failures since the mgntHdlr started                   */     
  double         reqAbortCnt;         /* number of aborted function requests                                   */ 
  double         reqTimeoutCnt;       /* number of function requests that have been aborted due to a timeout   */  

  epicsBoolean   init;                /* Initialization complete                                               */

} mgntHdlrDiag_ts;


typedef struct
{
  /* Supertype 1 */
  DBLIST_DECLARE(dbhEntry_ts) tols_ps; 
  DBLIST_DECLARE(dbhEntry_ts) tolf_ps;   
  /* Used for dblgetting only */
  /* Supertype 2  */ 
  DBLIST_DECLARE(dbhEntry_ts) hsta_ps; /* REMOVE - NOT USED */
  DBLIST_DECLARE(dbhEntry_ts) stat_ps; /* REMOVE - NOT USED */
  DBLIST_DECLARE(dbhEntry_ts) bdes_ps; /* REMOVE - NOT USED */
  /* MAGF 1 unit of */
  DBLIST_DECLARE(dbhEntry_ts) cstrMagf_ps;
} mgntDbPtrList_ts;

typedef struct
{
  /* Supertype 1  */  
  DBLIST_DECLARE(float) tols_ps; /* 4R4 */
  DBLIST_DECLARE(float) tolf_ps; /* 1R4 */
  /* Used for dblgetting only */
  DBLIST_DECLARE(unsigned long)  slcAllUnits_ps;
  /* Supertype 2  */
  DBLIST_DECLARE(unsigned short) hsta_ps;  /* REMOVE - NOT USED */
  DBLIST_DECLARE(unsigned short) stat_ps;  /* REMOVE - NOT USED */
  DBLIST_DECLARE(float)          bdes_ps;  /* REMOVE - NOT USED */
  /* MAGF */
  DBLIST_DECLARE(unsigned short) cstrMagf_ps;

} mgntDbDataList_ts;

typedef struct
{
  /* 
  ** current function being processed by the magnet handler
  ** If the function code is set to zero then there is 
  ** nothing being processed. 
  */
  unsigned short         activeFunc;
  epicsTimeStamp         reqStartTime_s;   /* time request started               */
  unsigned long          busyCounter;
  unsigned long          timeoutCounter;
  vmsstat_t              summaryStatus;
  epicsEventId           funcDoneEvent_ps; /* mutex to hold off Hdlr until func completes*/
  epicsMutexId           busyCounterMutex_ps;
} mgntFuncStruct_ts;

/* Magnet basic, top-level information */
typedef struct
{
  /* 
  ** current function being processed by the magnet handler
  */
  mgntFuncStruct_ts       func_s;
  unsigned short          nunits;              /* total # number of magnet units     */
  epicsTimeStamp          lastCheckTime_s;     /* time stamp of last mgntCheck  */
  epicsTimeStamp          lastCheckFeedbackTime_s; /* time stamp of last mgntCheckFeedback */
  epicsTimeStamp          lastTime_s;          /* time stamp of last mgntCheckTime */
  epicsBoolean            doUpdate;            /* flag set for any dblputs as result */
                                               /*  of CTRL=IDLE; updates metered     */
  unsigned short          updateTimeout;       /* 1 sec async dbupdate meters */
  /* database lists, pointers and data common to all prims that are setpoints   */
  mgntDbPtrList_ts        plist_s;             /* db ptr lists            */
  mgntDbDataList_ts       dlist_s;             /* db data lists           */

  /* Diagnostic information kept by the magnet handler thread */
  mgntHdlrDiag_ts         diag_s;
}mgntHdlrInfo_ts;

/*
** Magnet incoming mailbox message request data created
** on the Alpha by the functions, magfunc() and
** maglist(). This message is comprised of a standard
** header and a magnet specific body. The body contains
** of either a single or a list of tripet unsigned long.
** each triplet is of the form, primary, first unit and 
** last unit. The wildcard "ALL*" can be used for each.
** If the end unit "0" then a single unit defined by in
** the first unit field.
*/
typedef struct 
{
  unsigned long prim;
  unsigned long firstUnit;
  unsigned long lastUnit;
}mgntHdlrReqData_ts;

#endif /* mgntTypes_H */
