 /*===========================================================================*/
 /*      **CMS**=RMX_INCLUDE                                                  */
 /*                                                                           */
 /* Abs:  Structure definitions for several public structures in the micro's  */
 /*       database which support lowest-level software.                       */
 /*                                                                           */
 /* Name: slcrmxi:db_micro_struc.hm.                                          */
 /*                                                                           */
 /* Prev: <rmxc.h> or <i86.h>            selector.                            */
 /*       slctxt:slcparm.hc   (optional) N_JOBS.                              */
 /*       slctxt:netparm.hc              NETBUFLEN.                           */
 /*       slctxt:msghead.hc              fwd_hdr_ts.                          */
 /*                                                                           */
 /* Rem:  See also SLCTXT:DB_SUPSTR.TXT, SLCRMXI:SUPSTR.INC, DBNODES.INC,     */
 /*       MICMAIL.INC, and DBSENDCOM.INC (from which this include file was    */
 /*       translated). See also corresponding definitions in SLCTXT:CACHE.H.  */
 /*       You probably should be aware that in NETPARM.HC (C code) NETBUFLEN  */
 /*       is defined as 4096 (words) but in NETPARM.INC (Intel Fortran code)  */
 /*       NETBUFLEN is defined as 8192 (bytes).                               */
 /*                                                                           */
 /* Auth:  Feb-1983, Nan Phinney (NP)                                       */
 /* Revw:                                                                     */
 /*                                                                           */
 /*---------------------------------------------------------------------------*/
 /*                                                                           */
 /* Mod:  14-Jan-2004, Tony Gromme (TEG):                                     */
 /*           Add dbsndrcvdiagmsg_ts.                                         */
 /*       18-Apr-2000, Tony Gromme (TEG):                                     */
 /*           Translate into iC386 from (mostly) FORT86.  Let each subjob's   */
 /*           accumulation of pieces to be sent be sent separately if widely  */
 /*           enough separated.                                               */
 /*       23-Oct-1996, Robert C. Sass (RCS):                                  */
 /*           Add TCP forward header in common.                               */
 /*       11-Jun-1994, Ken Underwood (KKU) and Tony Gromme (TEG):             */
 /*           In SUPSTR.INC the following parameters are all 1-biased         */
 /*           subscripts into word arrays, but some of the items they point   */
 /*           to have been changed from words to longwords, namely:  in the   */
 /*           superblock header:  SP_LEN, SP_BLK_BEG, SP_SEG_BEG, the array   */
 /*           of hash chain roots;  in the UPS block:  SP_NXTH, SP_NXTP,      */
 /*           SP_NXTC, SP_NLEN;  and in the subblk:  SP_SPTR.                 */
 /*       20-Dec-1990, Tony Gromme (TEG):                                     */
 /*           Convert for 32-bit code.                                        */
 /*                                                                           */
 /*===========================================================================*/
 
#ifndef DB_MICRO_STRUC_HM
#include "msgTypes.h"       /* NETBUFLEN */
#include    <epicsEvent.h>  /* epicsEventId */
#include    <epicsMutex.h>  /* epicsMutexId */
#define MAXDBSNDPJOB  30    /* Max # of "dbsend" items that can accumulate  */
                            /* for any one subjob between dbsend's done on  */
                            /* behalf of that subjob.                       */
#define MINDBSNDWSEP  50    /* Minimum separation in words between "dbsend" */
                            /* items to have them sent separately.          */
 
 /* Corresponding definitions in slctxt:cache.h must be */
 /* identical to following:                             */
 
#define SUPERTYPE_ID_WMASK    0x000F /* Mask for extracting supertype number    */
                                     /*  (or "new" func code) from member "id"  */
                                     /*  in dbsuptypehdr_ts.                    */
#define SUPERTYPE_ID_IPLDWN   0x0040 /* If this bit is set then this message is */
                                     /*  part of its micro's database download  */
                                     /*  at bootup time.                        */
#define SUPERTYPE_ID_ACKREQ   0x0020 /* This bit requests an acknowledgement of */
                                     /*  the message begun by this header.      */
#define SUPERTYPE_ID_OVERRIDE 0x0010 /* If this bit is not set then the number  */
                                     /*  extracted with SUPERTYPE_ID_WMASK is a */
                                     /*  supertype number;  else said number is */
                                     /*  not a supertype number but one of the  */
                                     /*  following function codes:              */
#define SUPERHDR_FUNC_BTDN   1       /* Micro (at bootup time) is requesting    */
                                     /*  that its own database be downloaded,   */
                                     /*  or SCP is requesting same for micro    */
                                     /*  being booted up.                       */
#define SUPERHDR_FUNC_TIMB   2       /* Micro is requesting a time-of-day       */
                                     /*  update to be done (via SLCnet only).   */
#define SUPERHDR_FUNC_VPOL   4       /* Micro is requesting to be put on the    */
                                     /*  SLCnet VSC's poll list.                */
#define SUPERHDR_FUNC_DGINQ  6       /* Diagnostic inquiry request.             */
#define SUPERHDR_FUNC_DGRPY  7       /* Diagnostic inquiry reply.               */
#define SUPERHDR_FUNC_NOTFY  8       /* "DXUP" msg conveys db version #;  "DXDN" */
                                     /*  msg says DBEX is knowingly exiting.    */
#define SUPERHDR_FUNC_ACK   12       /* This message is an acknowledgement      */
                                     /*  requested by a previous message.       */
#define DBCURVERSIONBC  8            /* Bytecount of database current version */
                                     /*  number in ascii (by dbgen).          */
 
#define MAXDIAGWLEN 20               /* # of words of dbsndrcvdiagdata_ts       */

#define DBHASH   337                    /* Hashing value */

                                   /* Structure of beginning of each supertype */
                                   /*  block, as well as of beginning of any   */
 typedef struct                    /*  piece sent in either direction.         */
 {   unsigned short network,
                    netlsn,
                    id;
     unsigned long  len,
                    micr,
                    blkbeg,
                    segbeg;
 } dbsuptypehdr_ts;
 /* A superblock header must be at (near) beginning of any message in  either direction 
    between DBEX and DBMAIN.  A superblock header lives at  beginning of each superblock 
    of each micro in Alpha's memory, and hence at  beginning of each superblock in micro's 
    memory.  So the micro has to know to  actually copy the superblock header into micro's 
    memory from the first  !download! message for each superblock, but not at other times.  
    Element blkbeg  accomplishes that.  In the superblock header (structure dbsuptypehdr_ts 
    defined  in REF_RMX_INCLUDE:DB_MICRO_STRUC.HM, matching Mike Huffer's struct sbhdr  
    defined in REF_C_INC:CACHE.H): */
    /*        len    = size in words of entire superblock of which this   */
    /*                  message is a portion (size here includes          */
    /*                  superblock header).                               */
    /*        segbeg = 1-biased subscript into superblock (considered in  */
    /*                  its entirety as a word array including superblock */
    /*                  header) telling where this data is to go (copy    */
    /*                  operation may or may not overwrite superblock     */
    /*                  header).                                          */
    /*        blkbeg = 1-biased subscript into this message (considered   */
    /*                  as a word array including superblock header)      */
    /*                  telling where copy is to begin from (copy         */
    /*                  operation may or may not include superblock       */
    /*                  header).                                          */

 typedef union
 {   dbsuptypehdr_ts sbhdr;
     unsigned short  datw[NETBUFLEN];
 } dbsuptype_tu;
 
                                   /* Public structure containing pointers to */
                                   /*  memory allocated for supertype blocks  */
 typedef struct                    /*  belonging to this micro.               */
 {   union
     {   dbsuptype_tu  *supertype_ptr;
         void *sb0;               /* also save ST0 ptr here -add this for testing */
         unsigned short w4[4]; 
     } u_a[5];
 } dbnodes_ts;
 
                                   /* Array of "dbsend" items pending for */
 typedef struct                    /*  any one subjob.                    */
 {   unsigned long  msgc,
                    datwc,
                    errc,
                    difmin,
                    wxn,
                    wxa[MAXDBSNDPJOB][2];
 } dbhilo_job_ts;
 
                                   /* Structure of buffer for sending or */
 typedef struct                    /*  receiving a piece of database.    */
 {   fwd_hdr_ts      pxyhdr;
     dbsuptype_tu    supblk;
     dbsuptypehdr_ts nativeSB;     /* unpack dbsuptypehdr_ts portion of dbsuptype_tu */  
                                   /*  here so that it won't overwrite 1st word of data */
                                   /*  when native architecture RISC pads! */
 } dbsndrcvbuf_ts;
 
                                   /* Structure of buffer for sending or      */
 typedef struct                    /*  receiving ack or sending "download me" */
 {   fwd_hdr_ts      pxyhdr;       /*  request (no data included).            */
     dbsuptypehdr_ts suphdr;
 } dbrcvackbuf_ts;
 
                                   /* Structure of buffer for sending/receiving */
typedef struct
{ dbsuptypehdr_ts    suphdr;
  unsigned long      dbex_msgc_snd,  
                     dbex_msgc_rcv, 
                     micr_msgc_snd,
                     micr_msgc_rcv;
} dbsndrcvdiagdata_ts;

typedef union
{
  dbsndrcvdiagdata_ts data;
  unsigned short      datw[MAXDIAGWLEN];
} dbsndrcvdiagdata_tu;


typedef struct                    /*  diag inquiry request/reply.              */
{ 
  fwd_hdr_ts          pxyhdr;
  dbsndrcvdiagdata_tu diagdata;
} dbsndrcvdiagmsg_ts;
 
/* #if defined(N_JOBS)*/                /* Structure of "common" containing        */
                                   /*  micro_dbsend's output buffer, the      */
                                   /*  per-subjob lists of pieces to be sent, */
 typedef struct                    /*  and dbmicromail's input buffer.        */
 { 
     unsigned short  spare;
     unsigned        msgc_snd,
                     msgc_rcv,                         /* DEPRECATED */
                     msgc_wdog;
   /*dbhilo_job_ts   jbhiloST2[N_JOBS]; */
   /*dbhilo_job_ts   jbhiloST3[N_JOBS]; */
     dbsndrcvbuf_ts  rcvbuf,
                     sendbuf;
 } dbmaincom_ts;
/* #endif*/

 typedef struct
 {
   unsigned long  msgc_snd,
                  msgc_rcv,
                  msgc_wdog;
 } dbdiag_ts;

                                   /* Structure of buffer for receiving "DXUP" */
 typedef struct                    /*  & "DXDN" notifications from DBEX.       */
 {   fwd_hdr_ts      pxyhdr;
     dbsuptypehdr_ts suphdr;
     char            notify4[4],
                     dbversion_a[DBCURVERSIONBC];
 } dbexmnotifymsg_ts;
 

/* structure for encapsulating the dbupdateHiLo data/mutex/event/status */
typedef struct {
  dbhilo_job_ts     dbupdateSTHiLo[2];           /* 0 contains ST2 updates, 1 ST3 updates*/
  epicsMutexId      dbupdateHiLoMutex_ps;        /*  RW mutex for dbupdateSTxHILo_ps */  
  epicsEventId      dbupdateHiLoAckEvent_ps;     /* Event for dbupdate();signals DEBX ack completion */
  vmsstat_t         dbudpateHiLoAckStatus_s;     /* status of DBEX acks on CTL socket */
} dbupdate_ts;
 
#define DB_MICRO_STRUC_HM
#endif
 
