/*    
==================================================================
 
  Abs:  Find index into arr_a of matched key 
 
  Name: find_index_c.c
            Find_index_c() - Entry point 

  Args: arr_a                          Array of structures tobe 
          Type: array of struct        searched
          Use:  void *
          Acc:  read-only
          Mech: By reference

       elsiz                           Byte count of each element
          Type: integer                of given array
          Use:  longword (unsigned) 
          Acc:  read-only
          Mech: By value


       nelts                           Number of elements in the
          Type: integer                given array
          Use:  longword (unsigned) 
          Acc:  read-only
          Mech: By value

       key_p    57	57	                       Search key 
          Type: ???              
          Use:  void *
          Acc:  read-only
          Mech: By reference

       keysiz                          Byte count of search key (optional)
          Type: integer                Note:  If not supplied the key size
          Use:  longword (unsigned)    is assumed tobe the same as a
          Acc:  read-only              single element in the given array.
          Mech: By value          


       keyoff                          Offset of bytes of the start 
          Type: integer                of key within each element of
          Use:  longword (unsigned)    the given array. (optional)
          Acc:  read-only              Note: If not supplied an offset
          Mech: By value               of zero is used.

  Rem:  This routines earches the the array of structures, arr_a for
        a possible match with the key_p.
 
  Side: None        
 
  Ret: long 
            MATCH_NOT_FOUND (-1) - No match found 
            Otherwise            - Index into array of match found 

  Proto: util_proto.h
 
  Auth: 05-Jun-1985, Tony Gromme (TEG)
  Revw:
 
--------------------------------------------------------------
 
  Mod: 
        17-Feb-1998, K. Luchini (LUCHINI):
          Modify for vxWorks
        08-Jun-1996, Tony Gromme (TEG):
          Translated to C from Vax Macro.
 
==============================================================
*/
#include "debugPrint.h"
#ifdef  DEBUG_PRINT
  int findIndexFlag = DP_ERROR;
#endif
#include "string.h"                   /* memcmp() prototype */  

#define NO_MATCH_FOUND -1

long Find_index_c( const    void  *arr_a,
                   unsigned long   elsiz, 
                   unsigned long   nelts,
                   const    void  *key_p,
                   unsigned long   keysiz,
                   unsigned long   keyoff )
{  
     unsigned long  key_siz;
     unsigned long  key_off = 0;
     unsigned long  chx;
     long           j;


     /* If the key size is zero then use the element size */
     if ( keysiz==0 ) 
        key_siz = elsiz;
     else 
        key_siz = keysiz;

	 if (elsiz==2){
	   DEBUGPRINT(DP_INFO, findIndexFlag,
               ("Find_index_c: Try to find key= 0x%.4x \n", *((unsigned short *)(key_p)) ));
     }else if (elsiz==4){
	   DEBUGPRINT(DP_INFO, findIndexFlag,
               ("Find_index_c: Try to find key= 0x%.8lx in #elements= %lu, size=%lu of list %p \n", 
				(*((unsigned long *)(key_p))), nelts, elsiz, arr_a ));
	 }

     /* We have the arguments;  now do the work. */
     for ( chx = key_off, j = 0;  
           j < nelts && memcmp(&((char *) arr_a)[chx],(char *)key_p,key_siz);
           chx += elsiz, j++) {
         ;
     } /* End of FOR loop */
  
     if (j >= nelts)
       j = NO_MATCH_FOUND;
     return j;
} 
