/*=============================================================================

  Name: msgTypes.h

  Abs:  defines and typedefs for message structures used by slcIOC message queue threads
        MANY OF THESE STRUCTURES ARE SHARED WITH THE VMS CONTROL SYSTEM AND SLC MICROS

  Auth: 03-DEC-2004, Diane Fairley (dfairley):  created

  Rev:  02 feb 2005, Debbie Rogind

-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/
#ifndef msgTypes_H
#define msgTypes_H

/* 
 *  THE FOLLOWING CONSTANTS ARE TAKEN FROM NETPARM.HC
 *  ALL SHARED WITH THE VMS CONTROL SYSTEM AND THE SLC MICROS
 */
/*                                                                            */
#define NETBUFLEN 4096
/*      Standard network buffer length (in words) for database transfers      */
/*                                                                            */
#define NETHEADERLEN 10
/*       Number of words of header for Message Service.                       */
/*                                                                            */
#define NETVAXMSGLEN 1024
/*       Max size in words of message service msg, Vax-->micro                */
/*                                               or Vax-->Vax or micro-->micro*/
/*                                                                            */
#define NETVAXDATALEN (NETVAXMSGLEN-NETHEADERLEN)
/*       Max size in words of data area , Vax-->micro                         */
/*                                     or Vax-->Vax or micro-->micro          */
/*                                                                            */
#define NETMICMSGLEN 4096
/*       Max size in words of message service msg, micro-->Vax                */
/*                                                                            */
#define NETMICDATALEN (NETMICMSGLEN-NETHEADERLEN)
/*       Max size in words of data area , micro-->Vax                         */
/*                                                                            */
#define NETMICLONGMSGLEN 65535
/*       Max size in words of long message service msg, micro-->Vax           */
/*       64Kw                                                                 */
/*                                                                            */
#define TCPFWDHDRBYTELEN 12
/*       Length of fwd_hdr_ts in slctxt:msghead.hc for TCP msg service        */

#define DONE_MASK 128
/*       bit mask to indicate end of a multipacket message to the SLC controller*/

/* 
 *  THE FOLLOWING TYPEDEFS ARE TAKEN FROM MSGHEAD.HC
 *  ALL SHARED WITH THE VMS CONTROL SYSTEM AND THE SLC MICROS
 */
/* 
** The world standard msg header common to VAX and micro.
*/

typedef struct
{
   char          source[4];
   char          dest[4];
   int4u         timestamp[2];
   int2u         func;
   int2u         datalen;
} msgheader_ts;

 /* Forward header for the IP/TCP proxy.  See also msg_ip_types.h. */
 
  typedef struct
  {
    int2u  ip;   /* half of the IP address */
                         /* destination address when sent to FWD server */
                         /* source address when sent out of FWD server */
    int2u  port; /* TCP port number */
                        /* used as destination port when sent to FWD server */
                        /* used as source port when sent out of FWD server */
  } ip_port_ts;
 
  typedef union
  {
    ip_port_ts   s;       /* structure access */
    int4u        word;    /* longword access  */
  } ip_port_tu;
 
  typedef struct
  {
    int1u           byte0;     /* Low byte of forward header user field. */
    int1u           byte1;     /* Hi byte of forward header user field.  */
  } user_field_ts;
 
  typedef struct
  {
    ip_port_tu      ip_port_u; /* Half of the ip address, and the port number.  */
    int4u           len;       /* Stream bytecount minus this header.           */
    user_field_ts   user;      /* User defined.  Chunk count for large buffs.   */
    int1u           cmd;       /* fwd_server command, e.g., ping.               */
    int1u           crc;       /* 8 bit crc over header, delimit TCP stream.    */
  } fwd_hdr_ts;           /* Note that len is always in network byte order.     */
                          /* In the Alpha-to-micro direction each 16-bit word   */
                          /* in the 32-bit ip_port_ts is in network byte order. */
                          /* In the micro-to-Alpha direction the 32-bit         */
                          /* ip_port_u is left in host (little-endian) byte     */
                          /* order.  Note that fwd_hdr_ts must be an integral # */
                          /* of longwords in length.                            */

/* 
 *  THE FOLLOWING STRUCTURE IS TAKEN FROM MSGMAIL.HM
 *  ALL SHARED WITH THE VMS CONTROL SYSTEM AND THE SLC MICROS
 */

/* the standard message structure of all  incoming messages within the message service */
/* also the standard message structure of small outgoing messages within the message service */
typedef struct
{  
  unsigned long  job_id;    /* holds the slcIOC thread enum this message is for NOT THE JOB ENUM */
  fwd_hdr_ts     tcpfwdhdr; /* the tcp forward header used by the proxy */
  msgheader_ts   msgheader; /* the original slc message header */
  unsigned short reqdata[NETVAXMSGLEN-(sizeof(msgheader_ts)/2)]; /* the function code specific data */
} msgmail_ts;


/* the standard message structure of all large outgoing messages within the message service */
typedef struct
{  
  unsigned long  job_id;
  fwd_hdr_ts     tcpfwdhdr;
  msgheader_ts   msgheader;
  unsigned short reqdata[NETMICLONGMSGLEN-(sizeof(msgheader_ts)/2)];
} lrgmail_ts;

/* the standard message structure of the slcIOC Message Service message queue messages */
typedef struct
{ 
  unsigned short msgbytes;  /* number of bytes in the message */
  void *         msgptr_p; /* pointer to the message pool space holding the message */
} msgptr_ts;


#define MAX_SMALL_MSG_BYTES sizeof(msgmail_ts)
#define MAX_LARGE_MSG_BYTES sizeof(lrgmail_ts)
#define SMALL_MSGPOOL_ELEMENTS 50 /* start with 50 2kbyte small buffers */
#define LARGE_MSGPOOL_ELEMENTS 10 /* start with 10 128Kb large buffers */
#define MSG_HDRS_BYTES (sizeof(msgheader_ts)+sizeof(fwd_hdr_ts)+sizeof (unsigned long))
/* used when creating a  message queue */ 
#define MAX_QUEUED_MSGS   1000                  /* max number of messages queued ESTIMATE - TBD */
#define MAX_MSG_BYTES     sizeof(msgmail_ts)  /* max bytes in small or incoming TCP msgs */
#define MAX_MSGPTR_BYTES   sizeof(msgptr_ts)  /* max bytes in msgptr type message Q  */

#endif /* guard */
