/*============================================================================

  Name: slcAsync.h

  Abs:  Prototypes and data structures for interfacting with public Async utilities 
        in the slcIOC.

  Auth: 15-March-2005, Ron MacKenzie (ronm)

  Rev:  20-March-2005, Stephanie Allison

  Mods:
    01-Apr-2005 Ron MacKenzie (ronm)
      Add slcasyncDumpGlob for use by ioc console.

-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  (newest to oldest)

  21-Nov-2007 Ron MacKenzie (ronm)
    Add procChk3() proto  
  01-Apr-2005 Ron MacKenzie (ronm)
    Add slcasyncDumpGlob for use by ioc console.

=============================================================================*/
#ifndef slcAsync_H  /* guard macro */
#define slcAsync_H  /* guard macro */

#include <slcStructs.h>         /* vmsTimeStamp_ta, vmsstat_t,slcThread_te */
#include <epicsTime.h>                                   /* epicsTimeStamp */

#define SLC_MAX_N_ASYN 20               /* max num async functions per ioc */


/* When calling slcAsyncSetFcnStats use slcReqType_te to tell it if you're calling
 * as a result of a vms message or async cycling function request 
 */

typedef enum {
  slcAsync,
  slcMsg,
  none                /* neither of the above */
} slcReqType_te;

/*
 * This structure is sent in the reqdata portion of msgmail_ts when transmitting
 * messages from cstrAsync to Hdlr's.  It contains the name and index of
 * the cycling function.
 */

typedef struct {
  int  cyclIx;           /* index of cycling function */
  char cyclName[8];      /* name of cycling function  */
} asyncmail_ts;

/*
 * Scanning parameters
 */
typedef struct {
  unsigned long  mmsk;
  unsigned long  cmsk;
  unsigned long  fmsk;
  unsigned short hsta;
  short          scan_a[SLC_MAX_N_ASYN];
} slcScan_ts;

/* protoytpes */

int          slcAsyncInit(void);
void         slcAsyncExit(void);
void         slcAsyncDumpGlob(void);
int          slcAsyncLockStats(void);
int          slcAsyncUnlockStats(void);

void         slcAsyncSetAsyncExists(const epicsBoolean newVal);
void         slcAsyncSetCYCL(const int jobFuncIx, short newVal);
void         slcAsyncSetMTRL(const int jobFuncIx, short newVal);
void         slcAsyncSetMTRC(const int jobFuncIx, short newVal);
void         slcAsyncSetMAXT(const int jobFuncIx, short newVal);

int          slcAsyncGetFuncIxByName(const char * const name_p);
slcThread_te slcAsyncGetThreadEnum(const int cyclIx2Exe);
epicsBoolean slcAsyncGetAsyncExists(void);
epicsBoolean slcAsyncGetJMSK (unsigned long * const jmsk_p);
epicsBoolean slcAsyncGetCDSC (unsigned short * const cdsc_p);
epicsBoolean slcAsyncReady(void);

int slcAsyncGetCycleFcn(const int firstIx , const int numFunc,
                        const slcScan_ts * const scan_ps,
                        int * const cyclIx2Exe, 
			char* const cyclName2Exe_c,
                        epicsTimeStamp * const currentTime_ps);

int slcAsyncMeterDbSend(const int asyncFuncIx, const slcReqType_te requestType,
			const epicsBoolean dblputDone);

int slcAsyncSetFcnStats(const int asyncFuncIx, 
			const epicsTimeStamp * const startTime_ps, 
                        const slcReqType_te requestType);

int slcAsyncSetSendStats(const int asyncFuncIx, const vmsstat_t status,
                         const epicsTimeStamp * currentTime_ps);

int slcAsyncProcChk1(vmsTimeStamp_ta * const dataCTIM_aa,
                     vmsTimeStamp_ta * const dataUTIM_aa,
                     float           * const dataELPS_a);

int slcAsyncProcChk2(long  * const dataNRUN_a,
                     long  * const dataFAIL_a,
                     float * const dataPVAX_a,
                     float * const dataPUPD_a,
                     epicsBoolean * const NRUN_changed_p,
                     epicsBoolean * const FAIL_changed_p,
                     epicsBoolean * const PVAX_changed_p,
                     epicsBoolean * const PUPD_changed_p);

int slcAsyncProcChk3(void);


#endif /* slcAsync_H */
