/*============================================================================

  Name: slcStructs.h

  Abs:  This include file contains global structure defines used by
        most threads in the slcIOC.


  Auth: 22-dec-2004, Diane Fairley (DFAIRLEY): created from prototype

  Rev:  02 feb 2005, Debbie Rogind
-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  (newest to oldest)
        26-Sep-2005, K. Luchini (LUCHINI):
           Added  slcScpFunc_ts
        07-Apr-2005 Ron MacKenzie (ronm)
           Add 2 items to job table for cstr usage.         
           Remove async_e from job table because we don't have async jobs.
        02-jan-2004 dfairley 
           moved structure defines to slcStructs.h from slcTypes.h

=============================================================================*/
#ifndef slcStructs_H  /* guard macro */
#define slcStructs_H  /* guard macro */

#include <epicsThread.h>        /* epicsThreadId */
#include <epicsTime.h>          /* epicsTimeStamp */
#include <epicsTypes.h>         /* epicsBoolean */
#include <epicsMessageQueue.h>  /* epicsMessageQId */

#include "slcTypes.h"           /* int4u etc */
#include "db_micro_struc.h"     /* dbupdate_ts */

/* global struct of thread info */
typedef struct {        
  epicsThreadId       tid_ps;
  epicsBoolean        active;
  epicsBoolean        stop;
  epicsMessageQueueId mqId_ps;
  unsigned long       destination;    /* this is a 4-char destination name */
} slcThread_ts;                       /* stored in a ulong int for convenience */


/* global struct of slc micro Job to slcIOC thread mapping */
typedef struct {           
  char          jobName_a[5];        /* null terminated, max. 4-character job name */
  slcThread_te  hdlr_e;              /* associated hdlr enumeration */
  slcThread_te  async_e;             /* associated async enumeration */
  int           frstCyclFuncIx;      /* index into slcJobFunc_as for async jobs */
  int           numCyclFunc;         /* number of cycling functions for async jobs */
  dbupdate_ts * dbupdate_ps;         /* job db update structures  */
} slcJob_ts;

/* vmsTimeStamp type */
typedef unsigned long  vmsTimeStamp_ta[2];

/* exec thread diagnostics structure */
typedef struct {
  epicsTimeStamp lastStartTime;
  double         numStarts;
  double         numStops;
} execDiags_ts;
  
typedef struct {
     char           ascii_func[10];
     unsigned short func;
     unsigned long  timeout; 
     vmsstat_t      (*routine_pf)(int                 * const  nbytes_p,
                                   const void          * const  rMsg_p,
                                   void                * const  sMsg_p,
                                   const char          * const  cmd_c,
                                   const unsigned long * const  timeout_p,
                                   unsigned long       * const  update_p); 
     void            (*callback_pf)(void *evtHdlrArgs_p);    /* from cadef.h */ 
}slcScpFunc_ts;

#endif /* slcStructs_H */
