/*============================================================================

  Name: slcTypes.h

  Abs:  This include file contains global defines, macros, structures used by
        most threads in the slcIOC.


  Auth: 22-dec-2004, Diane Fairley (DFAIRLEY): created from prototype

  Rev:  02 feb 2005, Debbie Rogind
-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  
        04-Oct-2005, K. Luchini (LUCHINI):
                added defines for MGNT_ASYN_NAME and MGNT_HDLR_NAME
                added mgntAsync to slcThread_te
        11-Jul-2005, K. Luchini (LUCHINI):
                Add mgntHdlr to slcThreads_te
        08-apr-2005 dfairley, renumbered slcJobs_te
        24-Jan-2005, Ron MacKenzie (ronm)
                Added cstrAsync thread.
        18-Aug-2004, drogind, added sdCtl to sdSlcSocketEnum
        02-jan-2004 dfairley, moved structure defines to slcStruct.h
		22-Nov-2004 dfairley, renamed to slcGlobTypes.h no longer a prototype file
		moved tcp related stuff to tcpTypes.h
		24-Aug-2004, drogind, changed MAX_MSG_BYTES to sizeof(slcmsgmail_ts)

=============================================================================*/
#ifndef slcTypes_H  /* guard macro */
#define slcTypes_H  /* guard macro */

#include <epicsThread.h>        /* epicsThreadId */
#include <epicsTypes.h>         /* epicsBoolean */
#include <epicsMessageQueue.h>  /* epicsMessageQId */

/* define local functions as static  - TAKEN from epics dbdefs.h*/
#ifndef LOCAL
#define LOCAL static
#endif

/* define NULL to be 0  - taken from epics dbdefs.h */
#if defined(NULL)
#undef NULL
#endif
#define NULL    0

/* 
**  Macro to check status return from standard error codes.
**  this is used for checking all status return codes
**  of the type vmsstat_t. Particularly useful when trying
**  to report errors in the format and timing that the VMS 
**  control system expects.
*/
#define     SUCCESS( status)    ((status)&0x00000001)

/* 
   Typedef's for various flavors of integers. Done in anticipation of the
   switch to RMX3 when the definition of int will change. We try to
   make a typedef category for each class of parameter, guessing at
   this time which will stay as 2 byte integers and which can float to
   4 bytes at RMX III time. These are also used by VAX code for message
   definition etc.
   NOTE: THESE ARE CURRENTLY USED IN msgTypes.h FOR MESSAGE STRUCTURES
         SHARED WITH THE VMS CONTROL SYSTEM AND THE SLC MICROS

*/
typedef short             int2;           /* An I*2 */
typedef unsigned short    int2u;          /* An unsigned I*2 */
typedef long int          int4;           /* An I*4 */
typedef unsigned long int int4u;          /* An unsigned I*4 */
typedef unsigned char     int1u;          /* For unsigned byte arithmetic */
typedef unsigned char     charu;          /* For unsigned byte arithmetic */

/* 
   Give VMS style status its own type. THIS IS THE ERROR CODE TYPE RETURNED BY
   MANY FUNCTION CALLS.  USED FOR ANY ERROR CODE SHARED WITH THE VMS CONTROL
   SYSTEM AND THE SLC MICROS
*/
typedef int4u           vmsstat_t;      /* VMS type status stays 32 bits */


/* thread, message queue and socket defines */
#define NO_THREAD        0                  /* indicates no thread exists with this tid */
#define MAX_THREADS     33                  /*  max number of threads to create ESTIMATE NOW */
#define SLCEXEC          0                  /* enumeration for slcExec thread */
#define V017            "V017"              /* default destination */
#define V017_UL         1445998903          /* nice define for assigning default dest */
                                            /* this is the ulong value of "V017" */ 

/* maximum length of thread names */
#define MAX_THREADNAME_SIZE  17 /* 16 chars + null terminator */

/*  this enum is used to index into slcThreads */
typedef enum {                       /* enumeration of slc threads */
  slcExec, msgRecv, dbRecv, msgSend, /* do not change this order or add here */
  msgHdlr, cstrAsync, cstrHdlr,      /* add new threads begining here */
  bsacHdlr,                         /* beam synch acq. thread */
  mgntHdlr, mgntAsync,              /* magnet handler and async thread */
  statHdlr,                         /* for FECC, start here */
  anlgHdlr, camvHdlr, cratHdlr, 
  timeHdlr, klysHdlr, mpgHdlr,
  fbckHdlr, bcomHdlr, microHdlr,
  dbSend, dbHdlr,                   /* dont add here          */ 
  slcEnd                            /*do not add on this line */
} slcThread_te;

#define slcInvalid slcEnd

/* define mappings from SLC jobs to saIoc threads */
#define MAX_SLC_JOBS      32          /* the max number of SLC JOB IDs */

/* note - the following slcJobs_te enum is a copy of the FUNC_ definitions 
   in msgfunc.hc. It is shared with the slc Control System, and the rmxMicros. 
   Changes here must be reflected in msgfunc.hc.  
   The iocsJob = 0 is for internal slcIOC messages.
   NOTE: added dummy values to make the enumeration sequential
*/

typedef enum {
  iocsJob = 0, /* new for our own iocsh */
  bpmoJob = 1,  /* (FUNC_BPMO>>8) */
  mgntJob = 2,  /* (FUNC_MGNT>>8) */
  klysJob = 3, 
  statJob = 4, 
  fbckJob = 5, 
  timeJob = 6,  /* (FUNC_TIMING>>8) */
  camvJob = 7, 
  mpgJob  = 8, 
  cratJob = 9, 
  fdbkJob = 10, 
  pmJob   = 11, 
  dummyJob12 = 12,
  anlgJob    = 13, 
  micrreqJob = 14, 
  verexJob   = 15, 
  imagenJob  = 16, 
  mpsJob     = 17, 
  mpscJob    = 18, 
  dbsJob     = 19,   /* (FUNC_DB>>8)*/
  microJob   = 20, 
  bcomJob    = 21, 
  diagJob    = 22, 
  dummyJob23 = 23, 
  dummyJob24 = 24, 
  cstrJob    = 25, /* (FUNC_TEST>>8)*/
  mcomJob    = 26, 
  msgfwdJob  = 27, 
  msgJob     = 28, 
  dummyJob29 = 29, 
  netJob     = 30,  /* (FUNC_MSG>>8) */
  md86Job    = 31, 
  invalidJob = 32 /* return this for error conditions */
} slcJob_te;

/* the following are the 4 character string names for each job */
#define MSG_JOB_NAME   "MSG"
#define CSTR_JOB_NAME  "TEST"
#define DBS_JOB_NAME   "DBS"
#define TIME_JOB_NAME  "TIME"
#define BPMO_JOB_NAME  "BPMO"
#define MGNT_JOB_NAME  "MGNT"
#define IOCS_JOB_NAME  "IOCS"
#define STAT_JOB_NAME  "STAT"
#define ANLG_JOB_NAME  "ANLG"
#define CAMV_JOB_NAME  "CAMV"
#define CRAT_JOB_NAME  "CRAT"
#define KLYS_JOB_NAME  "KLYS"
#define MPG_JOB_NAME   "MPG"
#define FBCK_JOB_NAME  "FBCK"
#define BCOM_JOB_NAME  "BCOM"
#define MICRO_JOB_NAME  "MICR"


/* the following are name strings for each slc-aware thread */
/* the first 4 characters are significant for some epics related things?*/
#define EXEC_NAME      "ExecThrd"
#define MSG_SEND_NAME  "mSndThrd"
#define MSG_RECV_NAME  "mRcvThrd"
#define MSG_HDLR_NAME  "mHdrThrd"
#define DB_SEND_NAME   "dSndThrd"
#define DB_RECV_NAME   "dRcvThrd"
#define DB_HDLR_NAME   "dHdrThrd"
#define CSTR_ASYN_NAME "cAsyThrd"
#define CSTR_HDLR_NAME "cHdrThrd"
#define BSAC_HDLR_NAME "bHdrThrd"
#define BSAC_ASYN_NAME "bAsyThrd"
#define STAT_HDLR_NAME "statmain"
#define MGNT_HDLR_NAME "magHThrd"
#define MGNT_ASYN_NAME "magAThrd"  
#define ANLG_HDLR_NAME "anlgThrd"
#define CAMV_HDLR_NAME "camvThrd"
#define CRAT_HDLR_NAME "cratThrd"
#define TIME_HDLR_NAME "timeThrd"
#define KLYS_HDLR_NAME "klysThrd"
#define MPG_HDLR_NAME  "mpgThrd"
#define FBCK_HDLR_NAME "fbckThrd"
#define BCOM_HDLR_NAME "bcomThrd"
#define MICRO_HDLR_NAME "micrThrd"


#endif /* slcTypes_H */
