/*=============================================================================

  Name: util/tcpTypes.h

  Abs:  file of defines and typedefs required for tcp socket globals

  Auth: 03-dec-2004, Diane Fairley (DFAIRLEY): created
  Rev:  02 feb 2005, Debbie Rogind

-----------------------------------------------------------------------------*/

#include "copyright_SLAC.h"	/* SLAC copyright comments */
 
/*-----------------------------------------------------------------------------
 
  Mod:  (newest to oldest)  
        DD-MMM-YYYY, My Name:
           Changed such and such to so and so. etc. etc.
        DD-MMM-YYYY, Your Name:
           More changes ... The ordering of the revision history 
           should be such that the NEWEST changes are at the HEAD of
           the list.
 
=============================================================================*/

#ifndef tcpTypes_H
#define tcpTypes_H

#include <osiSock.h>        /* SOCKET */

/*
 * TCP port numbers on the micro and proxy to which each of host based
 * SLCMSGSVC and DBEX and DBEXcontrol and TIMEX will connect.
 *
 * NOTE: THESE PORT VALUES ARE SHARED WITH THE SLC MICROS AND VMS CONTROL SYSTEM
 *       DO NOT CHANGE UNLESS CHANGED IN THE MICROS AND VMS SYSTEM
 */
#define PROXIES_HOST_MSG_PORT  6060   /* Port on proxy for all Host messages */
#define MICROS_HOST_MSG_PORT   6060   /* Port on micro for Host SLCMSGSVC messages */
#define MICROS_HOST_DBEX_PORT  6070   /* Port on micro for Host DBEX messages */
#define MICROS_HOST_TIMEX_PORT 6080   /* Port on micro for Host TIMEX messages */
#define MICROS_HOST_CTL_PORT   6090   /* Port on micro for Host Control messages */

/*
 * Function codes placed in the forward header telling the proxy server
 * what to do with the message.  One example is the PX_KEEPALIVE_FUNC.
 * This implements an application level ping to the proxy server.
 *
 * NOTE: THESE PORT VALUES ARE SHARED WITH THE SLC MICROS AND VMS CONTROL SYSTEM
 *       DO NOT CHANGE UNLESS CHANGED IN THE MICROS AND VMS SYSTEM
 */

#define PX_FORWARD_FUNC   1         /* Forward using IP/PORT values           */
#define PX_KEEPALIVE_FUNC 2         /* Ask proxy server to return an echo     */
#define PX_REGISTER_FUNC 3          /* Register this name with the proxy      */
#define PX_FORWARD_ALIAS_FUNC 4     /* Forward using the alias name           */
#define PX_REGISTER_PORT_FUNC 5     /* Register this port with the proxy      */
#define PX_DISABLE_CONN_FUNC  6     /* Disable all connections sans this one  */
#define PX_ENABLE_CONN_FUNC   7     /* Enable data flow on all connections    */
#define PX_FORWARD_RECEIPT_FUNC 8   /* Forward this receipt to the micro      */

/*
 * The top order nibble of the low order byte of command (fwd_cmd) to the 
 * proxy may contain one of these values (usually masked in).
 * NOTE: THESE VALUES ARE SHARED WITH THE SLC MICROS AND VMS CONTROL SYSTEM
 *       DO NOT CHANGE UNLESS CHANGED IN THE MICROS AND VMS SYSTEM
 */

#define PX_REQUEST_PROXY_RECEIPT  0x40  /* Proxy server returns receipt Bit */
#define PX_REQUEST_APP_RECEIPT    0x80  /* Application returns receipt Bit  */

/*
 * The forward header's crc field generally contains a constant 0x55, so we
 * give that value a name here.
 * NOTE: THESE PORT VALUES ARE SHARED WITH THE SLC MICROS AND VMS CONTROL SYSTEM
 *       DO NOT CHANGE UNLESS CHANGED IN THE MICROS AND VMS SYSTEM
 */
 
#define PX_FWDHDR_CRC_CONSTVAL    0x55


#define MAX_SOCKETS     5   /* max number of sockets created */

/* the sdTmx and sdErr sockets are not needed now - future? */
typedef enum {              /* enumeration of sockets */
  sdMsgSrvc, sdDbex, sdCtl, sdTmx, sdErr, 
  sdEnd /*do not add after here */
} slcSocket_te;

/* this is a structure in case we need to add more fields in the future */
typedef struct {            /* global struct of socket info */
  SOCKET  sd;               /* a socket descriptor */
} slcSocket_ts;


#endif /* guard */
